package com.accenture.eVote.adminms.controller;

import com.accenture.eVote.adminms.dto.*;
import com.accenture.eVote.adminms.model.request.ChiusuraVotoRequest;
import com.accenture.eVote.adminms.model.request.SielRequest;
import com.accenture.eVote.adminms.model.request.TempoElettoraleRequest;
import com.accenture.eVote.adminms.model.response.ChiusuraVotoResponse;
import com.accenture.eVote.adminms.service.ElezioniService;
import jakarta.validation.Valid;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.Date;
import java.util.List;


@RestController
@RequestMapping("/protected/admin/elezioni")
public class ElezioniController {

    @Autowired
    private ElezioniService elezioniService;

    @Autowired
    private RestTemplate restTemplate;

    @Value("${url.urnams.votobc}")
    private String urlUrnaVotoBc;

    @SneakyThrows
    @GetMapping("/date-attive-siel")
    public ResponseEntity<List<DateAttiveDTO>> getDateAttiveSiel() {
        return ResponseEntity.ok().body(elezioniService.getDateAttiveSiel());
    }

    @GetMapping("/date-attive-evote/{id}")
    public ResponseEntity<List<ElezioneEvoteDTO>> getDateAttive(@PathVariable(value="id", required = true) Long idDataElezione) {
        return ResponseEntity.ok().body(elezioniService.getDateAttiveEvote(idDataElezione));
    }

    @GetMapping("/elezione-attiva")
    public ResponseEntity<ElezioneAttivaDTO> getElezioneAttiva() {
        return ResponseEntity.ok().body(elezioniService.getElezioneAttiva());
    }

    @PostMapping("/date-elezioni")
    public ResponseEntity<DateElezioniDTO> setDateElezioni(@RequestBody @Valid List<DateAttiveDTO> dateAttive) throws SQLException {
        return ResponseEntity.ok().body(elezioniService.setDateElezioni(dateAttive).get(0));
    }

    @GetMapping("/elettori-digitali/{id}")
    public ResponseEntity<Long> getElettoriDigitali(@PathVariable(value="id", required = true) Long idDataElezione) {
        return ResponseEntity.ok().body(elezioniService.getElettoriDigitali(idDataElezione));
    }

    @PutMapping("/tempo-elettorale/date")
    public ResponseEntity<Object> setConfTemporaleFine(@RequestBody @Valid TempoElettoraleRequest tempoElettoraleRequest) {

        ZoneId zonaOrariaItalia = ZoneId.of("Europe/Rome");
        ZonedDateTime orarioCorrenteItalia = ZonedDateTime.now(zonaOrariaItalia).withNano(0);
        Timestamp timestamp = Timestamp.from(orarioCorrenteItalia.toInstant().minusSeconds(60));

        // Controllo parametri di ingresso idElezione e tipoFinestra
        if(tempoElettoraleRequest.getFinestraElezioneDTO().getIdDataElezione() == null && tempoElettoraleRequest.getFinestraElezioneDTO().getTipoFinestraTemporale() == null) {
            return ResponseEntity.status(887).body("L'id elezione e il tipo finestra non possono essere null");
        }

        // Controllo date in ingresso
        if (tempoElettoraleRequest.isFinestraAttiva()) {
            if (tempoElettoraleRequest.getDataFine().before(timestamp)
                    || tempoElettoraleRequest.getDataFine().before(tempoElettoraleRequest.getDataInizio())) {
                return ResponseEntity.status(888).body("Le date selezionate non sono valide");
            }
        }
        else {
            if ((tempoElettoraleRequest.getDataFine().before(timestamp) ||
                    tempoElettoraleRequest.getDataInizio().before(timestamp))
                    || tempoElettoraleRequest.getDataFine().before(tempoElettoraleRequest.getDataInizio())) {
                return ResponseEntity.status(888).body("Le date selezionate non sono valide");
            }
        }

        // Controllo flag preleva da Siel
        if(elezioniService.isPrelevaFlagAttivo(tempoElettoraleRequest.getFinestraElezioneDTO().getIdDataElezione()).equals(Boolean.FALSE) &&
                tempoElettoraleRequest.getFlagTastoAttivaImmediato().equals(Boolean.TRUE)) {
            return ResponseEntity.status(889).body("Il preleva da Siel non è stato eseguito");
        }

        return ResponseEntity.ok().body(elezioniService.setConfTempoElettorale(tempoElettoraleRequest));
    }

    @GetMapping("/tempo-elettorale")
    public ResponseEntity<List<ConfTempoElettoraleDTO>> getConfTempoElettorale (FinestraElezioneDTO finestraElezioneDTO) {
        if(finestraElezioneDTO.getIdDataElezione() == null || finestraElezioneDTO.getTipoFinestraTemporale() == null){
            throw new IllegalArgumentException("L'id non può essere null");
        }
        return ResponseEntity.ok().body(elezioniService.getConfTempoElettorale(finestraElezioneDTO));
    }

    @GetMapping("/guida-voto/{id}")
    public ResponseEntity<GuidaVotoDTO> getGuidaVoto (@PathVariable(value="id", required = true )Long idGuidaVoto){
        if(idGuidaVoto == null) {
            throw new IllegalArgumentException("L'id guida voto non può essere null");
        }
        return ResponseEntity.ok().body(elezioniService.getGuidaVoto(idGuidaVoto));
    }

    @PostMapping("/guida-voto")
    public ResponseEntity<GuidaVotoDTO> setGuidaVoto (@RequestBody @Valid GuidaVotoDTO guidaVotoDTO){
        return ResponseEntity.ok().body(elezioniService.setGuidaVoto(guidaVotoDTO));
    }

    @PostMapping("/preleva-dati-siel")
    public void prelevaDatiSiel (@RequestBody @Valid SielDTO sielDTO) {
        // La data delle elezioni deve essere in formato testuale (DD/MM/YYY)
        elezioniService.prelevaDatiSiel(sielDTO.getDataElezione());
    }

    @GetMapping("/invio-dati-SIEL/{id}")
    public ResponseEntity<Object>  getScrutinioAggregato (@PathVariable(value="id", required = true )Long tipologiaElezione){
        if(tipologiaElezione == null) {
            throw new IllegalArgumentException("La tipologia elezione non può essere null");
        }
        Long timer = elezioniService.getScrutinioAggregato(tipologiaElezione);

        if (timer != null) {
            return ResponseEntity.ok().body(timer);
        } else {
            String errorMessage = "Il parametro aggregazione non 'e configurato.";
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(errorMessage);
        }
    }

    @PostMapping("/chiusura-voto")
    public ResponseEntity<ChiusuraVotoResponse> setChiusuraVoto(@RequestBody @Valid ChiusuraVotoRequest chiusuraVotoRequest) throws SQLException {
        elezioniService.setChiusuraVoto(chiusuraVotoRequest);

        List<String> listGeoId = elezioniService.setScrutinio(chiusuraVotoRequest.getIdDataElezione());
        ResponseUrna respUrna = restTemplate.postForEntity(urlUrnaVotoBc, listGeoId, ResponseUrna.class).getBody();
        elezioniService.setVoti(respUrna, chiusuraVotoRequest.getIdDataElezione());

        return ResponseEntity.ok().body(new ChiusuraVotoResponse("Elezione chiusa"));
    }

    @PostMapping("/trasferisci-dati-siel")
    public void trasferisciDatiSiel (@RequestBody @Valid SielRequest sielRequest)  {
        if(sielRequest.getInDate() == null) {
            throw new IllegalArgumentException("La data non può essere null");
        }
        //Crea un oggetto SimpleDateFormat per il formato di data desiderato (DD/MM/AAAA)
        SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy");

        //Imposta la modalità "strict" per il parsing, in modo che il formato sia rigidamente rispettato (es: 12/10/2017 --> valido | 35/24/2017 --> non valido)
        dateFormat.setLenient(false);

        try {
            //Cerca di fare il parsing della stringa dell'utente in un oggetto Date
            Date date = dateFormat.parse(sielRequest.getInDate());
            elezioniService.trasferisciDatiSiel(sielRequest);

        } catch (ParseException e) {
            //Se il parsing solleva un'eccezione, la stringa non è una data valida
            throw new IllegalArgumentException("Il formato della data non è valido");
        }
    }

    @GetMapping("/preleva-eseguito/{id}")
    public ResponseEntity<Boolean> isPrelevaEseguito (@PathVariable(value="id", required = true) Long idDataElezione) {
        return ResponseEntity.ok().body(elezioniService.isPrelevaFlagFaseVoto(idDataElezione));
    }

    @GetMapping("/elettori-digitali-cda")
    public ResponseEntity<Integer> getElettoriDigitaliCda(Long idDataElezione, String prefettura) {

        // Recupero la lista di geoId dalla prefettura
        List<String> listGeoId = elezioniService.findTerritoriByPrefettura(idDataElezione, prefettura);

        return ResponseEntity.ok().body(elezioniService.getElettoriDigitaliCDA(listGeoId, idDataElezione));
    }

}