package com.accenture.eVote.adminms.controller;

import com.accenture.eVote.adminms.dto.EnteDTO;
import com.accenture.eVote.adminms.dto.ResacontoDTO;
import com.accenture.eVote.adminms.model.request.DettaglioPreferenzeListeRequest;
import com.accenture.eVote.adminms.model.response.DettaglioPreferenzeListeResponse;
import com.accenture.eVote.adminms.service.ScrutinioService;
import jakarta.validation.Valid;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.sql.SQLException;
import java.util.List;


@RestController
@RequestMapping("/protected/admin/scrutinio")
public class ScrutinioController {
    private final Logger log = LoggerFactory.getLogger(ScrutinioController.class);

    @Autowired
    private ScrutinioService scrutinioService;

    @GetMapping("/scrutinio-aggregato")
    public ResponseEntity<List<EnteDTO>> getScrutinioAggregato(@RequestParam @Valid Long idElezione) throws SQLException {
        log.info("Recuperando il resoconto della elezione...");
        return ResponseEntity.ok(scrutinioService.getScrutinioAggregato(idElezione));
    }

    @GetMapping("/resoconto-scrutinio")
    public ResponseEntity<ResacontoDTO> getResacontoScrutinio(@RequestParam @Valid Long idElezione, @RequestParam @Valid String geoIdEnte) throws SQLException {
        log.info("Recuperando il resoconto dello scrutinio...");
        return ResponseEntity.ok(scrutinioService.getResacontoScrutinio(idElezione, geoIdEnte));
    }

    @GetMapping("/dettaglio-preferenze-liste")
    public ResponseEntity<Object> getDettaglioPreferenzeListe (DettaglioPreferenzeListeRequest dettaglioPreferenzeListeRequest){
        log.info("Recuperando dettaglio preferenze liste...");

        List<DettaglioPreferenzeListeResponse> responseList = scrutinioService.getDettaglioPreferenzeListe(dettaglioPreferenzeListeRequest);

        if(responseList.isEmpty()){
            String errorMessage = "Candidati non disponibili.";
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(errorMessage);
        }else{
            return ResponseEntity.ok(responseList);
        }
    }

    @GetMapping("/scrutinio-aggregato-cda")
    public ResponseEntity<List<EnteDTO>> getScrutinioAggregatoCda(Long idElezione, String prefettura) throws SQLException {
        log.info("Recuperando il resoconto della elezione...");
        return ResponseEntity.ok(scrutinioService.getScrutinioAggregatoCDA(idElezione, prefettura));
    }

}

