package com.accenture.eVote.adminms.service;

import com.accenture.eVote.adminms.dto.*;
import com.accenture.eVote.adminms.entity.*;
import com.accenture.eVote.adminms.entity.id.IdConfigParametri;
import com.accenture.eVote.adminms.mapper.ConfTempoElettoraleMapper;
import com.accenture.eVote.adminms.mapper.DateAttiveMapper;
import com.accenture.eVote.adminms.mapper.DateElezioniMapper;
import com.accenture.eVote.adminms.mapper.GuidaVotoMapper;
import com.accenture.eVote.adminms.model.request.ChiusuraVotoRequest;
import com.accenture.eVote.adminms.model.request.SielRequest;
import com.accenture.eVote.adminms.model.request.TempoElettoraleRequest;
import com.accenture.eVote.adminms.repository.*;
import com.accenture.eVote.adminms.util.Crittografia;
import com.accenture.eVote.adminms.util.ValidatoreVoti;
import com.accenture.eVote.adminms.util.enums.ElettoriEnum;
import com.accenture.eVote.adminms.util.enums.FaseElettoraleEnum;
import com.accenture.eVote.adminms.util.enums.PrelevaDaSielEnum;
import com.accenture.eVote.adminms.util.enums.ParametriEnum;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.sql.*;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class ElezioniService {

    @Autowired
    private DateAttiveRepository dateAttiveRepository;
    @Autowired
    private DateElezioniRepository dateElezioniRepository;
    @Autowired
    private ElezioniRepository elezioniRepository;
    @Autowired
    private ConsensoVotoRepository consensoVotoRepository;
    @Autowired
    private ElettoreRepository elettoreRepository;
    @Autowired
    private ConfTempoElettoraleRepository confTempoElettoraleRepository;
    @Autowired
    private GuidaVotoRepository guidaVotoRepository;
    @Autowired
    private ConfigParametriRepository configParametriRepository;
    @Autowired
    private TerritorioRepository territorioRepository;
    @Autowired
    private ScrutiniRepository scrutiniRepository;
    @Autowired
    private VotiListaRepository votiListaRepository;
    @Autowired
    private VotiCandListaRepository votiCandListaRepository;
    @Autowired
    private VotantiRepository votantiRepository;
    @Autowired
    private ElevoEnteRepository elevoEnteRepository;
    @Autowired
    private DateAttiveMapper dateAttiveMapper;
    @Autowired
    private DateElezioniMapper dateElezioniMapper;
    @Autowired
    private ConfTempoElettoraleMapper confTempoElettoraleMapper;
    @Autowired
    private GuidaVotoMapper guidaVotoMapper;
    @Autowired
    private Crittografia crittografia;
    @Autowired
    private ValidatoreVoti validatoreVoti;
    @Value("${spring.datasource.url}")
    private String url;

    @Value("${spring.datasource.username}")
    private String username;

    @Value("${spring.datasource.password}")
    private String password;

    @SneakyThrows
    public List<DateAttiveDTO> getDateAttiveSiel() throws SQLException {
        //Connessione DB
        Connection connection = DriverManager.getConnection(url, username, password);

        try{
            //Refresh della vista contenente i territori completi associati alle elezioni
            executeStatement(connection, "{call REFRESH_MV_SIEL_DATE_ATTIVE ()}");
        }finally {
            //Chiusura conessione DB
            connection.close();
        }

        return dateAttiveMapper.toDto(dateAttiveRepository.findAll());
    }

    public List<DateElezioniDTO> setDateElezioni(List<DateAttiveDTO> dateAttive) throws SQLException {
        Connection connection = DriverManager.getConnection(url, username, password);

        try{
            String dataElezione = dateAttive.get(0).getDataElezione().toString();
            DateTimeFormatter inputFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            LocalDate date = LocalDate.parse(dataElezione, inputFormatter);
            DateTimeFormatter outputFormatter = DateTimeFormatter.ofPattern("dd/MM/yyyy");
            String outputString = date.format(outputFormatter);

            callPrelevaDaSielProcedure(outputString, "PE", connection);
        }finally {
            connection.close();
        }

        List<DateElezioniDTO> dateElezioniDTO = new ArrayList<>();

        dateAttive.forEach(data -> {
            DateElezioni dateElezioni = new DateElezioni(data.getChiaveDataElezione(), data.getDataElezione());
            dateElezioniDTO.add(dateElezioniMapper.toDto(dateElezioniRepository.save(dateElezioni)));
            //TODO:chiedere cosa fare per il flag
            Elezioni elezioni = new Elezioni(data.getChiaveElezione(), data.getChiaveDataElezione(),
                    data.getIdTipoElezione(), FaseElettoraleEnum.INIZIALIZZAZIONE.getValore(), PrelevaDaSielEnum.NON_ESEGUITO.getFlag());
            elezioniRepository.save(elezioni);
        });

        return dateElezioniDTO;
    }

    public Long getElettoriDigitali(Long idDataElezione) {
        return elettoreRepository.countElettori(idDataElezione);
    }

    public List<ConfTempoElettoraleDTO> setConfTempoElettorale(TempoElettoraleRequest tempoElettoraleRequest) {

        List<Long> listIdElezione = elezioniRepository.findIdElezioneByIdDataElezione(tempoElettoraleRequest.getFinestraElezioneDTO().getIdDataElezione());
        List<ConfTempoElettorale> listConfTemp = new ArrayList<>();

        listIdElezione.forEach(idElezione -> {
            Optional<ConfTempoElettorale> opt = confTempoElettoraleRepository.findById(
                    new IdConfTempoElettorale(idElezione, tempoElettoraleRequest.getFinestraElezioneDTO().getTipoFinestraTemporale()));
            opt.ifPresent(listConfTemp::add);
        });

        List<ConfTempoElettorale> listConfTemporale = new ArrayList<>();

        if (listConfTemp.isEmpty()) {
            listIdElezione.forEach(idElezione -> {
                ConfTempoElettorale confTempoElettorale = new ConfTempoElettorale();
                confTempoElettorale.setFinestraElezione(tempoElettoraleRequest);
                confTempoElettorale.setElezione(idElezione);
                confTempoElettoraleRepository.save(confTempoElettorale);
                listConfTemporale.add(confTempoElettorale);
            });
            return confTempoElettoraleMapper.toDto(listConfTemporale);
        } else {
            List<ConfTempoElettoraleDTO> listConfTemporaleDTO = new ArrayList<>();
            listConfTemp.forEach(confTempoElettorale -> {
                confTempoElettorale.setFinestraElezione(tempoElettoraleRequest);
                ConfTempoElettorale confTemp = confTempoElettoraleRepository.save(confTempoElettorale);

                Optional<Elezioni> elezione = elezioniRepository.findById(confTempoElettorale.getElezione());
                if (elezione.isPresent() && tempoElettoraleRequest.getFlagTastoAttivaImmediato().equals(Boolean.TRUE)) {
                    if (tempoElettoraleRequest.getFinestraElezioneDTO().getTipoFinestraTemporale().equals(ElettoriEnum.FINESTRA_CONSENSO.getValore())) {
                        elezione.get().setCodFaseElettorale(FaseElettoraleEnum.ESPRESSIONE_CONSENSO.getValore());
                    } else if (tempoElettoraleRequest.getFinestraElezioneDTO().getTipoFinestraTemporale().equals(ElettoriEnum.FINESTRA_VOTO.getValore())) {
                        elezione.get().setCodFaseElettorale(FaseElettoraleEnum.VOTO.getValore());
                    }
                    elezioniRepository.save(elezione.get());
                }

                ConfTempoElettoraleDTO confTempDTO = confTempoElettoraleMapper.toDto(confTemp);
                listConfTemporaleDTO.add(confTempDTO);
            });
            return listConfTemporaleDTO;
        }
    }

    public List<ConfTempoElettoraleDTO> getConfTempoElettorale(FinestraElezioneDTO finestraElezioneDTO) {
        List<ConfTempoElettorale> confTempoElettorale =
                confTempoElettoraleRepository.findByFinestraDataElezione(finestraElezioneDTO.getIdDataElezione(), finestraElezioneDTO.getTipoFinestraTemporale());
        return confTempoElettoraleMapper.toDto(confTempoElettorale);
    }

    public GuidaVotoDTO getGuidaVoto(Long idGuidaVoto) {
        Optional<GuidaVoto> guidaVoto = guidaVotoRepository.findById(idGuidaVoto);
        return guidaVotoMapper.toDto(guidaVoto.get());
    }

    public GuidaVotoDTO setGuidaVoto(GuidaVotoDTO guidaVotoDTO) {
        GuidaVoto guidaVoto = guidaVotoMapper.toEntity(guidaVotoDTO);
        guidaVotoRepository.save(guidaVoto);
        return guidaVotoDTO;
    }

    @SneakyThrows
    public void prelevaDatiSiel(String dataElezioni) {
        //Connessione DB
        Connection connection = DriverManager.getConnection(url, username, password);
        CallableStatement callableStatement = connection.prepareCall("{call REFRESH_VISTE_MATERIALIZZATE (?)}");
        try{
            //Impostazione del parametro input
            callableStatement.setNull(1, Types.VARCHAR);
            callableStatement.execute();
            callableStatement.close();

            callPrelevaDaSielProcedure(dataElezioni, "PRE", connection);

            callableStatement = connection.prepareCall("{call CREA_BASE_SCHEDE_ELETTORALI (?, ?)}");
            callableStatement.setString(1, dataElezioni);
            callableStatement.setNull(2, Types.NUMERIC);

            callableStatement.execute();
        }finally {
            callableStatement.close();
            connection.close();
        }

    }

    private static void callPrelevaDaSielProcedure(String dataElezioni, String macrofase, Connection connection) throws SQLException {
        CallableStatement callableStatement = connection.prepareCall("{call PRELEVA_DA_SIEL (?, ?, ?, ?)}");
        try{
            //Impostazione del parametro input
            callableStatement.setString(1, dataElezioni);
            callableStatement.setString(2, macrofase);
            callableStatement.setNull(3, Types.VARCHAR);
            callableStatement.setLong(4, 1);
            callableStatement.execute();
        }finally {
            callableStatement.close();
        }

    }


    private static void executeStatement(Connection connection, String storedProcedure) throws SQLException {
        CallableStatement callableStatement = connection.prepareCall(storedProcedure);
        try{
            callableStatement.execute();
        }finally {
            callableStatement.close();
        }
    }

    public ElezioneAttivaDTO getElezioneAttiva() {
        List<ElezioneAttivaDTO> elezioniAttive = elezioniRepository.getElezioneAttiva(FaseElettoraleEnum.ELIMINATA.getValore());
        if (elezioniAttive.isEmpty()) {
            return new ElezioneAttivaDTO(null, null, 0L);
        } else {
            return elezioniAttive.get(0);
        }
    }

    public List<ElezioneEvoteDTO> getDateAttiveEvote(Long idDataElezione) {
        List<ElezioneEvoteDTO> elezioneEvoteDTOS = elezioniRepository.getElezioneByIdDataElezione(idDataElezione);
        List<ConfTempoElettorale> listConfTemp = confTempoElettoraleRepository.findByFinestraDataElezione(idDataElezione, ElettoriEnum.FINESTRA_VOTO.getValore());
        if (!listConfTemp.isEmpty()) {
            elezioneEvoteDTOS.forEach(elezione -> {
                elezione.setDataInizio(listConfTemp.get(0).getDataInizio());
                elezione.setDataFine(listConfTemp.get(0).getDataFine());
            });
        }
        return elezioneEvoteDTOS;
    }

    public Long getScrutinioAggregato(Long tipologiaElezione) {
        IdConfigParametri idConfigParametri = new IdConfigParametri(ParametriEnum.MASSIMO_LIVELLO_AGGREGAZIONE.getValore(), tipologiaElezione);
        Optional<ConfigParametri> configParametri = configParametriRepository.findById(idConfigParametri);
        return configParametri.map(ConfigParametri::getValoreParametro).orElse(null);
    }

    public List<String> setChiusuraVoto(ChiusuraVotoRequest chiusuraVotoRequest) throws SQLException {

        // recupero delle'elezione
        List<Elezioni> elezioni = elezioniRepository.findByIdDataElezione(chiusuraVotoRequest.getIdDataElezione());

        // recupero tutti i record di finestre temporale per ogni elezione
        List<ConfTempoElettorale> listConfTemp = new ArrayList<>();
        elezioni.forEach(elezione -> {
            listConfTemp.add(confTempoElettoraleRepository.findByElezioneAndTipoFinestraTemporale(elezione.getIdElezione(), ElettoriEnum.FINESTRA_CONSENSO.getValore()));
            listConfTemp.add(confTempoElettoraleRepository.findByElezioneAndTipoFinestraTemporale(elezione.getIdElezione(), ElettoriEnum.FINESTRA_VOTO.getValore()));
        });

        ZoneId zonaOrariaItalia = ZoneId.of("Europe/Rome");
        ZonedDateTime orarioCorrenteItalia = ZonedDateTime.now(zonaOrariaItalia).withNano(0);
        Timestamp timestamp = Timestamp.from(orarioCorrenteItalia.toInstant());

        // Se la data fine finestra è maggiore alla data chiusura elezione, si chiude la finestra
        listConfTemp.forEach(finestra -> {
            if (finestra.getDataFine().after(timestamp)) {
                finestra.setDataFine(timestamp);
            }
        });

        // Setting fase elezione chiuso
        elezioni.forEach(e -> e.setCodFaseElettorale(FaseElettoraleEnum.CHIUSURA_ELEZIONE.getValore())); // CHIUSURA VOTO in tabella CT_TIPFE_TIPOFASEELETTORALE

        // salvataggio chiusura elezione
        elezioniRepository.saveAll(elezioni);

        //Connessione DB
        Connection connection = DriverManager.getConnection(url, username, password);

        try{
            DateTimeFormatter inputFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            Optional<DateElezioni> dataElezioni = dateElezioniRepository.findById(chiusuraVotoRequest.getIdDataElezione());
            if (dataElezioni.isPresent()) {
                LocalDate date = LocalDate.parse(dataElezioni.get().getDataElezione().toString(), inputFormatter);
                DateTimeFormatter outputFormatter = DateTimeFormatter.ofPattern("dd/MM/yyyy");
                String outputString = date.format(outputFormatter);

                callPrelevaDaSielProcedure(outputString, "ELE", connection);
            }
        }finally {
            //Chiusura conessione DB
            connection.close();
        }

        List<Long> listIdElezione = new ArrayList<>();
        elezioni.forEach(elezione -> {
            listIdElezione.add(elezione.getIdElezione());
        });

        return getGeoIdList(listIdElezione);
    }

    public List<String> setScrutinio(Long idDataElezione) {
        // recupero delle'elezione
        List<Elezioni> elezioni = elezioniRepository.findByIdDataElezione(idDataElezione);

        List<Long> listIdElezione = new ArrayList<>();
        elezioni.forEach(elezione -> {
            listIdElezione.add(elezione.getIdElezione());
        });
        return getGeoIdList(listIdElezione);
    }

    private List<String> getGeoIdList(List<Long> listElezioni) {
        return territorioRepository.findGeoIdByIdElezione(listElezioni);
    }

    public void setVoti(ResponseUrna respUrna, Long idDataElezione) {

        ObjectMapper objectMapper = new ObjectMapper();

        // Recupero voti criptati, decriptaggio e pulizia dato
        List<VotoBcDTO> votoBcDTOS = respUrna.getVotoBcList();
        List<String> listDecodVoti = new ArrayList<>();
        votoBcDTOS.forEach(voto -> {
            try {
                voto.setVoto(voto.getVoto().replaceAll("\"", ""));
                listDecodVoti.add(crittografia.decodificaVoto(voto.getVoto()));
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        });

        // Mapping delle stringhe json negli oggetti Scheda
        List<SchedeVotoDTO> schedeVotoDTOList = listDecodVoti.stream()
                .map(jsonString -> {
                    try {
                        return objectMapper.readValue(jsonString, new TypeReference<List<SchedeVotoDTO>>() {
                        });
                    } catch (JsonProcessingException e) {
                        e.printStackTrace();
                        throw new RuntimeException(e);
                    }
                })
                .flatMap(List::stream)
                .collect(Collectors.toList());
        // todo recuperare configurazione voto per verifica alternanza genere in questo caso teniamo costante true

        Boolean alternanzaGenere = Boolean.TRUE;

        if (alternanzaGenere){
            schedeVotoDTOList = validatoreVoti.validazioneAlternanzaGenere(schedeVotoDTOList);
        }

        // Mapping di voto
        // Divido le schede per territorio
        Map<Long, List<SchedeVotoDTO>> schedePerTerritorio = schedeVotoDTOList.stream()
                .collect(Collectors.groupingBy(schedeVotoDTO -> schedeVotoDTO.getLista().getTerritorio()));

        // recupero lo scrutunio (CL_SCRUT_SCRUTINI) attraverso territorio (non padre)
        List<Scrutini> scrutini = schedeVotoDTOList.stream()
                .map(scheda -> scrutiniRepository.findByFkTerpa(scheda.getLista().getTerritorio()))
                .distinct()
                .collect(Collectors.toList());
        // TODO passaggio a DTO per scrutinio
        scrutini.stream()
                .map(scrutinio -> {
                    // Set schede bianche
                    scrutinio.setNumeroSchedeBianche(
                            // recupero le schede bianche per territorio
                            schedePerTerritorio.get(scrutinio.getFkTerpa()).stream()
                                    .filter(scheda -> scheda.getBianca().equals(true)) // conto le schede bianche per quel territorio
                                    .count());
                    // Set votanti maschi
                    scrutinio.setNumeroVotantiM(0L); // Costante a 0
                    // Set votanti femmine
                    scrutinio.setNumeroVotantiF(
                            schedePerTerritorio.get(scrutinio.getFkTerpa()).stream()
                                    .filter(scheda -> scheda.getVotata().equals(true)) // conto le schede votate (che includono anche le bianche)
                                    .count());
                    // Set sezioni pervenute candidato
                    scrutinio.setSezioniPervenuteCandidato(1L); // Costante a 1
                    // Set sezioni pervenute consiglio
                    scrutinio.setSezioniPervenuteConsiglio(1L); // Costante a 1
                    return scrutinio;
                }).collect(Collectors.toList());


        // recupero i votanti (CL_VOTAN_VOTANTI) attraverso territorio (non padre)
        List<Votanti> votanti = schedeVotoDTOList.stream()
                .map(scheda -> votantiRepository.findByIdTerpa(scheda.getLista().getTerritorio()))
                .distinct()
                .collect(Collectors.toList());
        // TODO passaggio a DTO per votanti
        votanti.stream()
                .map(votante -> {
                    votante.setNumeroVotantiF(scrutini.stream()
                            .filter(scrutinio -> scrutinio.getFkTerpa().equals(votante.getIdTerpa()))
                            .map(Scrutini::getNumeroVotantiF) // Setto il numero di votanti F di scrutinio
                            .collect(Collectors.toList())
                            .get(0));
                    votante.setNumeroVotantiM(0L); // Costante 0
                    votante.setVotantiTotali(votante.getNumeroVotantiF()); // Setto lo stesso valore del numeroVotantiF
                    return votante;
                })
                .collect(Collectors.toList());


        // Setto lo scrutinio su ogni scheda
        schedeVotoDTOList.stream()
                .map(scheda -> {
                    Optional<Long> idScrutinio = scrutini.stream()
                            .filter(scrutinio -> scrutinio.getFkTerpa().equals(scheda.getLista().getTerritorio()))
                            .map(Scrutini::getSeqScrutinio)
                            .findFirst();
                    scheda.getLista().setIdScrutinio(idScrutinio.get());
                    return scheda;
                })
                .collect(Collectors.toList());

        // Calcolo dei voti di lista
        // Mappa con <idLista, numero voti> iterazione

        Map<ListaSchedaDTO, Long> votiListe = schedeVotoDTOList.stream()
                .filter(scheda -> !scheda.getLista().getIdlista().equals(0L))
                .map(SchedeVotoDTO::getLista)
                .collect(Collectors.groupingBy(
                        lista -> new ListaSchedaDTO(lista),
                        Collectors.counting()));

        // Calcolo dei voti dei candidati di lista
        // Mappa con < Chiave composta (idLista&candidado) , numero voti>

        // divido tutti i voti su singolo candidato
        List<ListaSchedaPerCandidatiDTO> listaSchedaPerCandidatiDTO = schedeVotoDTOList.stream()
                .map(SchedeVotoDTO::getLista)
                .flatMap(lista ->
                        lista.getListaCandidati().stream()
                                .map(candidato -> new ListaSchedaPerCandidatiDTO(lista, candidato)))
                .collect(Collectors.toList());

        // Raggruppo e conto i voti per singolo candidato

        Map<ListaSchedaPerCandidatiDTO, Long> votiCandidati = listaSchedaPerCandidatiDTO.stream()
                .collect(Collectors.groupingBy(
                        lista -> new ListaSchedaPerCandidatiDTO(lista),
                        Collectors.counting()));

        // Caricamento dei dati su CL_VOTLI_VOTILISTA
        // Chiave primaria composta da l'ID scrutinio e l'id lista

        List<VotiLista> conteggioVotiLista = votiListe.entrySet().stream()
                .map(conteggioLista -> new VotiLista(
                        conteggioLista.getKey().getIdScrutinio(),
                        conteggioLista.getKey().getIdlista(),
                        conteggioLista.getValue(),
                        -1L))
                .collect(Collectors.toList());

        // Caricamento dei voti su CL_VOTCL_VOTICANDLISTA
        // Chiave primaria composta da scrutinio -> lista -> candidato

        List<VotiCandLista> conteggioVotiCandidato = votiCandidati.entrySet().stream()
                .map(conteggioCandidato -> new VotiCandLista(
                        conteggioCandidato.getKey().getIdScrutinio(),
                        conteggioCandidato.getKey().getIdlista(),
                        conteggioCandidato.getKey().getId(),
                        conteggioCandidato.getValue()))
                .collect(Collectors.toList());

        // Salvataggio voti liste
        votiListaRepository.saveAll(conteggioVotiLista);

        // Salvataggio voti candidati
        votiCandListaRepository.saveAll(conteggioVotiCandidato);

        // Caricamento degli elettori per territorio
        // Richiamo la procedure per ogni id territorio presente negli scrutini

        scrutini.stream()
                .map(Scrutini::getFkTerpa)
                .forEach(idTerritorio -> scrutiniRepository.caricaElettori(idTerritorio));

        // Chiusura dello scrutinio
        // COMPLETA_SCRUT_NON_VOT (ID_DATA_ELEZIONE);

        scrutiniRepository.completaScrutinioNonVot(idDataElezione);

    }

    @SneakyThrows
    public void trasferisciDatiSiel(SielRequest sielRequest) {
        //Connessione DB
        Connection connection =  DriverManager.getConnection(url, username, password);
        CallableStatement callableStatement = connection.prepareCall("{call TRASFERISCI_A_SIEL (?, ?, ?, ?, ?)}");

        try{
            //Impostazione dei parametro input

            callableStatement.setString(1, sielRequest.getInDate());
            if (sielRequest.getInTipoElezione() != null) {
                callableStatement.setLong(2, sielRequest.getInTipoElezione());
            } else {
                callableStatement.setNull(2, java.sql.Types.BIGINT);
            }
            if (sielRequest.getInTipoElezione() != null) {
                callableStatement.setLong(2, sielRequest.getInTipoElezione());
            } else {
                callableStatement.setNull(2, java.sql.Types.BIGINT);
            }

            if (sielRequest.getInAmbito() != null) {
                callableStatement.setLong(3, sielRequest.getInAmbito());
            } else {
                callableStatement.setNull(3, java.sql.Types.BIGINT);
            }

            if (sielRequest.getInTerritorio() != null) {
                callableStatement.setLong(4, sielRequest.getInTerritorio());
            } else {
                callableStatement.setNull(4, java.sql.Types.BIGINT);
            }

            if (sielRequest.getInCodGruppo() != null) {
                callableStatement.setString(5, sielRequest.getInCodGruppo());
            } else {
                callableStatement.setNull(5, java.sql.Types.VARCHAR);
            }


            callableStatement.execute();
        }finally {
            callableStatement.close();
            connection.close();
        }
    }

    public Boolean isPrelevaFlagFaseVoto(Long idDataElezione) {
        return elezioniRepository.findByIdDataElezione(idDataElezione)
                .stream()
                .findFirst()
                .map(elezione -> {
                    Long codFaseElettorale = elezione.getCodFaseElettorale();
                    Character flagPreleva = elezione.getFlagPrelevaDaSiel();

                    // Se il preleva da siel è stato eseguito e la fase elettorale è uguale o maggiore a la fase di voto
                    if (codFaseElettorale >= FaseElettoraleEnum.VOTO.getValore() && flagPreleva.equals(PrelevaDaSielEnum.ESEGUITO.getFlag())) {
                        return Boolean.TRUE;
                    // Se il preleva da siel è stato eseguito e la fase elettorale è menore di la fase di voto
                    } else if (codFaseElettorale < FaseElettoraleEnum.VOTO.getValore() && flagPreleva.equals(PrelevaDaSielEnum.ESEGUITO.getFlag())) {
                        return Boolean.FALSE;
                    } else {
                        return Boolean.FALSE;
                    }
                }).orElse(Boolean.FALSE);
    }

    public Boolean isPrelevaFlagAttivo(Long idDataElezione) {
        return elezioniRepository.findByIdDataElezione(idDataElezione)
                .stream()
                .findFirst()
                .map(elezione -> {
                    Character flagPreleva = elezione.getFlagPrelevaDaSiel();

                    if (flagPreleva.equals(PrelevaDaSielEnum.ESEGUITO.getFlag())) {
                        return Boolean.TRUE;
                    } else {
                        return Boolean.FALSE;
                    }
                }).orElse(Boolean.FALSE);
    }

    public Integer getElettoriDigitaliCDA(List<String> geoId, Long idDataElezione) {
        return Math.toIntExact(elevoEnteRepository.findAllByGeoId(geoId, idDataElezione));
    }

    public List<String> findTerritoriByPrefettura(Long idDataElezione, String prefettura) {
        List<Elezioni> listIdElezioni = elezioniRepository.findByIdDataElezione(idDataElezione);
        List<Long> listId = new ArrayList<>();
        listIdElezioni.forEach(elezione -> listId.add(elezione.getIdElezione()));
        List<Territorio> listTerritorio = territorioRepository.findByIdElezioniAndGeoIdPfGuida(listId, prefettura);
        List<Long> listIdTerritorioPadre = new ArrayList<>();
        listTerritorio.forEach(territorio -> {
            listIdTerritorioPadre.add(territorio.getIdTerritorioPadre());
        });
        return territorioRepository.findGeoIdByListTerritoriPadre(listIdTerritorioPadre);
    }
}
