package com.accenture.eVote.adminms.service;

import com.accenture.eVote.adminms.dto.EnteDTO;
import com.accenture.eVote.adminms.dto.ResacontoDTO;
import com.accenture.eVote.adminms.entity.ScrutiniMV;
import com.accenture.eVote.adminms.entity.Territorio;
import com.accenture.eVote.adminms.entity.VotCandListaMV;
import com.accenture.eVote.adminms.mapper.ScrutiniMVMapper;
import com.accenture.eVote.adminms.model.request.DettaglioPreferenzeListeRequest;
import com.accenture.eVote.adminms.model.response.DettaglioPreferenzeListeResponse;
import com.accenture.eVote.adminms.repository.ScrutiniMVRepository;
import com.accenture.eVote.adminms.repository.TerritorioRepository;
import com.accenture.eVote.adminms.repository.VotClMVRepository;
import com.accenture.eVote.adminms.repository.VotListaMVRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class ScrutinioService {

    @Autowired
    private VotClMVRepository votClMVRepository;

    @Autowired
    private ScrutiniMVMapper scrutiniMapper;

    @Autowired
    private ScrutiniMVRepository scrutiniMVRepository;

    @Autowired
    private VotListaMVRepository votListaMVRepository;

    @Autowired
    private TerritorioRepository territorioRepository;

    @Value("${spring.datasource.url}")
    private String url;

    @Value("${spring.datasource.username}")
    private String username;

    @Value("${spring.datasource.password}")
    private String password;

    private static final String MV_EVOTE_SCRUTINI = "MV_EVOTE_SCRUTINI";
    private static final String MV_EVOTE_VOTCL = "MV_EVOTE_VOTCL";
    private static final String MV_EVOTE_VOTLI = "MV_EVOTE_VOTLI";

    public List<DettaglioPreferenzeListeResponse> getDettaglioPreferenzeListe(DettaglioPreferenzeListeRequest dettaglioPreferenzeListeRequest) {
        List<VotCandListaMV> listaVotCl = votClMVRepository.findByElezioneListaEnte(dettaglioPreferenzeListeRequest.getIdElezione(), dettaglioPreferenzeListeRequest.getIdLista(), dettaglioPreferenzeListeRequest.getCodEnte());// se non sono PK modificare find
        List<DettaglioPreferenzeListeResponse> listaResponse = new ArrayList<DettaglioPreferenzeListeResponse>();
        for (VotCandListaMV votCl: listaVotCl) {
            DettaglioPreferenzeListeResponse dettaglioPreferenzeListeResponse = new DettaglioPreferenzeListeResponse();
            dettaglioPreferenzeListeResponse.setNumCandidato(votCl.getNumCandidato());
            dettaglioPreferenzeListeResponse.setNome(votCl.getNome());
            dettaglioPreferenzeListeResponse.setCognome(votCl.getCognome());
            dettaglioPreferenzeListeResponse.setNumVotiCandidatoLista(votCl.getNumVotiCandidatoLista());
            listaResponse.add(dettaglioPreferenzeListeResponse);
        }
        return listaResponse;
    }

    public List<EnteDTO> getScrutinioAggregato(Long idElezione) throws SQLException {
        refreshVisteMaterializzate();

        //TODO: è possibile che la lista di scrutini sia vuota?
        List<ScrutiniMV> scrutini = scrutiniMVRepository.findByIdElezione(idElezione);
        List<Long> listChiaveEnte = new ArrayList<>();
        scrutini.forEach(s -> {
            listChiaveEnte.add(s.getChiaveEnte());
        });

        List<Map<String, Object>> resultList = votListaMVRepository.findByElezioneAndChiaveEnte(idElezione, listChiaveEnte);
        return getListEnti(resultList);
    }

    public ResacontoDTO getResacontoScrutinio(Long idElezione, String geoIdEnte) throws SQLException {
        refreshVisteMaterializzate();

        //TODO: è possibile che la lista di scrutini sia vuota?
        ScrutiniMV scrutinio = scrutiniMVRepository.findByIdElezioneAndGeoIdEnte(idElezione, geoIdEnte);
        return new ResacontoDTO(scrutinio.getGeoIdEnte(), scrutinio.getNumSchedeBianche(), scrutinio.getElettNumTotali(), scrutinio.getNumVotantiTotali());
    }

    public List<EnteDTO> getScrutinioAggregatoCDA(Long idElezione, String prefettura) throws SQLException {

        refreshVisteMaterializzate();
        List<Territorio> listTerritori = territorioRepository.findByIdElezioneAndGeoIdPfGuida(idElezione, prefettura);

        List<Long> listChiaveEnte = new ArrayList<>();
        listTerritori.forEach(territorio -> {
            listChiaveEnte.add(territorio.getIdTerritorioPadre());
        });

        List<Map<String, Object>> resultList = votListaMVRepository.findByElezioneAndChiaveEnte(idElezione, listChiaveEnte);
        return getListEnti(resultList);
    }

    private void refreshVisteMaterializzate() throws SQLException {
        List<String> procedures = new ArrayList<>();
        procedures.add(MV_EVOTE_SCRUTINI);
        procedures.add(MV_EVOTE_VOTCL);
        procedures.add(MV_EVOTE_VOTLI);

        //Conessione DB
        Connection connection = DriverManager.getConnection(url, username, password);

        CallableStatement callableStatement = connection.prepareCall( "{call REFRESH_VISTE_MATERIALIZZATE_ELE (?)}");
        //Impostazione del parametro input
        procedures.forEach(p -> {
            try {
                callableStatement.setString(1, p);
                callableStatement.execute();

            } catch (SQLException e) {
                throw new RuntimeException(e);
            }
        });
        callableStatement.close();
        //Chiusura conessione DB
        connection.close();
    }

    private static List<EnteDTO> getListEnti(List<Map<String, Object>> resultList) {
        Map<String, List<Object>> groupedResults = new HashMap<>();

        // Group results by descEnte
        for (Map<String, Object> result : resultList) {
            String descEnte = (String) result.get("descEnte");

            // If the entry for descEnte doesn't exist in the map, create it
            groupedResults.computeIfAbsent(descEnte, k -> new ArrayList<>());

            // Add the result map to the list for the descEnte
            groupedResults.get(descEnte).add(result);
        }

        // Create EnteDTO objects
        List<EnteDTO> enteDTOList = new ArrayList<>();
        for (Map.Entry<String, List<Object>> entry : groupedResults.entrySet()) {
            String descEnte = entry.getKey();
            List<Object> votListaDTOList = entry.getValue();

            enteDTOList.add(new EnteDTO(descEnte, votListaDTOList));
        }
        return enteDTOList;
    }

}
