package com.accenture.eVote.apiGateway.config;

import com.accenture.eVote.apiGateway.strategy.impl.AuthStrategyAdmin;
import com.accenture.eVote.apiGateway.strategy.impl.AuthStrategyElettore;
import com.accenture.eVote.apiGateway.strategy.impl.AuthStrategyVoto;
import com.accenture.eVote.apiGateway.util.Autenticazione;
import com.accenture.eVote.apiGateway.util.Crittografia;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.factory.AbstractGatewayFilterFactory;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.function.client.WebClient;

import java.util.Objects;

@Component
public class AuthenticationFilter extends AbstractGatewayFilterFactory<AuthenticationFilter.Config> {
    private final WebClient webClient;
    @Autowired
    private Autenticazione autenticazione;
    @Autowired
    private Crittografia crittografia;
    @Value("${url.oracle.qa.validate}")
    private String urlValidazioneQa;
    @Value("${headers.session.id}")
    private String headerSessionId;

    @Autowired
    public AuthenticationFilter(WebClient webClient) {
        super(Config.class);
        this.webClient = webClient;
    }

    @Override
    public GatewayFilter apply(Config config) {
        return (exchange, chain) -> {


            // Recupero dello user session id
            String userIdCoded = exchange.getRequest().getHeaders().getFirst(headerSessionId);

            // Traduzione e modifica dell'header

            exchange.getRequest().mutate().headers(httpHeaders -> {
                httpHeaders.set(headerSessionId, crittografia.decodificaUserId(userIdCoded));
            });

            switch (Objects.requireNonNull(exchange.getRequest().getHeaders().getFirst("X-App-Key"))) {
                case "admin-ms":
                    autenticazione.setAuthStrategy(new AuthStrategyAdmin(crittografia));
                    break;
                case "voto-ms":
                    autenticazione.setAuthStrategy(new AuthStrategyVoto(crittografia, webClient, urlValidazioneQa));
                    break;
                case "elettori-ms":
                    autenticazione.setAuthStrategy(new AuthStrategyElettore(crittografia));
                    break;
                default:
                    exchange.getResponse().setStatusCode(HttpStatus.UNAUTHORIZED);
                    return exchange.getResponse().setComplete();
            }
            return autenticazione.executeStrategy(exchange, chain);
        };
    }

    public static class Config {
        // Configurazioni del filtro, se necessario
    }
}
