package com.accenture.eVote.apiGateway.strategy.impl;

import com.accenture.eVote.apiGateway.strategy.AuthStrategy;
import com.accenture.eVote.apiGateway.util.Crittografia;
import com.accenture.eVote.apiGateway.util.EndpointCDA;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.http.HttpStatus;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

public class AuthStrategyAdmin implements AuthStrategy {
    private Crittografia crittografia;

    public AuthStrategyAdmin(Crittografia crittografia){
        this.crittografia = crittografia;
    }

    private final String CDA = "X";

    @Override
    public Mono<Void> verifica(ServerWebExchange exchange, GatewayFilterChain chain) {
        // Ottieni il token JWT dall'header della richiesta
        String jwtToken = exchange.getRequest().getHeaders().getFirst("Authorization");

        if (exchange.getRequest().getMethod().toString().equals("OPTIONS")) {
            return chain.filter(exchange);
        }

        try {
            // Decodifica del token
            String jwtTokenDec = crittografia.decodificaTokenAdmin(jwtToken);
            // Verifica la validità del token (controlla anche la scadenza)
            if (crittografia.verificaScadenzaTokenAdmin(jwtTokenDec)) {
                // Verifico l'accesso alla risorsa in base al ruolo
                String ruolo = crittografia.getRuoloFromTokenAdmin(jwtTokenDec);
                // Se il ruolo è CDA verifico che questo sia un endpoint consentito
                if (ruolo.equals(CDA) && !EndpointCDA.verificaEndpoint(exchange.getRequest().getURI().getPath())){
                    // Il token è valido,ma l'endpoint richiamato non è permesso per i CDA blocca la richiesta
                    exchange.getResponse().setStatusCode(HttpStatus.UNAUTHORIZED);
                    return exchange.getResponse().setComplete();

                }else {
                    // Il token è valido e l'endpoint è valido per CDA permetti alla richiesta di passare
                    return chain.filter(exchange);
                }
            } else {
                exchange.getResponse().setStatusCode(HttpStatus.UNAUTHORIZED);
                return exchange.getResponse().setComplete(); // Il token non è valido, blocca la richiesta
            }
        } catch (Exception e) {
            e.printStackTrace();
            exchange.getResponse().setStatusCode(HttpStatus.UNAUTHORIZED);
            return exchange.getResponse().setComplete(); // Il token non è valido, blocca la richiesta
        }

    }
}
