package com.accenture.eVote.apiGateway.strategy.impl;

import com.accenture.eVote.apiGateway.strategy.AuthStrategy;
import com.accenture.eVote.apiGateway.util.Crittografia;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.http.HttpStatus;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;
public class AuthStrategyElettore implements AuthStrategy {
    private Crittografia crittografia;
    public AuthStrategyElettore(Crittografia crittografia){
        this.crittografia = crittografia;
    }
    @Override
    public Mono<Void> verifica(ServerWebExchange exchange, GatewayFilterChain chain) {
        // Ottieni il token JWT dall'header della richiesta
        String jwtToken = exchange.getRequest().getHeaders().getFirst("Authorization");

        if (exchange.getRequest().getMethod().toString().equals("OPTIONS")){
            return chain.filter(exchange);
        }

        try {
            // Decodifica del token
            String jwtTokenDec = crittografia.decodificaTokenData(jwtToken);
            // Verifica la validità del token (controlla anche la scadenza)
            if (crittografia.verificaScadenzaTokenElettori(jwtTokenDec)) {
                return chain.filter(exchange); // Il token è valido, permetti la richiesta di passare
            } else {
                exchange.getResponse().setStatusCode(HttpStatus.UNAUTHORIZED);
                return exchange.getResponse().setComplete(); // Il token non è valido, blocca la richiesta
            }
        } catch (Exception e) {
            exchange.getResponse().setStatusCode(HttpStatus.UNAUTHORIZED);
            return exchange.getResponse().setComplete(); // Il token non è valido, blocca la richiesta
        }
    }
}
