package com.accenture.eVote.apiGateway.strategy.impl;

import com.accenture.eVote.apiGateway.model.request.OracleValidazioneRequest;
import com.accenture.eVote.apiGateway.model.response.OracleValidazioneResponse;
import com.accenture.eVote.apiGateway.strategy.AuthStrategy;
import com.accenture.eVote.apiGateway.util.Crittografia;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.http.HttpStatus;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

public class AuthStrategyVoto implements AuthStrategy {
    private Crittografia crittografia;
    private WebClient webClient;
    private String urlValidazioneQa;

    public AuthStrategyVoto(Crittografia crittografia, WebClient webClient, String urlValidazioneQa) {
        this.crittografia = crittografia;
        this.webClient = webClient;
        this.urlValidazioneQa = urlValidazioneQa;
    }

    @Override
    public Mono<Void> verifica(ServerWebExchange exchange, GatewayFilterChain chain) {
        // Ottieni il token JWT dall'header della richiesta
        String jwtToken = exchange.getRequest().getHeaders().getFirst("Authorization");

        if (exchange.getRequest().getMethod().toString().equals("OPTIONS")) {
            return chain.filter(exchange);
        }
        OracleValidazioneRequest oracleValidazioneRequest = new OracleValidazioneRequest(crittografia.getQaFromTokenVoto(jwtToken));

        if (oracleValidazioneRequest != null && oracleValidazioneRequest.getQualifiedAttribute() != null) {
            try {

                // Recupero attributo qualificato da token

                return webClient.post()
                        .uri(urlValidazioneQa)
                        .bodyValue(oracleValidazioneRequest)
                        .retrieve()
                        .bodyToMono(OracleValidazioneResponse.class)
                        .flatMap(oracleValidazioneResponse -> {
                            if (oracleValidazioneResponse.getVerified()) {
                                return chain.filter(exchange);
                            } else {
                                exchange.getResponse().setStatusCode(HttpStatus.UNAUTHORIZED);
                                return exchange.getResponse().setComplete();
                            }
                        });

            } catch (Exception e) {
                // L'attributo qualificato non è stato verificato correttamente
                e.printStackTrace();
                exchange.getResponse().setStatusCode(HttpStatus.UNAUTHORIZED);
                return exchange.getResponse().setComplete();
            }
        } else {
            exchange.getResponse().setStatusCode(HttpStatus.UNAUTHORIZED);
            return exchange.getResponse().setComplete();
        }
    }
}
