import { Injectable } from '@angular/core';
import { HttpErrorResponse, HttpEvent, HttpHandler, HttpInterceptor, HttpRequest } from '@angular/common/http';
import { Observable, throwError } from 'rxjs';
import { catchError } from "rxjs/operators";
import { Router } from "@angular/router";

import { AuthService } from "../modules/secured/services/auth.service";
import { environment } from 'src/environments/environment';


@Injectable()
export class AuthInterceptor implements HttpInterceptor {

  constructor(private authService: AuthService, private router: Router) {
  }

  intercept(request: HttpRequest<unknown>, next: HttpHandler): Observable<HttpEvent<unknown>> {
    
    const token = this.authService.getLocalStorage("jwtTokenData");

    if (!token) {
      return next.handle(request);
    }
    // Aggiungi i token all'header della richiesta
    let modifiedRequest = request;

    if (token) {
      modifiedRequest = modifiedRequest.clone({
        headers: modifiedRequest.headers.set("Authorization", token)
      });
    }

    modifiedRequest = modifiedRequest.clone({
      headers: modifiedRequest.headers.set("X-App-Key", "admin-ms")
    });


    return next.handle(modifiedRequest).pipe(
      catchError(err => {
        if (err instanceof HttpErrorResponse) {
          if (err.status === 401) {
          }
        }
        
        return throwError(err)
      })
    )
  }
}
