import { HttpErrorResponse } from '@angular/common/http';
import { Component } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { throwError } from 'rxjs';
import { DateAttiveEvoteResponseDto } from '../dto/date-attive-evote-response.dto';
import { ElezioneFinestraTempoElettoraleDto } from '../dto/elezione-finestra-tempo-elettorale.dto';
import { TempoElettoraleRequestDto } from '../dto/tempo-elettorale-request.dto';
import { TempoElettoraleResponseDto } from '../dto/tempo-elettorale-response.dto';
import { GestioneConsensoVotoService } from '../services/gestione-consenso.service';


@Component({
    selector: 'app-gestione-fase-voto',
    templateUrl: './gestione-fase-voto.component.html',
    styleUrls: ['./gestione-fase-voto.component.css']
})
export class GestioneFaseVotoComponent {




    showModal: boolean = false;
    showModal2: boolean = false;
    dataFormatted: string = '';
    compiled: boolean = false;
    schedeDaMostrare: DateAttiveEvoteResponseDto[] = [];
    tempoElettoraleInizioResp?: TempoElettoraleResponseDto;
    tempoElettoraleFineResp?: TempoElettoraleResponseDto;
    getFinestraTemporaleResponse: TempoElettoraleResponseDto[] = [];

    showError: boolean = false;
    immediato: boolean = false;
    booleanCloseDate: boolean = false;
    booleanError: boolean = false;
    pristine: boolean = true;
    canModify: boolean = true;
    flgAttivaImmediato: boolean = false;

    todayDate?: string;
    errorMessage?: string;



    myForm = new FormGroup({
        dataInizio: new FormControl('', Validators.compose([Validators.required])),
        oraInizio: new FormControl('', Validators.compose([Validators.required])),
        dataFine: new FormControl('', Validators.compose([Validators.required])),
        oraFine: new FormControl('', Validators.compose([Validators.required])),

    });
    constructor(private activeRoute: ActivatedRoute, private gestioneConsensoVotoService: GestioneConsensoVotoService, private router: Router) {
    }
    ngOnInit() {

        this.flgAttivaImmediato = false;

        this.todayDate = this.getDataOdierna();
        if (this.myForm.pristine) {
            this.pristine = true;
        }

        this.dataFormatted = this.activeRoute.snapshot.params["dataFormatted"];
        this.schedeDaMostrare = JSON.parse(this.activeRoute.snapshot.params["schedaCaricata"]);
        if (this.schedeDaMostrare !== undefined) {

            // contrllo se sono gia state impostate data inizio/fine
            var idDataElezione = this.schedeDaMostrare[0].idDataElezione !== undefined ? this.schedeDaMostrare[0].idDataElezione : 0;
            var request = new ElezioneFinestraTempoElettoraleDto(idDataElezione, 2);
            // con questa chiamata valorizzo i campi di form
            this.getTempoElettorale(request);
        }

    }

    onSubmit() {
        // Qui puoi gestire la logica quando il form viene inviato

        if (!this.myForm.valid) {

            this.compiled = false;

        }
        if (this.myForm.valid) {

            this.setConsensoVoto();

            this.myForm.controls.dataInizio.disable();
            this.myForm.controls.oraInizio.disable();
            this.myForm.controls.dataFine.disable();
            this.myForm.controls.oraFine.disable();

            this.compiled = true;
        }
    }

    getTempoElettorale(request: ElezioneFinestraTempoElettoraleDto) {
        this.gestioneConsensoVotoService.getTempoElettorale(request).subscribe({
            next: res => {
                this.getFinestraTemporaleResponse = res;
                //qui  la data  arriva timestamp
                var dataInitFormat = new Date();
                if (this.getFinestraTemporaleResponse[0] !== undefined) {
                    if (this.getFinestraTemporaleResponse[0].dataInizio !== undefined) {

                        var dataInizio = this.getFinestraTemporaleResponse[0].dataInizio;
                        var dateStart = new Date(dataInizio !== undefined ? dataInizio : '');
                        var dayStart = dateStart.getDate();
                        const monthStart = dateStart.getMonth() + 1;
                        const yearStart = dateStart.getFullYear();
                        var dataStartforCalendar = yearStart + "-" + this.padZero(monthStart) + "-" + this.padZero(dayStart);// `${yearStart}-${this.padZero(monthStart)}-${dayStart}`;
                        this.myForm.controls['dataInizio'].setValue(dataStartforCalendar);
                        this.myForm.controls.dataInizio.disable();
                        var oraInizio = this.padZero(dateStart.getHours()) + ":" + this.padZero(dateStart.getMinutes());
                        this.myForm.controls['oraInizio'].setValue(oraInizio);
                        this.myForm.controls.oraInizio.disable();

                        if (dateStart < dataInitFormat) {
                            this.booleanCloseDate = true;
                        }
                        this.compiled = true;
                        this.pristine = false;

                    }
                    if (this.getFinestraTemporaleResponse[0].dataFine !== undefined) {
                        var dataFine = this.getFinestraTemporaleResponse[0].dataFine;
                        var dateEnd = new Date(dataFine !== undefined ? dataFine : '');
                        var dayEnd = dateEnd.getDate();
                        const monthEnd = dateEnd.getMonth() + 1;
                        const yearEnd = dateEnd.getFullYear();
                        var dataEndForCalendar = yearEnd + "-" + this.padZero(monthEnd) + "-" + this.padZero(dayEnd);
                        this.myForm.controls['dataFine'].setValue(dataEndForCalendar);
                        this.myForm.controls.dataFine.disable();

                        var oraFine = this.padZero(dateEnd.getHours()) + ":" + this.padZero(dateEnd.getMinutes());
                        this.myForm.controls['oraFine'].setValue(oraFine);
                        this.myForm.controls.oraFine.disable();

                        if (dateEnd <= dataInitFormat) {
                            this.booleanCloseDate = false;
                            this.canModify = false;
                        }
                    }

                }
            },
            error(error: HttpErrorResponse) {
                let errorMessage = 'errore durante il recupero fase voto';
                if (error.error instanceof ErrorEvent) {
                    errorMessage = error.error.message;
                } else {
                    errorMessage = `Error Code: ${error.status}\nMessage: ${error.message}`;
                }
                return throwError(() => {
                    return errorMessage;
                });
            }
        });
    }

    prelevaDatiSiel() {

        if (this.schedeDaMostrare !== undefined) {
            // formato data dd/mm/yyy mentre attuale timestamp

            var date = new Date(this.schedeDaMostrare[0].dataElezione);
            var day = date.getDate();
            const month = date.getMonth() + 1;
            const year = date.getFullYear();
            var dataSChedeForSiel = `${day}/${this.padZero(month)}/${year}`;


            this.gestioneConsensoVotoService.getDatiFromSIEL(dataSChedeForSiel).subscribe({
                next: res => {
                },
                error(error: HttpErrorResponse) {
                    let errorMessage = 'errore durante il recupero dati da siel';
                    if (error.error instanceof ErrorEvent) {
                        errorMessage = error.error.message;
                    } else {
                        errorMessage = `Error Code: ${error.status}\nMessage: ${error.message}`;
                    }
                    return throwError(() => {
                        return errorMessage;
                    });
                }
            });
        }
    }

    setConsensoVoto() {
        var dataInizioFormatted = "";
        var dataFineFormatted = "";

        var idDataElezione = this.schedeDaMostrare[0].idDataElezione !== undefined ? this.schedeDaMostrare[0].idDataElezione : 0;

        var elezioneFinestraReq = new ElezioneFinestraTempoElettoraleDto(idDataElezione, 2);

        
        var dataInizio = this.myForm.controls['dataInizio'].value;
        var oraInizio = this.myForm.controls['oraInizio'].value;

        dataInizioFormatted = dataInizio?.concat('T') || '';
        dataInizioFormatted = dataInizioFormatted.concat(oraInizio?.concat(':00.000+02:00') || '');

        var dataFine = this.myForm.controls['dataFine'].value;
        var oraFine = this.myForm.controls['oraFine'].value;

        dataFineFormatted = dataFine?.concat('T') || '';
        dataFineFormatted = dataFineFormatted.concat(oraFine?.concat(':00.000+02:00') || '');

        if (this.immediato) {

            var dataInizioDaEndpoint = new Date(this.getFinestraTemporaleResponse.length > 0 && this.getFinestraTemporaleResponse[0].dataInizio || '');
            var dataIniziotoFormat = new Date();
            if (!(dataInizioDaEndpoint < dataIniziotoFormat)) {

                this.flgAttivaImmediato = true;

                var dayStart = dataIniziotoFormat.getDate();
                const monthStart = dataIniziotoFormat.getMonth() + 1;
                const yearStart = dataIniziotoFormat.getFullYear();
                const hourStart = dataIniziotoFormat.getHours();
                const minutesStart = dataIniziotoFormat.getMinutes();

                dataInizioFormatted = yearStart + "-" + this.padZero(monthStart) + "-" + this.padZero(dayStart) + "T" + this.padZero(hourStart) + ":" + this.padZero(minutesStart) + ":00.000+02:00";
            }
            var dataFineToFormat = new Date();
            if (dataInizioDaEndpoint < dataIniziotoFormat) {
                this.booleanCloseDate = true;
                var dayEnd = dataFineToFormat.getDate();
                const monthEnd = dataFineToFormat.getMonth() + 1;
                const yearEnd = dataFineToFormat.getFullYear();
                const hourEnd = dataFineToFormat.getHours();
                const minutesEnd = dataFineToFormat.getMinutes();

                dataFineFormatted = yearEnd + "-" + this.padZero(monthEnd) + "-" + this.padZero(dayEnd) + "T" + this.padZero(hourEnd) + ":" + this.padZero(minutesEnd) + ":00.000+02:00";
            }

        }
        var requestInizio = new TempoElettoraleRequestDto(elezioneFinestraReq, dataInizioFormatted, dataFineFormatted, this.booleanCloseDate, this.flgAttivaImmediato);

        this.gestioneConsensoVotoService.setTempoElettorale(requestInizio).subscribe({

            next: res => {
                this.tempoElettoraleInizioResp = res;
                this.booleanError = false;
            },
            error: error => {
                this.booleanError = true;
                this.errorMessage = 'errore durante salvataggio fase voto ';
                if (error.error instanceof ErrorEvent) {
                    this.errorMessage = error.error.message;
                } else {
                    if (error.status === 887) {
                        this.errorMessage = `Id elezione e tipo finestra non possono essere null`
                    } else {
                        this.errorMessage = " Si è verificato un errore con la gestione della finestra di fase di voto.";

                    }
                    if (error.status === 888) {
                        this.errorMessage = `Le date selezionate non sono valide`;
                    } else {
                        this.errorMessage = " Si è verificato un errore con la gestione della finestra di fase di voto.";


                    }
                }
                return throwError(() => {
                    return this.errorMessage;
                });

            }
        });
    }

    apriModifica() {
        this.compiled = false;

        if (this.booleanCloseDate) {
            this.myForm.controls.dataFine.enable();
            this.myForm.controls.oraFine.enable();
        } else if (this.canModify) {
            this.myForm.controls.dataInizio.enable();
            this.myForm.controls.oraInizio.enable();
            this.myForm.controls.dataFine.enable();
            this.myForm.controls.oraFine.enable();
        }
    }

    openModal() {
        this.booleanError = false;
        this.showModal = true;

    }

    openFaseImmediato() {
        this.immediato = true;
        this.showModal = true;

    }

    closeModal() {
        this.showModal = false;
        this.immediato = false;
        // window.location.reload();
    }

    openModal2() {

        this.setConsensoVoto();

        this.showModal = false;
        this.showModal2 = true;

    }

    closeModal2() {
        this.showModal2 = false;
        window.location.reload();
    }

    padZero(num: number | undefined) {
        if (num != undefined) return num < 10 ? '0' + num : num.toString();
        return undefined;
    }

    goToElezioni() {
        this.router.navigate(['app/elezioni'])
    }

    verifyPristine() {
        if (!this.myForm.controls.dataInizio.pristine
            && !this.myForm.controls.oraInizio.pristine
            && !this.myForm.controls.dataFine.pristine
            && !this.myForm.controls.oraFine.pristine) {
            this.pristine = false;
        }
    }

    getDataOdierna(): string {
        var dataOdierna = new Date();
        var dayStart = dataOdierna.getDate();
        const monthStart = dataOdierna.getMonth() + 1;
        const yearStart = dataOdierna.getFullYear();
        return yearStart + "-" + this.padZero(monthStart) + "-" + this.padZero(dayStart);

    }

}


