import { HttpErrorResponse } from '@angular/common/http';
import { Component } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { throwError } from 'rxjs';
import { GestioneGuidaVotoService } from '../services/gestione-guida-voto.service';
import { GuidaVotoDto } from '../dto/guida-voto.dto';
import { DateAttiveEvoteResponseDto } from '../dto/date-attive-evote-response.dto';

@Component({
  selector: 'app-gestione-guida-voto',
  templateUrl: './gestione-guida-voto.component.html',
  styleUrls: ['./gestione-guida-voto.component.css']
})
export class GestioneGuidaVotoComponent {

  gestioneGuidaVotoDto: GuidaVotoDto = new GuidaVotoDto;

  vecchioContenutoGuida?: string = '';

  editModeOnSwitch: boolean = false;

  booleanError: boolean = false;

  booleanSuccess: boolean = false;

  showModal: boolean = false;

  showSecondModal: boolean = false;

  seconModalSuccess: boolean = false;

  dataParsata: string = '';

  schedeInfo?: DateAttiveEvoteResponseDto;

  constructor(private router: ActivatedRoute, private route: Router, private gestioneGuidaVotoService: GestioneGuidaVotoService) {
  }

  ngOnInit() {

    // qui entra oggetto passato da routing selezione-elezioni
    let responseSchedeInfo = JSON.parse(this.router?.snapshot.paramMap.get("schedeInfo") || '""');
    this.schedeInfo = responseSchedeInfo;

    this.dataParsata = this.gestioneGuidaVotoService.formatDate(responseSchedeInfo.dataElezione);

    this.gestioneGuidaVotoService.getGuidaVoto(responseSchedeInfo.idTipologiaElezione).subscribe({
      next: res => {
        this.gestioneGuidaVotoDto = res;
        this.vecchioContenutoGuida = res.contenutoGuidaVoto !== undefined ? res.contenutoGuidaVoto : '';
      },
      error(error: HttpErrorResponse) {
        let errorMessage = 'errore durante il recupero della guida al voto digitale';
        if (error.error instanceof ErrorEvent) {
          errorMessage = error.error.message;
        } else {
          errorMessage = `Error Code: ${error.status}\nMessage: ${error.message}`;
        }
        return throwError(() => {
          return errorMessage;
        });
      }
    });
  }

  saveEdited() {

    this.gestioneGuidaVotoService.setGuidaVoto(this.gestioneGuidaVotoDto).subscribe({
      next: res => {
        this.gestioneGuidaVotoDto = res;
        this.editModeOnSwitch = false;
        this.booleanSuccess = true;
        this.booleanError = false;
        this.vecchioContenutoGuida = this.gestioneGuidaVotoDto?.contenutoGuidaVoto;
      },
      error: error => {
        this.booleanError = true;
        this.booleanSuccess = false;
      }
    });

  }

  confirmReset() {
    this.seconModalSuccess = true;
    this.discardChanges();
  }

  discardChanges() {
    this.gestioneGuidaVotoDto.contenutoGuidaVoto = this.vecchioContenutoGuida;
    this.editModeOnSwitch = false;
  }

  editModeOn() {
    this.editModeOnSwitch = true;
  }

  editModeOff() {
    this.editModeOnSwitch = false;
  }


  back() {
    this.route.navigate(['/app/elezioni']);
  }

  openModal() {
    this.booleanSuccess = false;
    this.showModal = true;
  }

  closeModal() {
    this.booleanSuccess = false;
    this.showModal = false;
  }

  openResetModal() {
    this.seconModalSuccess = false;
    this.showSecondModal = true;
  }

  closeResetModal() {
    this.showSecondModal = false;
  }

}