
import { HttpErrorResponse } from '@angular/common/http';
import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { throwError } from 'rxjs';
import { DateAttiveEvoteResponseDto } from '../dto/date-attive-evote-response.dto';
import { TrasferisciDatiSIELRequestDto } from '../dto/trasferisci-dati-siel-request.dto';
import { AuthService } from '../services/auth.service';
import { ScrutinioService } from '../services/scrutinio.service';

@Component({
  selector: 'app-inviodati',
  templateUrl: './inviodati.component.html',
  styleUrls: ['./inviodati.component.css']
})
export class InviodatiComponent implements OnInit {

  showModalSIEL: boolean = false;
  showSecondModalSIEL: boolean = false;
  booleanErrorSIEL: boolean = false;

  dataFormatted: any;
  schedeDaMostrare: DateAttiveEvoteResponseDto[] = [];
  codGruppo?: string;

  constructor(private activeRoute: ActivatedRoute, private router: Router, private scrutinioService: ScrutinioService, private authService: AuthService) {
  }

  ngOnInit() {
    this.dataFormatted = this.activeRoute.snapshot.params["dataFormatted"];
    this.schedeDaMostrare = JSON.parse(this.activeRoute.snapshot.params["schedaCaricata"]);
    this.codGruppo = this.authService.getLocalStorage("codGruppo") || '';
  }

  inviaDatiSiel() {

    var dataElezione = this.getDataFormatted(this.schedeDaMostrare[0].dataElezione);
    var idTipoElezione = undefined;
    var idTerritorio = undefined;
    // DCSE, vale ‘0’, per le utenze di tipo Corte d’Appello Estero vale ‘PFE’ (PFE030 per Firenze) seguito dal codice geo_id della provincia
    var codGruppo = this.codGruppo


    var request = new TrasferisciDatiSIELRequestDto(dataElezione, codGruppo, idTipoElezione, idTerritorio);

    this.scrutinioService.sendToSIEL(request).subscribe({
      next: res => {
        this.booleanErrorSIEL = false;
      },
      error: error => {
        this.booleanErrorSIEL = true;
        let errorMessage = 'errore durante il trasferimento dati al SIEL';
        if (error.error instanceof ErrorEvent) {
          errorMessage = error.error.message;
        } else {
          errorMessage = `Error Code: ${error.status}\nMessage: ${error.message}`;
        }
        return throwError(() => {
          return errorMessage;
        });
      }
    });

  }

  goToElezioni() {
    this.router.navigate(['app/elezioni'])
  }


  openModalSIEL() {
    this.booleanErrorSIEL = false;
    this.showModalSIEL = true;

  }


  closeModalSIEL() {
    this.showModalSIEL = false;

  }

  openModal2SIEL() {
    this.inviaDatiSiel();
    this.showSecondModalSIEL = true;

  }

  closeModal2SIEL() {
    this.showSecondModalSIEL = false;
    window.location.reload();

  }

  padZero(num: number | undefined) {
    if (num != undefined) return num < 10 ? '0' + num : num.toString();
    return undefined;
  }

  getDataFormatted(date: string): string {
    var dataOdierna = new Date(date);
    var dayStart = dataOdierna.getDate();
    const monthStart = dataOdierna.getMonth() + 1;
    const yearStart = dataOdierna.getFullYear();
    const hourStart = dataOdierna.getHours();
    const minutesStart = dataOdierna.getMinutes();
    return this.padZero(dayStart) + "/" + this.padZero(monthStart) + "/" + yearStart;

  }

}