import { HttpErrorResponse } from '@angular/common/http';
import { Component, Input, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { DateAttiveEvoteResponseDto } from '../dto/date-attive-evote-response.dto';
import { ElezioneFinestraTempoElettoraleDto } from '../dto/elezione-finestra-tempo-elettorale.dto';
import { GestioneConsensoVotoService } from '../services/gestione-consenso.service';

@Component({
  selector: 'app-navtempielettorali',
  templateUrl: './navtempielettorali.component.html',
  styleUrls: ['./navtempielettorali.component.css']
})
export class NavtempielettoraliComponent implements OnInit {

  @Input()
  schedeCaricate: DateAttiveEvoteResponseDto[] = [];

  isGestioneConsenso: boolean = false;
  isFaseVoto: boolean = false;
  isChiusuraElezioni: boolean = false;
  isInvioDati: boolean = false;
  fase: number = 0;
  dataFineConsenso: string = '';
  dataFineVoto: string = '';

  @Input()
  dataFormatted?: string = '';

  constructor(private router: Router, private activeRoute: ActivatedRoute, private gestioneConsenso: GestioneConsensoVotoService) { }

  openGestioneConsenso() {
    this.router.navigate(['app/tempiElettorali/modificaconsensovoto', { dataFormatted: this.dataFormatted, schedaCaricata: this.schedeCaricate }]);

  }


  openGestioneFaseVoto() {
    this.router.navigate(['app/tempiElettorali/gestione-fase-voto', { dataFormatted: this.dataFormatted, schedaCaricata: this.schedeCaricate }]);

  }
  openChiusuraElezioni() {
    this.router.navigate(['app/tempiElettorali/chiusuraelezioni', { dataFormatted: this.dataFormatted, schedaCaricata: this.schedeCaricate }]);

  }
  openInvioDati() {
    this.router.navigate(['app/tempiElettorali/inviodati', { dataFormatted: this.dataFormatted, schedaCaricata: this.schedeCaricate }]);

  }

  ngOnInit() {
    this.schedeCaricate = this.activeRoute.snapshot.params["schedaCaricata"];
    this.isGestioneConsenso = this.router.url.includes("modificaconsenso");
    this.isFaseVoto = this.router.url.includes("gestione-fase");
    this.isChiusuraElezioni = this.router.url.includes("chiusuraelezioni");
    this.isInvioDati = this.router.url.includes("inviodati");
    const schede = JSON.parse(this.activeRoute.snapshot.params["schedaCaricata"]);

    this.fase = schede[0].codFaseElettorale ? schede[0].codFaseElettorale : 0;

    this.initDateFine(schede[0].idDataElezione, 1);
    this.initDateFine(schede[0].idDataElezione, 2);

  }

  initDateFine(idData: number, fase: number) {
    var request = new ElezioneFinestraTempoElettoraleDto(idData, fase);
    this.gestioneConsenso.getTempoElettorale(request).subscribe({
      next: res => {
        if (fase == 1){
          this.dataFineConsenso = res[0]?.dataFine !== undefined ? res[0].dataFine : '0';
        }
        if (fase == 2) {
          this.dataFineVoto = res[0]?.dataFine !== undefined ? res[0].dataFine : '0';
        }
        //qui  la data  arriva timestamp
      },
      error(error: HttpErrorResponse) {
        let errorMessage = 'errore durante il recupero fase voto';
        if (error.error instanceof ErrorEvent) {
          errorMessage = error.error.message;
        } else {
          errorMessage = `Error Code: ${error.status}\nMessage: ${error.message}`;
        }
        return errorMessage;
      }
    });
    
  }

  isTimeConsenso() {
    const currentDateMillis = new Date().getTime();
    const millisecondi = Date.parse(this.dataFineConsenso);
    return millisecondi < currentDateMillis;
  }

  isTimeVoto() {
    const currentDateMillis = new Date().getTime();
    const millisecondi = Date.parse(this.dataFineVoto);
    return millisecondi < currentDateMillis;
  }
}
