
import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { GestioneGuidaVotoService } from '../services/gestione-guida-voto.service';
import { ScrutinioAggregatoResponseDto } from '../dto/scrutinio-aggregato-response.dto'
import { ScrutinioService } from '../services/scrutinio.service'
import { DateAttiveEvoteResponseDto } from '../dto/date-attive-evote-response.dto';
import { HttpErrorResponse } from '@angular/common/http';
import { throwError } from 'rxjs';
import { ListaCandidataDto } from '../dto/lista-candidata.dto';
import { CandidatiListaResponseDto } from '../dto/candidati-lista-response.dto';
import { TrasferisciDatiSIELRequestDto } from '../dto/trasferisci-dati-siel-request.dto';
import { UserRoleEnum } from '../enum/user-role-enum';
import { AuthService } from '../services/auth.service';


@Component({
    selector: 'app-resoconto-elezione',
    templateUrl: './resoconto-elezione.component.html',
    styleUrls: ['./resoconto-elezione.component.css']
})
export class ResocontoElezioneComponent implements OnInit {

    showModalSIEL: boolean = false;
    showSecondModalSIEL: boolean = false;
    booleanErrorSIEL: boolean = false;
    showModalDettPreferenze: boolean = false;

    dataFormatted: any;
    schedeDaMostrare?: DateAttiveEvoteResponseDto;
    listeCandidati?: ScrutinioAggregatoResponseDto[];
    places?: string[];
    dettCandidatiList?: CandidatiListaResponseDto[];
    listaModale?: string;
    showSIELButton: boolean = false;
    candidatiToSend: ListaCandidataDto = new ListaCandidataDto(0, "");

    DCSE = UserRoleEnum.DCSE;
    CDA = UserRoleEnum.CDA;
    userRole?: string;
    codGruppo?: string;

    constructor(private activeRoute: ActivatedRoute, private router: Router, private gestioneGuidaVotoService: GestioneGuidaVotoService, private scrutinioService: ScrutinioService, private authService: AuthService) {
    }

    ngOnInit() {
        this.schedeDaMostrare = JSON.parse(this.activeRoute.snapshot.params["schedeInfo"]);
        this.dataFormatted = this.gestioneGuidaVotoService.formatDate(this.schedeDaMostrare?.dataElezione || '');

        this.userRole = this.authService.getLocalStorage("flagGruppoUtente") || '';
        this.codGruppo = this.authService.getLocalStorage("codGruppo") || '';


        if (this.schedeDaMostrare?.idElezione !== null && this.schedeDaMostrare?.idElezione !== undefined) {

            switch (this.userRole) {
                case UserRoleEnum.DCSE: {
                    this.caricaScrutiniDCSE();
                    break;
                }
                case UserRoleEnum.CDA: {
                    this.caricaScrutiniCDA();
                    break;
                }
            }

        }
    }


    caricaScrutiniDCSE() {
        this.scrutinioService.getScrutinioAggregatoDCSE(this.schedeDaMostrare?.idElezione || 0).subscribe({
            next: res => {
                this.listeCandidati = res;
                this.listeCandidati.forEach(listaCandidati => {// Ordina listVotListaDTO all'interno di ogni listaCandidati
                    listaCandidati.listVotListaDTO?.sort((a, b) => (a.numLista ?? 0) - (b?.numLista ?? 0));

                    listaCandidati.nomeEnte = listaCandidati.nomeEnte.toLocaleLowerCase();

                    this.scrutinioService.getResocontoScrutinio(this.schedeDaMostrare?.idElezione || 0, listaCandidati.listVotListaDTO[0].codEnte || '').subscribe(value => {
                        listaCandidati.resocontoScrutinio = value;
                        listaCandidati.idPerCollapse = listaCandidati.listVotListaDTO[0].codEnte;
                    });

                });

                if (this.listeCandidati.length > 0) {

                    this.showSIELButton = true;
                    this.booleanErrorSIEL = false;
                }
            },
            error: error => {
                let errorMessage = 'errore durante il recupero dello scrutinio DCSE aggregato ';
                if (error.error instanceof ErrorEvent) {
                    errorMessage = error.error.message;
                } else {
                    errorMessage = `Error Code: ${error.status}\nMessage: ${error.message}`;
                }
                return throwError(() => {
                    return errorMessage;
                });
            }
        });
    }

    caricaScrutiniCDA() {

        var prefettura = this.codGruppo;

        this.scrutinioService.getScrutinioAggregatoCDA(this.schedeDaMostrare?.idElezione || 0, prefettura || "" ).subscribe({
            next: res => {
                this.listeCandidati = res;
                // porzione per inserire anche dati per schede 
                this.listeCandidati.forEach(listaCandidati => {
                    listaCandidati.listVotListaDTO?.sort((a, b) => (a.numLista ?? 0) - (b?.numLista ?? 0));

                    this.scrutinioService.getResocontoScrutinio(this.schedeDaMostrare?.idElezione || 0, listaCandidati.listVotListaDTO[0].codEnte || '').subscribe(value => {
                        listaCandidati.resocontoScrutinio = value;
                        listaCandidati.idPerCollapse = listaCandidati.listVotListaDTO[0].codEnte;
                    });
                })
                if (this.listeCandidati.length > 0) {

                    this.showSIELButton = true;
                }
            },
            error: error => {
                let errorMessage = 'errore durante il recupero dello scrutinio CDA aggregato ';
                if (error.error instanceof ErrorEvent) {
                    errorMessage = error.error.message;
                } else {
                    errorMessage = `Error Code: ${error.status}\nMessage: ${error.message}`;
                }
                return throwError(() => {
                    return errorMessage;
                });
            }
        });
    }




    inviaDatiSiel(listaCandidati: ListaCandidataDto) {

        var dataElezione = this.getDataFormatted(this.schedeDaMostrare?.dataElezione || '');
        var idTipoElezione = this.schedeDaMostrare?.idTipologiaElezione;
        var idTerritorio = listaCandidati.idTerritorio;

        // DCSE, vale ‘0’, per le utenze di tipo Corte d’Appello Estero vale ‘PFE’ (PFE030 per Firenze) seguito dal codice geo_id della provincia
        var codGruppo =  this.codGruppo

        var request = new TrasferisciDatiSIELRequestDto(dataElezione, codGruppo, idTipoElezione, idTerritorio);

        this.scrutinioService.sendToSIEL(request).subscribe({
            next: res => {
                    this.booleanErrorSIEL = false;
            },
            error: error => {
                this.booleanErrorSIEL = true;
                let errorMessage = 'errore durante il trasferimento dati al SIEL';
                if (error.error instanceof ErrorEvent) {
                    errorMessage = error.error.message;
                } else {
                    errorMessage = `Error Code: ${error.status}\nMessage: ${error.message}`;
                }
                return throwError(() => {
                    return errorMessage;
                });
            }
        });

    }

    goToElezioni() {
        this.router.navigate(['app/elezioni'])
    }

    goToDettaglio(listaCandidati: ListaCandidataDto) {
        this.scrutinioService.getDettaglioPreferenze(this.schedeDaMostrare?.idElezione || 0, listaCandidati.lista || 0, listaCandidati.codEnte || '').subscribe(res => {
            // utilizzo var di appoggio per riempire dati da mostrare su modale
            this.dettCandidatiList = res;
            this.listaModale = listaCandidati.descLista
            // Imposta la variabile per controllare la visualizzazione della modale
            this.showModalDettPreferenze = true;
        });
    }

    openModalSIEL() {
        this.booleanErrorSIEL = false;
        this.showModalSIEL = true;

    }

    openModalSIELCity(listaCandidati: ScrutinioAggregatoResponseDto) {
        this.candidatiToSend = listaCandidati.listVotListaDTO[0];
        this.booleanErrorSIEL = false;
        this.showModalSIEL = true;

    }


    closeModalSIEL() {
        this.showModalSIEL = false;
        this.showModalSIEL = false;
        this.showSecondModalSIEL = false;
        // window.location.reload();

    }

    openModal2SIEL() {
        this.inviaDatiSiel(this.candidatiToSend);
        this.showSecondModalSIEL = true;

    }

    closeModalDettPreferenze() {
        this.showModalDettPreferenze = false;
    }

    padZero(num: number | undefined) {
        if (num != undefined) return num < 10 ? '0' + num : num.toString();
        return undefined;
    }

    getDataFormatted(date: string): string {
        var dataOdierna = new Date(date);
        var dayStart = dataOdierna.getDate();
        const monthStart = dataOdierna.getMonth() + 1;
        const yearStart = dataOdierna.getFullYear();
        const hourStart = dataOdierna.getHours();
        const minutesStart = dataOdierna.getMinutes();
        return this.padZero(dayStart) + "/" + this.padZero(monthStart) + "/" + yearStart;

    }

}