import { Component, EventEmitter, Output } from '@angular/core';
import { Router } from '@angular/router';
import { DataInfoSchedaDto } from '../dto/data-info-scheda.dto';
import { DateInfoSchedeService } from '../services/date-info-schede.service';
import { ResponseSetDataDto } from '../dto/response-set-data.dto';


@Component({
  selector: 'app-selezione-date-elezioni',
  templateUrl: './selezione-date-elezioni.component.html',
  styleUrls: ['./selezione-date-elezioni.component.css']
})
export class SelezioneDateElezioniComponent {



  constructor(private router: Router, private dateInfoService: DateInfoSchedeService) {
  }

  @Output() switchToConfirmComponent = new EventEmitter<Boolean>();

  selectedDate?: string;

  dates: DataInfoSchedaDto[] = [];

  schedeDaMostrare: DataInfoSchedaDto[] = [];

  responseSetDate?: ResponseSetDataDto[] = [];

  infoScheda?: DataInfoSchedaDto;

  datePicked: boolean = false;

  showModal: boolean = false;

  showSecondModal?: boolean;

  syncSchede: boolean = false;



  ngOnInit() {
    this.loadDateInfoSchede();
  }


  selectDate(selected: DataInfoSchedaDto) {

    if (this.infoScheda === null || this.infoScheda === undefined || selected.dataParsata !== this.infoScheda.dataElezione) {
      this.syncSchede = false;
      this.showSecondModal = undefined;

      this.infoScheda = selected;
      this.selectedDate = selected.dataElezione;

      this.schedeDaMostrare = [];

      this.dateInfoService.getAllDateInfoSchede().subscribe(res => {
        res.forEach(singleRes => {
          singleRes.dataParsata = this.dateInfoService.formatDate(singleRes.dataElezione);
          if (singleRes.dataElezione === selected.dataElezione) {
            this.schedeDaMostrare.push(singleRes);
          }
        });
      this.syncSchede = true;
      });

      this.datePicked = true;
    }
  }

  apriSchede() {
    this.switchToConfirmComponent.emit(true);
    this.router.navigate(['/app/home/selezione-elezione', { schede: JSON.stringify(this.responseSetDate) }]);
    this.closeModal();
  }

  loadDateInfoSchede() {
    this.dateInfoService.getAllDateInfoSchede().subscribe(res => {

      let itemsShowDropdown = new Array();

      res.forEach(singleRes => {

        if (!itemsShowDropdown.includes(singleRes.dataElezione)) {
          itemsShowDropdown.push(singleRes.dataElezione);
          singleRes.dataParsata = this.dateInfoService.formatDate(singleRes.dataElezione);
          this.dates.push(singleRes);
        }
      });
    });
  }

  isSenato(schedaToShow: DataInfoSchedaDto): boolean {
    return schedaToShow?.descrTipoElezione === "Senato Estero"
  }

  isCamera(schedaToShow: DataInfoSchedaDto): boolean {
    return schedaToShow?.descrTipoElezione === "Camera Estero"
  }



  openModal() {
    if (this.showSecondModal !== undefined && !this.showSecondModal) {
      this.showSecondModal = undefined;
    }
    this.showModal = true;
  }

  closeModal() {
    this.showModal = false;
  }

  confermaData() {
    this.schedeDaMostrare.filter(res => res.dataElezione === this.selectedDate);
    this.dateInfoService.setDataElezione(this.schedeDaMostrare).subscribe({
      next: res => {
        this.responseSetDate = res;
        this.showSecondModal = true;
      },
      error: error => {
        this.showSecondModal = false;
      }
    });
  }

  annullaSelezione() {

  }


}