import { HttpErrorResponse } from '@angular/common/http';
import { Component } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { throwError } from 'rxjs';
import { DateAttiveEvoteResponseDto } from '../dto/date-attive-evote-response.dto';
import { UserRoleEnum } from '../enum/user-role-enum';
import { AuthService } from '../services/auth.service';
import { DateInfoSchedeService } from '../services/date-info-schede.service';


@Component({
  selector: 'app-selezione-elezione',
  templateUrl: './selezione-elezione.component.html',
  styleUrls: ['./selezione-elezione.component.css'],
})
export class SelezioneElezioneComponent {

  dateAttiveResponse?: DateAttiveEvoteResponseDto[] = [];

  schedeDaMostrare: DateAttiveEvoteResponseDto[] = [];

  dateFormatted?: string;

  totaleVotantiDigitali: number = 0;

  countVotantiSenato: number = 0;
  countVotantiCamera: number = 0;

  totaleElettoriDigitali: number = 0;

  userRole?: string;
  statoElezione?: number;

  DCSE = UserRoleEnum.DCSE;
  CDA = UserRoleEnum.CDA;

  constructor(private dateInfoService: DateInfoSchedeService, private router: ActivatedRoute, private rout: Router, private authService: AuthService) {
  }

  isSenato(schedaToShow: DateAttiveEvoteResponseDto): boolean {
    return schedaToShow?.tipologiaelezione === "Senato Estero"
  }

  isCamera(schedaToShow: DateAttiveEvoteResponseDto): boolean {
    return schedaToShow?.tipologiaelezione === "Camera Estero"
  }

  isOther(schedaToShow: DateAttiveEvoteResponseDto): boolean {
    return (schedaToShow?.tipologiaelezione !== "Camera Estero" && schedaToShow?.tipologiaelezione !== "Senato Estero")
  }

  ngOnInit() {
    // dcse = M cda = X
    this.userRole = this.authService.getLocalStorage("flagGruppoUtente") || '';
    switch (this.authService.getLocalStorage("flagGruppoUtente")) {
      case UserRoleEnum.DCSE: {
          this.getElettoriEVotantiDigitaliDCSE();
          break;
      }
      case UserRoleEnum.CDA: {
          this.getElettoriEVotantiDigitaliCDA();
          break;
      }
  }
    this.getDateElezioneInContesto();

  }

  getDateElezioneInContesto() {
    let responseSetDate = JSON.parse(this.router?.snapshot.paramMap.get("schede") || '""');


    this.dateInfoService.getDateAttiveEvote(responseSetDate.idDataElezione).subscribe(res => {
      this.dateFormatted = this.dateInfoService.formatDate(res[0].dataElezione)
      this.statoElezione = res[0].codFaseElettorale;
      this.schedeDaMostrare = res;
    });
  }

  getElettoriEVotantiDigitaliDCSE() {
    let responseSetDate = JSON.parse(this.router?.snapshot.paramMap.get("schede") || '""');

    this.dateFormatted = this.dateInfoService.formatDate(responseSetDate.dataElezione)

    this.dateInfoService.getElettoriDigitaliDCSE(responseSetDate.idDataElezione).subscribe({
      next: res => {
        this.totaleElettoriDigitali = res;
      },
      error(error: HttpErrorResponse) {
        let errorMessage = 'errore durante il recupero del numero degli elettori digitali';
        if (error.error instanceof ErrorEvent) {
          errorMessage = error.error.message;
        } else {
          errorMessage = `Error Code: ${error.status}\nMessage: ${error.message}`;
        }
        return throwError(() => {
          return errorMessage;
        });
      }
    }
    );

    this.dateInfoService.getVotantiDigitaliDCSE().subscribe({
      next: res => {
        res.countVotantiCamera === null? 0 : this.countVotantiCamera = res.countVotantiCamera;
        res.countVotantiSenato === null? 0 : this.countVotantiSenato = res.countVotantiSenato;
       
      },
      error(error: HttpErrorResponse) {
        let errorMessage = 'errore durante il recupero del numero dei votanti digitali';
        if (error.error instanceof ErrorEvent) {
          errorMessage = error.error.message;
        } else {
          errorMessage = `Error Code: ${error.status}\nMessage: ${error.message}`;
        }
        return throwError(() => {
          return errorMessage;
        });
      }
    }
    );
  }

  getElettoriEVotantiDigitaliCDA() {
    let responseSetDate = JSON.parse(this.router?.snapshot.paramMap.get("schede") || '""');
    let codGruppo =  this.authService.getLocalStorage("codGruppo") || '""';
    this.dateFormatted = this.dateInfoService.formatDate(responseSetDate.dataElezione)

    this.dateInfoService.getElettoriDigitaliCDA(responseSetDate.idDataElezione, codGruppo).subscribe({
      next: res => {
        this.totaleElettoriDigitali = res;
      },
      error(error: HttpErrorResponse) {
        let errorMessage = 'errore durante il recupero del numero degli elettori digitali';
        if (error.error instanceof ErrorEvent) {
          errorMessage = error.error.message;
        } else {
          errorMessage = `Error Code: ${error.status}\nMessage: ${error.message}`;
        }
        return throwError(() => {
          return errorMessage;
        });
      }
    }
    );

    this.dateInfoService.getVotantiDigitaliCDA(responseSetDate.idDataElezione, codGruppo).subscribe({
      next: res => {
        res.countVotantiCamera === null? 0 : this.countVotantiCamera = res.countVotantiCamera;
        res.countVotantiSenato === null? 0 : this.countVotantiSenato = res.countVotantiSenato;
        
      },
      error(error: HttpErrorResponse) {
        let errorMessage = 'errore durante il recupero del numero dei votanti digitali';
        if (error.error instanceof ErrorEvent) {
          errorMessage = error.error.message;
        } else {
          errorMessage = `Error Code: ${error.status}\nMessage: ${error.message}`;
        }
        return throwError(() => {
          return errorMessage;
        });
      }
    }
    );
  }

  apriGuidaVoto(scheda: DateAttiveEvoteResponseDto) {

    this.rout.navigate(['/app/elezioni/gestione-guida-voto', { schedeInfo: JSON.stringify(scheda) }]);
  }

  apriResoconto(scheda: DateAttiveEvoteResponseDto) {

    this.rout.navigate(['/app/elezioni/resoconto-elezione', { schedeInfo: JSON.stringify(scheda) }]);
  }

  isInTime(schedaToShow: DateAttiveEvoteResponseDto): boolean {
    const currentDateMillis = new Date().getTime();
    const millisecondiFine = Date.parse(schedaToShow.dataFine ? schedaToShow.dataFine : '0');
    const millisecondiInizio = Date.parse(schedaToShow.dataInizio ? schedaToShow.dataInizio : '0');
    return millisecondiFine >= currentDateMillis && millisecondiInizio <= currentDateMillis;
  }
}
