import { HttpClient, HttpErrorResponse, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { CookieService } from 'ngx-cookie-service';
import { Observable, throwError } from 'rxjs';
import { environment } from '../../../../environments/environment';


@Injectable({
  providedIn: 'root'
})
export class AuthService {

  constructor(
    private router: Router,
    private http: HttpClient,
    private cookieService: CookieService
  ) { }

  setLocalStorage(key: string, value: any) {
    localStorage.setItem(key, value);
  }

  getLocalStorage(key: string) {
    return localStorage.getItem(key);
  }

  removeLocalStorage(key: string) {
    localStorage.removeItem(key);
  }

  mockLoginCall(username: string) {
    this.setLocalStorage("JWT", "jwtMock")
    this.setLocalStorage("Username", username)
  }

  doLogout() {
    localStorage.clear();
    this.cookieService.deleteAll();

    window.location.href = environment.logout;
  }

  doLogin(sessionId: string): Observable<any> {

    const url = environment.baseHost + environment.loginUrl;

    return this.http.get<any>(url, {
      headers: new HttpHeaders({
        'SMSESSION': sessionId
      })
    })

  }

}
