import { Injectable } from '@angular/core';
import { DataInfoSchedaDto } from '../dto/data-info-scheda.dto';
import { HttpClient, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs';
import { environment } from '../../../../environments/environment';
import { ResponseSetDataDto } from '../dto/response-set-data.dto';
import { DateAttiveEvoteResponseDto } from '../dto/date-attive-evote-response.dto';




@Injectable({
    providedIn: 'root'
})

export class DateInfoSchedeService {

    constructor(private http: HttpClient) { }


    monthNames = [
        "gennaio", "febbraio", "marzo", "aprile",
        "maggio", "giugno", "luglio", "agosto",
        "settembre", "ottobre", "novembre", "dicembre"];


    getAllDateInfoSchede(): Observable<DataInfoSchedaDto[]> {
        const url = environment.baseHost + environment.getDataAndInfoSchedaPath;
        return this.http.get<DataInfoSchedaDto[]>(url);
    }

    setDataElezione(dataSelected: DataInfoSchedaDto[]): Observable<DataInfoSchedaDto[]> {
        const url = environment.baseHost + environment.setDataElezionePath;
        return this.http.post<DataInfoSchedaDto[]>(url, dataSelected);
    }

    getElezioniAttive(): Observable<ResponseSetDataDto> {
        const url = environment.baseHost + environment.getDateAttivePath;
        return this.http.get<ResponseSetDataDto>(url);
    }

    getElettoriDigitaliDCSE(idDataElezione: string): Observable<number> {
        const url = environment.baseHost + environment.getElettoriDigitaliPathDCSE + idDataElezione;
        return this.http.get<number>(url);
    }

    getElettoriDigitaliCDA(idDataElezione: string, codGruppo: string): Observable<number> {
        const url = environment.baseHost + environment.countElettoriCDA;
        let params = new HttpParams().set("idDataElezione", idDataElezione).set("prefettura", codGruppo);
        return this.http.get<number>(url, { params });
    }

    getVotantiDigitaliDCSE() {
        const url = environment.apiSchede + environment.countVotantiDCSE
        return this.http.get<any>(url);
    }

    getVotantiDigitaliCDA(idDataElezione: string, codGruppo: string) {
        const url = environment.apiSchede + environment.countVotantiCDA
        let params = new HttpParams().set("idDataElezione", idDataElezione).set("prefettura", codGruppo);
        return this.http.get<any>(url, { params });
    }


    getDateAttiveEvote(id: string): Observable<DateAttiveEvoteResponseDto[]> {
        const url = environment.baseHost + environment.getDateAttiveEvote + id;
        return this.http.get<DateAttiveEvoteResponseDto[]>(url);
    }

    formatDate(dateToFormat: string) {
        var date = new Date(dateToFormat);
        var day = date.getDate();
        const month = this.monthNames[date.getMonth()];
        const year = date.getFullYear();
        return `${day} ${month} ${year}`;

    }

}