import { Injectable } from '@angular/core';
import { HttpClient, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs';
import { environment } from '../../../../environments/environment';
import { TempoElettoraleRequestDto } from '../dto/tempo-elettorale-request.dto';
import { TempoElettoraleResponseDto } from '../dto/tempo-elettorale-response.dto';
import { ElezioneFinestraTempoElettoraleDto } from '../dto/elezione-finestra-tempo-elettorale.dto';


@Injectable({
    providedIn: 'root'
})


export class GestioneConsensoVotoService {

    requestObj?: TempoElettoraleRequestDto;

    respObj?: TempoElettoraleResponseDto;

    constructor(private http: HttpClient) { }

    checkPrelevaSIEL(idDataElezione: number){
        const url = environment.baseHost + environment.checkSIEL + '/' + idDataElezione;
        return this.http.get<boolean>(url);
    }


     // formato data dd/mm/yyy 
    getDatiFromSIEL(dataElezione: string): Observable<any> {
        const url = environment.baseHost + environment.getPrelevaDatiSIEL;
        return this.http.post<any>(url, {dataElezione: dataElezione});
    }

    // recupera le date precedentemente salvate
    getTempoElettorale(finestraElezioneDTO  : ElezioneFinestraTempoElettoraleDto ): Observable<TempoElettoraleResponseDto[]> {
        const url = environment.baseHost + environment.getTempoElettoraleUrl;
        var params = new HttpParams().set("idDataElezione", finestraElezioneDTO.idDataElezione).set("tipoFinestraTemporale", finestraElezioneDTO.tipoFinestraTemporale )
        return this.http.get<TempoElettoraleResponseDto[]>(url, {params} );
    }


    setTempoElettorale(tempoElett: TempoElettoraleRequestDto): Observable<TempoElettoraleResponseDto> {
        const url = environment.baseHost + environment.setDataTempoElettoraleUrl;
        return this.http.put<TempoElettoraleResponseDto>(url, tempoElett);
    }

    monthNames = [
        "gennaio", "febbraio", "marzo", "aprile",
        "maggio", "giugno", "luglio", "agosto",
        "settembre", "ottobre", "novembre", "dicembre"];

    formatDate(dateToFormat: string) {
        var date = new Date(dateToFormat);
        var day = date.getDate();
        const month = this.monthNames[date.getMonth()];
        const year = date.getFullYear();
        return `${day} ${month} ${year}`;

    }

}