import { Injectable } from '@angular/core';
import { HttpClient, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs';
import { environment } from '../../../../environments/environment';
import { ScrutinioAggregatoResponseDto } from '../dto/scrutinio-aggregato-response.dto';
import { CandidatiListaResponseDto } from '../dto/candidati-lista-response.dto';
import { ResocontoScrutinioResponseDto } from '../dto/resoconto-scrutinio-response.dto';
import { TrasferisciDatiSIELRequestDto } from '../dto/trasferisci-dati-siel-request.dto';


@Injectable({
    providedIn: 'root'
})
export class ScrutinioService {


    constructor(private http: HttpClient) { }

    getScrutinioAggregatoDCSE(idElezione: number): Observable<ScrutinioAggregatoResponseDto[]> {
        const url = environment.baseHost + environment.getScrutinioAggregatoDCSE;
        var params = new HttpParams().set("idElezione", idElezione)

        return this.http.get<ScrutinioAggregatoResponseDto[]>(url, { params });
    }

    getScrutinioAggregatoCDA(idElezione: number, prefettura: string): Observable<ScrutinioAggregatoResponseDto[]> {
        const url = environment.baseHost + environment.getScrutinioAggregatoCDA;
        var params = new HttpParams().set("idElezione", idElezione).set("prefettura", prefettura)

        return this.http.get<ScrutinioAggregatoResponseDto[]>(url, { params });
    }

    getDettaglioPreferenze(idElezione: number, idLista: number, codEnte: string): Observable<CandidatiListaResponseDto[]> {
        const url = environment.baseHost + environment.dettaglioPreferenze;
        var params = new HttpParams().set("idElezione", idElezione).set("idLista", idLista).set("codEnte", codEnte)

        return this.http.get<CandidatiListaResponseDto[]>(url, { params });
    }

    getResocontoScrutinio(idElezione: number, geoIdEnte: string): Observable<ResocontoScrutinioResponseDto> {
        const url = environment.baseHost + environment.resocontoScrutinio;
        var params = new HttpParams().set("idElezione", idElezione).set("geoIdEnte", geoIdEnte)

        return this.http.get<ResocontoScrutinioResponseDto>(url, { params });
    }

    sendToSIEL(requestDto: TrasferisciDatiSIELRequestDto) {
        const url = environment.baseHost + environment.transferToSIEL;

        return this.http.post<any>(url, requestDto);
    }

}