import { Component, HostListener, OnDestroy, OnInit } from '@angular/core';
import { BreakpointObserver, BreakpointState } from '@angular/cdk/layout';
import { SchedaelettoraleService } from '../../services/schedaelettorale/schedaelettorale.service';
import { TipoElezione } from '../../models/TipoElezione';
import { SchedaElezioneResponse } from '../../models/SchedaElezioneResponse';
import { ActivatedRoute, Router } from '@angular/router';
import { Scheda } from '../../models/Scheda/Scheda';
import { SchedaService } from '../../services/scheda/scheda.service';
import { DatePipe } from '@angular/common';

import { environment } from 'src/environments/environment';

import { JwtHelperService } from '@auth0/angular-jwt';
import { AuthService } from '../../services/auth.service';
import { first, fromEvent, Subject, takeUntil } from 'rxjs';
import { InvioSchede, Lista, ListaCandidati, ListSchedeVoto } from '../../models/Scheda/InvioSchede';
import {Title} from "@angular/platform-browser";

const SMALL_WIDTH_BK = 1000;

@Component({
  selector: 'app-homepage',
  templateUrl: './homepage.component.html',
  styleUrls: ['./homepage.component.scss']
})
export class HomepageComponent implements OnInit, OnDestroy {
  
@HostListener('document:keydown', ['$event'])
onEscape(event:KeyboardEvent){
  if (event.key == "Escape") { 
    if(this.showModal){
      this.showModal = false;
    }
  }
}
  private unsubscriber: Subject<void> = new Subject<void>();

  imagePath = '../../../../voto/assets/homepage/Group 1000002225.png'
  imagePath2 = '../../../../voto/assets/homepage/Group 1000002226.png'
  rightPath = '../../../../voto/assets/homepage/right.png';

  token?: any;
  encryptedText: string = ''; // La tua stringa crittografat
  decryptedText: string = '';
  
  isScreenSmall: boolean = false;
  responseReceived: SchedaElezioneResponse | undefined;
  dataElezione: any;
  letters = '0123456789ABCDEF';
  color = '#';
  schede: Array<Scheda> = [];
  ripartizione: String = "";
  attributoQualificato: String = "";
  idTerritorio: number = 0;
  showModal: boolean = false;
  showLoader: boolean = false; 
  tipoElezioneModal: any;
  schedaModal:any;
  //consente di ottenere un colore randomico per le schede in base al tipo di elezione selezionata.
  //Senato estero e Camera estero sono preimpostati, altre tipologie di elezioni vengono generate
  public getRandomColor() {
    this.color = "#";
    for (var i = 0; i < 6; i++) {

      this.color += this.letters[Math.floor(Math.random() * 16)];
    }
    return this.color;
  }
  public gotoGuidavoto(id: number) {
    this.router.navigateByUrl('app/guidavoto/' + id);
  }
  public gotoVota(idElezione: number, id: number) {
    this.router.navigate(['app/votaelencoliste', { 'idElezione': idElezione, 'id': id }]);

  }

  idSchede: Array<number> = []
  public getQueryParamFromMalformedURL(name: string): String {
    const results = new RegExp('[\\?&]' + name + '=([^&#]*)').exec(decodeURIComponent(this.router.url));
    if (!results) {
      return "";
    }
    return results[1] || "";
  }
  public getSchede() {
    for (let i = 0; i < (sessionStorage.getItem("numberOfSchede")!=null ? Number.parseInt( sessionStorage.getItem("numberOfSchede")!) : 0); i++) {
      if ((sessionStorage.getItem("scheda " + i) == null || sessionStorage.getItem("scheda " + i) == undefined) && this.idSchede.length > 0 && !Number.isNaN(this.idSchede[0])) {
        this.schedaService.getScheda(this.idSchede[i] + '').subscribe(
          response => {
            this.dataElezione = this.datePipe.transform(response.dataElezione, 'dd/MM/yyyy')
            //determina la ripartizione cui appartiene la scheda
            this.ripartizione = response.descTerritorio;
            document.title = "Schede elettorali accettate - Ripartizione " + this.ripartizione + " | Portale Votazione";
            //QUESTA PORZIONE DI CODICE E' STATA ELIMINATA PER MOTIVI DI SICUREZZA  
            ;
            
            setTimeout(() => {                           // <<<---using ()=> syntax
              this.schede.splice(i, 0, response);

            }, 100);

          },
          error => { 
            
          }
        )
      }
      else if (sessionStorage.getItem("scheda " + i) != null) {

        let schedaStringed:any;
        //QUESTA PORZIONE DI CODICE E' STATA ELIMINATA PER MOTIVI DI SICUREZZA  
        let schedaDecoded = JSON.parse(schedaStringed)
        this.dataElezione = this.datePipe.transform(schedaDecoded.dataElezione, 'dd/MM/yyyy')
        //determina la ripartizione cui appartiene la scheda
        this.ripartizione = schedaDecoded?.descTerritorio.charAt(0)+  schedaDecoded?.descTerritorio.toLowerCase().slice(1);
        
        this.schede.splice(i, 0, schedaDecoded);

      }
    }

  }



  ngOnInit(): void {

    const count = localStorage.getItem('openCounter');
    if(count === null){
      localStorage.setItem('openCounter', '1');
    }
    else {
      localStorage.setItem('openCounter', parseInt(count) + 1 + '');
    }

  

    //blocca il pulsante indietro in fase di voto
    history.pushState(null, '');
    history.scrollRestoration = 'manual',

      fromEvent(window, 'popstate')
        .pipe(takeUntil(this.unsubscriber))
        .subscribe((_) => {
          history.pushState(null, '');
        });
    if (this.route.snapshot.queryParamMap.get('token') && this.route.snapshot.queryParamMap.get('token') != 'null') {
      sessionStorage.clear()
      let jwtString: string = this.route.snapshot.queryParamMap.get('token')!;

      let decoded:any;
      //QUESTA PORZIONE DI CODICE E' STATA ELIMINATA PER MOTIVI DI SICUREZZA  

      this.authService.setSessionStorage('jwtTokenVoto', this.token);
      this.authService.setSessionStorage('qualifiedAttribute', decoded.voto.qualifiedAttribute);
      
    }

    this.getSchede();
    if (this.schede != undefined) {
      let numSchede = this.schede.length;
      let numSchedeVot = 0;
      for (let scheda of this.schede) {
        if (scheda != undefined && (scheda.votata || scheda.tempoScaduto)) {
          numSchedeVot += 1;
        }
      }
      if (numSchede != 0 && numSchede == numSchedeVot) {
        this.setVoto()
      }
    }
    this.breakpointObserver.observe([`(max-width:${SMALL_WIDTH_BK}px)`])
      .subscribe((state: BreakpointState) => {
        this.isScreenSmall = state.matches;
      })
  }
  constructor(private breakpointObserver: BreakpointObserver,
    private schedeEletService: SchedaelettoraleService, private schedaService: SchedaService,
    private router: Router, private datePipe: DatePipe,
    private route: ActivatedRoute,
    private authService: AuthService, private titleService:Title
  ) {
    this.titleService.setTitle("Schede elettorali accettate - Ripartizione " + this.ripartizione + " | Portale Votazione");

  }
  ngOnDestroy(): void {
    this.unsubscriber.next();
    this.unsubscriber.complete();

  }

  openLoader() {
    this.showLoader = true;

  }
  closeLoader() {

    this.showLoader = false;
  }
  openModal(tipoElezione: number, scheda: number) {
    this.tipoElezioneModal = tipoElezione;
    this.schedaModal = scheda;
    this.showModal = true;

  }
  closeModal() {

    this.showModal = false;

  }

  setVoto() {
    
    this.showLoader = true
    let invioSchede: InvioSchede = new InvioSchede();
    invioSchede.setAttributoQualificato(sessionStorage.getItem("qualifiedAttribute")!);
    let listaSchede: ListSchedeVoto[] = [];
    if (this.schede.length > 0 && this.schede[0].listaSchedaDTOList.length > 0) {
      invioSchede.setGeoId(sessionStorage.getItem("geoId")!);
      for (let scheda of this.schede) {
        let listaScheda = new ListSchedeVoto();
        listaScheda.setChiaveScheda(scheda.idScheda);
        if (!scheda.tempoScaduto) {
          listaScheda.setVotata(true);
          invioSchede.setIdSchedeVotate(invioSchede.getIdSchedeVotate() + `${scheda.idScheda};`) 
          let hasLista: boolean = false;
          for (let lista of scheda.listaSchedaDTOList) {
            if (lista.selected) {
              hasLista =true;
              listaScheda.setLista(new Lista());
              listaScheda.getLista().setIdlista(lista.lista.id);
              listaScheda.getLista().setDescLista(lista.lista.descrizione);
              let listaCandidati: ListaCandidati[] = [];
              if (lista.selectedCand1 != undefined) {
                let candidato = new ListaCandidati(lista.selectedCand1.id,
                  lista.selectedCand1.nome, lista.selectedCand1.cognome);
                listaCandidati.push(candidato);
              }
              if (lista.selectedCand2 != undefined ) {
                let candidato2 = new ListaCandidati(lista.selectedCand2.id,
                  lista.selectedCand2.nome, lista.selectedCand2.cognome);
                listaCandidati.push(candidato2);
              }
              listaScheda.getLista().setListaCandidati(listaCandidati)
            }
          }
          if(!hasLista){
            listaScheda.setBianca(true)
          }

        }
        else{
          invioSchede.setIdSchedeNonVotate(invioSchede.getIdSchedeNonVotate() + `${scheda.idScheda};`) 
        }
        listaSchede.push(listaScheda)

      }
    }
    invioSchede.setListSchedeVoto(listaSchede);

    this.schedaService.setVoto(invioSchede).then(
      res =>{
        localStorage.setItem("qualifiedAttribute", sessionStorage.getItem("qualifiedAttribute")!)
        let idSchedeVotate = "";
        let idSchedeNonVotate="";
        
        for(let scheda of invioSchede.getListSchedeVoto()){

          if(scheda.getVotata()){
            idSchedeVotate += `${scheda.getChiaveScheda()};`
          }
          else{
            idSchedeNonVotate += `${scheda.getChiaveScheda()};`
          }
        }
        //QUESTA PORZIONE DI CODICE E' STATA ELIMINATA PER MOTIVI DI SICUREZZA  

        setTimeout( ()=>{
          window.close()
        }
        , 15000);
      }
    )
  }
}
