import { BreakpointState, BreakpointObserver } from '@angular/cdk/layout';
import { Component, HostListener, OnDestroy, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { TimervotoService } from '../../services/timervoto/timervoto.service';
import { SchedaService } from '../../services/scheda/scheda.service';
import { ListaSchedaDTOList } from '../../models/Scheda/ListaSchedaDTOList';
import { Candidato } from '../../models/Scheda/Candidato';
import { DomSanitizer } from '@angular/platform-browser';
import { Contrassegno } from '../../models/Scheda/Contrassegno';
import { Scheda } from '../../models/Scheda/Scheda';
import { fromEvent, Subject, takeUntil } from 'rxjs';
import { EventManager } from '@angular/platform-browser';
import { environment } from 'src/environments/environment';
import {Title} from "@angular/platform-browser";

const SMALL_WIDTH_BK = 720;

@Component({
  selector: 'app-votaelencoliste',
  templateUrl: './votaelencoliste.component.html',
  styleUrls: ['./votaelencoliste.component.scss']
})
export class VotaelencolisteComponent implements OnInit, OnDestroy {

  @HostListener('document:keydown', ['$event'])
  onEscape(event: KeyboardEvent) {
    if (event.key == "Escape") {
      if (this.showModal) {
        this.showModal = false;
      }
      else if (this.showModal2) {
        this.showModal2 = false;
      }
      else if (this.modalRipulisci) {
        this.modalRipulisci = false;
      }
      if (this.showModalGenere) {
        this.showModalGenere = false
      }
      if (this.showModalGenereErrore) {
        this.showModalGenere = false

      }
    }
  }
  tempoScaduto: boolean = false

  ripartizione: string = ""
  liste: ListaSchedaDTOList[] | undefined;
  scheda: Scheda | undefined;
  //booleano per il modale di consegna scheda bianca
  showModal: boolean = false;
  //booleano per il modale di conferma del voto
  showModal2: boolean = false;
  //array per convertire il numero di preferenze in numero di occorrenze, per mostrare in ngFor tante select quante sono le preferenze
  arrayNumPreferenze: Array<Object> = []
  isSchedaAndCandidato: boolean = false;
  //indica se una scheda è stata selezionata
  schedaSelected: boolean = false;
  //indica quale lista è stata selezionata
  listaSelezionata: ListaSchedaDTOList | undefined;
  descrizione = '';
  //il colore di base è quello della camera, ma varia in base alla tipologia di elezione selezionata
  cardColorChange = 'camera';
  //il colore del bordo segue quello delle card
  borderColor = "";
  //indica quante preferenze ho selezionato per il modale di conferma finale
  numPreferenzeSelected: number = 0;
  isScreenSmall: boolean = false;
  imagePath = '../../../../voto/assets/votaelencoliste/Group 1000003574.png';
  banPath = '../../../../voto/assets/votaelencoliste/Ban.png';
  consegnaPath = '../../../../voto/assets/votaelencoliste/Shape.png';
  crossPath = '../../../../voto/assets/votaelencoliste/3705080.png';
  //booleano per il modale di avviso per vincolo di genere
  showModalGenere = false;
  //booleano per il modale di vincolo di genere non rispettato
  showModalGenereErrore = false;
  modalRipulisci = false;
  numberOfScheda = 0;
  setSchedaChiamato: boolean = false;
  tempoDaPosticipare: boolean = false;


  constructor(private breakpointObserver: BreakpointObserver, private router: Router, private activatedRoute: ActivatedRoute
    , private timerService: TimervotoService, private schedaService: SchedaService, private sanitizer: DomSanitizer, private titleService : Title) {
    timerService.getTempoScaduto().subscribe(tmp => this.tempoScaduto = tmp)
    timerService.getTempoDaPosticipare().subscribe(tmp => this.tempoDaPosticipare = tmp)

    setTimeout( ()=>{
      this.titleService.setTitle(this.scheda!.descrizioneElezione +" Ripartizione " + this.ripartizione + " area privata | Portale Votazione");

    }, 1)
  }
  private unsubscriber: Subject<void> = new Subject<void>();

  ngOnDestroy(): void {

    this.unsubscriber.next();
    this.unsubscriber.complete();
    //Se esco dalla pagina di voto la scheda viene salvata
    if (this.tempoScaduto) this.setScheda(false, true)

  }
  ngOnInit(): void {



    //blocca il pulsante indietro in fase di voto
    history.pushState(null, '');
    history.scrollRestoration = 'manual';

    fromEvent(window, 'popstate')
      .pipe(takeUntil(this.unsubscriber))
      .subscribe((_) => {
        history.scrollRestoration = 'manual';
        history.pushState(null, '');
      });

    //controlla la dimensione dello schermo
    this.breakpointObserver.observe([`(max-width:${SMALL_WIDTH_BK}px)`])
      .subscribe((state: BreakpointState) => {
        this.isScreenSmall = state.matches;
      })
    this.descrizione = this.activatedRoute.snapshot.params['idElezione'].includes("4") ? 'Camera Estero' : 'Senato Estero';
    if (!this.descrizione.includes("Camera")) {
      this.cardColorChange = "senato";
      this.borderColor = "senatoBorder";
    }
    else {
      this.borderColor = "cameraBorder";

    }

    for (let i = 0; i < (sessionStorage.getItem("numberOfSchede") != null ? Number.parseInt(sessionStorage.getItem("numberOfSchede")!) : 0); i++) {
      if (sessionStorage.getItem("scheda " + i) != null) {
        let schedaStringed:any;//QUESTA PORZIONE DI CODICE E' STATA ELIMINATA PER MOTIVI DI SICUREZZA  
        let schedaDecoded = JSON.parse(schedaStringed)
        if (schedaDecoded.idScheda + "" == this.activatedRoute.snapshot.params['id']) {
          sessionStorage.setItem("ripartizioneScheda", schedaDecoded?.descrizioneElezione + " - Ripartizione " + schedaDecoded?.descTerritorio.charAt(0) + schedaDecoded?.descTerritorio.toLowerCase().slice(1) + "")
          this.ripartizione = schedaDecoded?.descTerritorio.charAt(0) + schedaDecoded?.descTerritorio.toLowerCase().slice(1)
          this.numberOfScheda = i;
          this.scheda = schedaDecoded;
          this.liste = this.scheda?.listaSchedaDTOList;
          this.liste?.forEach((lista) => {
            this.getImg(lista.lista.contrassegno)
            //compilo l'attributo fullname del candidato per sintetizzare in un'unica variabile tutto il nominativo del candidato
            lista.candidatoLista.forEach((candidato) => {
              candidato.fullName = `${candidato.nome} ${candidato.cognome} ${candidato.altroNome != null ? candidato.altroNome.descrizione : ''} ${candidato.descrizioneAltronome != null ? candidato.descrizioneAltronome : ''} `
            })
          })
          for (let i = 1; i <= this.scheda?.numPreferenze!; i++) {
            this.arrayNumPreferenze.push(new Object());
          }
          if (sessionStorage.getItem("listaSelezionata") != undefined && sessionStorage.getItem("listaSelezionata") != null) {
           //QUESTA PORZIONE DI CODICE E' STATA ELIMINATA PER MOTIVI DI SICUREZZA  
            this.schedaSelected = true;
            this.liste?.forEach((lista) => {
              if (lista.lista.id == this.listaSelezionata?.lista.id) {
                lista.selected = true
                lista.selectedCand1 = this.listaSelezionata.selectedCand1
                lista.selectedCand2 = this.listaSelezionata.selectedCand2
                for (let candidato of lista.candidatoLista) {
                  if (candidato.fullName == this.listaSelezionata.selectedCand1?.fullName ||
                    candidato.fullName == this.listaSelezionata.selectedCand2?.fullName) {
                    this.isSchedaAndCandidato = true
                    candidato.selected = true
                    this.numPreferenzeSelected += 1
                  }
                }
              }
            })      
          }
        }
      }
    }
  }

  //imposta il candidato selezionato all'interno del dropdown in modo che sia visibile
  setCandidato(idLista: number, candidato: Candidato, numCandidato: number) {
    this.liste?.forEach((lista) => {
      if (lista.lista.id == idLista) {
        if (numCandidato == 1) {
          lista.candidatoLista.forEach((cand) => {
            if (lista.selectedCand1 != null && cand.fullName == lista.selectedCand1.fullName) {
              cand.selected = false;
              this.numPreferenzeSelected -= 1

            }
          })
          lista.selectedCand1 = candidato;
          this.numPreferenzeSelected += 1
        }
        else if (numCandidato == 2) {
          lista.candidatoLista.forEach((cand) => {
            if (lista.selectedCand2 != null && cand.fullName == lista.selectedCand2.fullName) {
              cand.selected = false;
              this.numPreferenzeSelected -= 1

            }
          })
          lista.selectedCand2 = candidato;
          this.numPreferenzeSelected += 1

        }
        this.listaSelezionata = lista
        //QUESTA PORZIONE DI CODICE E' STATA ELIMINATA PER MOTIVI DI SICUREZZA  

      }
      candidato.selected = true;
      this.isSchedaAndCandidato = true
      if (this.isSchedaAndCandidato && this.scheda?.numPreferenze! > 1
        && this.listaSelezionata != undefined
        && this.listaSelezionata.selectedCand1 != undefined
        && this.listaSelezionata.selectedCand2 != undefined
        && this.listaSelezionata?.selectedCand1!.sesso === this.listaSelezionata?.selectedCand2!.sesso) {
        this.showModalGenereErrore = true;
      }
      else if (this.isSchedaAndCandidato && this.scheda?.numPreferenze! > 1) {
        if (this.listaSelezionata != undefined && this.listaSelezionata.selectedCand1 != undefined
          && this.listaSelezionata.selectedCand2 != undefined
          && this.listaSelezionata?.selectedCand1!.sesso !== this.listaSelezionata?.selectedCand2!.sesso) {
          this.showModalGenereErrore = false;
          this.showModalGenere = false;
        }
        else
          this.showModalGenere = true;

      }
    })
  }
  //seleziona la lista
  selectScheda(idLista: number) {
    if (!this.schedaSelected) {
      this.schedaSelected = true;
      this.liste?.forEach((lista) => {
        if (lista.lista.id == idLista) {
          lista.selected = true;
          this.listaSelezionata = lista
          //QUESTA PORZIONE DI CODICE E' STATA ELIMINATA PER MOTIVI DI SICUREZZA  

        }
      })
    }

  }
  //ripulisce le liste
  deselectScheda() {
    this.schedaSelected = false
    this.liste?.forEach((lista) => {
      lista.selected = false;
      this.listaSelezionata = undefined;
      sessionStorage.removeItem("listaSelezionata");
      lista.selectedCand1 = undefined;
      lista.selectedCand2 = undefined;
      lista.candidatoLista.forEach((cand) => {
        cand.selected = false
      })
    })
    this.isSchedaAndCandidato = false
    this.numPreferenzeSelected = 0;

  }

  //converte il contrassegno da Array di stringhe a immagine
  public getImg(contrassegno: Contrassegno): void {
    contrassegno.imgSrc = this.sanitizer.bypassSecurityTrustUrl('data:image/png;base64,' + contrassegno.blobImmagine);
  }

  openModal() {
    this.showModal = true;
  }
  closeModal() {
    this.showModal = false;
  }


  openModal2() {
    this.showModal2 = true;
  }
  closeModal2() {
    this.showModal2 = false;
  }

  closeModalGenere() {
    this.showModalGenere = false;
  }
  closeModalGenereErrore() {
    this.showModalGenereErrore = false;
    setTimeout(() => { this.closeModalGenere() }, 10)
  }
  openRipulisci() {

    this.modalRipulisci = true;

  }
  setScheda(isVotazioneEffettuata: boolean, isTempoScaduto: boolean) {
    this.setSchedaChiamato = true;
    if (!isVotazioneEffettuata) {
      this.deselectScheda();
    }
    this.scheda!.votata = true;
    this.scheda!.tempoScaduto = isTempoScaduto;
    if (isTempoScaduto) {
    }
   //QUESTA PORZIONE DI CODICE E' STATA ELIMINATA PER MOTIVI DI SICUREZZA  
    this.router.navigateByUrl("voto/app/home")
  }

  isSafariMobile(): boolean {
    return /iPhone|iPad/.test(navigator.userAgent)
  }
}
