package com.accenture.eVote.elettorims.config;

import com.accenture.eVote.elettorims.util.Crittografia;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

@Component
public class JwtTokenInterceptor implements HandlerInterceptor {
    private final Logger log = LoggerFactory.getLogger(JwtTokenInterceptor.class);
    @Autowired
    private Crittografia crittografia;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        // Ottieni il token JWT dall'header della richiesta
        String jwtToken = request.getHeader("Authorization");

        if (request.getMethod().equals("OPTIONS")) {
            return true;
        }

        try {
            // Decodifica del token
            String jwtTokenDec = crittografia.decodificaTokenData(jwtToken);
            // Verifica la validità del token (controlla anche la scadenza)
            if (crittografia.verificaScadenzaToken(jwtTokenDec)) {
                // Il token è valido e ne estendo la durata di 10 minuti
                // response.addHeader("Authorization", crittografia.estendiDurataToken(jwtTokenDec));
                return true; // Il token è valido, permetti la richiesta di passare
            } else {
                response.sendError(HttpStatus.UNAUTHORIZED.value(), "Token scaduto o non valido");
                return false; // Il token non è valido, blocca la richiesta
            }
        } catch (Exception e) {
            response.sendError(HttpStatus.UNAUTHORIZED.value(), "Token scaduto o non valido");
            return false; // Il token non è valido, blocca la richiesta
        }
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {
        // Questo metodo viene chiamato dopo l'esecuzione del metodo del controller
        // Puoi implementare la logica qui se hai bisogno di eseguire azioni dopo il completamento della richiesta
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {
        // Questo metodo viene chiamato dopo che la risposta è stata inviata al client
        // Puoi implementare la logica qui se hai bisogno di eseguire azioni dopo che la risposta è stata completata
    }
}