package com.accenture.eVote.elettorims.controller;

import com.accenture.eVote.elettorims.dto.ConfTempoVotoDTO;
import com.accenture.eVote.elettorims.dto.ConsensoVotoDTO;
import com.accenture.eVote.elettorims.model.request.ConsensoVotoRequest;
import com.accenture.eVote.elettorims.model.request.ConsensoVotoRequestBody;
import com.accenture.eVote.elettorims.service.ConsensoVotoService;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/protected/elettori/consenso-voto")
public class ConsensoVotoController {
    @Autowired
    private ConsensoVotoService consensoVotoService;
    @Value("${headers.session.id}")
    private String headerSessionId;

    @PutMapping("")
    public ResponseEntity<ConsensoVotoDTO> setConsensoVoto(@RequestBody @Valid ConsensoVotoRequestBody consensoVotoRequestBody, HttpServletRequest request) {

        ConsensoVotoRequest consensoVotoRequest = new ConsensoVotoRequest();
        Long userId = Long.valueOf(request.getHeader(headerSessionId));

        if (userId == null) {
            throw new IllegalArgumentException("L'id user non può essere null");
        }

        // Setting dello userId decodificato di sessione
        consensoVotoRequest.setUserId(userId);
        consensoVotoRequest.setFlagConsenso(consensoVotoRequestBody.getFlagConsenso());

        ConsensoVotoDTO consensoVotoDTO = consensoVotoService.creaModificaConsenso(consensoVotoRequest);
        if (consensoVotoDTO == null) {
            throw new RuntimeException("Finestra temporale non rispettata...");
        } else {
            return ResponseEntity.ok().body(consensoVotoDTO);
        }
    }

    //API per verificare la presenza del consenso voto con finestra attiva
    @GetMapping("/trova-consenso")
    public ResponseEntity<ConsensoVotoDTO> getFinestraConsensoVoto(HttpServletRequest request) {

        Long userId = Long.valueOf(request.getHeader(headerSessionId));

        if (userId == null) {
            throw new IllegalArgumentException("L'id user non può essere null");
        }

        ConsensoVotoDTO consensoVoto = consensoVotoService.getFinestraConsensoVoto(userId);
        return ResponseEntity.ok().body(consensoVoto);
    }

    //API per verificare la presenza dell'elezione.
    @GetMapping("/trova-voto")
    public ResponseEntity<ConfTempoVotoDTO> getFinestraVoto(HttpServletRequest request) {

        Long userId = Long.valueOf(request.getHeader(headerSessionId));

        if (userId == null) {
            throw new IllegalArgumentException("L'id non può essere null");
        }

        ConfTempoVotoDTO confTempoVotoDTO = consensoVotoService.getFinestraVoto(userId);

        return ResponseEntity.ok().body(confTempoVotoDTO);
    }


}