package com.accenture.eVote.elettorims.controller;

import com.accenture.eVote.elettorims.dto.ElettoreDTO;
import com.accenture.eVote.elettorims.dto.ElettoreSchedaDTO;
import com.accenture.eVote.elettorims.dto.RicevutaVotoRequestDTO;
import com.accenture.eVote.elettorims.model.response.VerificaVotoResponse;
import com.accenture.eVote.elettorims.service.ElettoreService;
import io.github.resilience4j.bulkhead.annotation.Bulkhead;
import io.github.resilience4j.retry.annotation.Retry;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;

import java.util.List;

@RestController
@RequestMapping("/protected/elettori/info")
public class ElettoreController {

    @Autowired
    ElettoreService elettoreService;
    @Autowired
    private RestTemplate restTemplate;
    @Value("${url.filems.download}")
    private String urlDownload;
    @Value("${url.filems.inviomail}")
    private String urlInvioMail;
    @Value("${headers.session.id}")
    private String headerSessionId;

    @GetMapping("/id")
    public ResponseEntity<Object> getElettoreById(HttpServletRequest request) {

        Long userId = Long.valueOf(request.getHeader(headerSessionId));

        try {
            return ResponseEntity.ok().body(elettoreService.getElettoreById(userId));
        } catch (Exception e) {
            return new ResponseEntity<>(e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @GetMapping("/download-ricevuta-voto")
    @Retry(name = "retry-instance")
    @Bulkhead(name = "bulkhead-instance")
    public ResponseEntity<Resource> downloadRicevutaVoto(HttpServletRequest request, @RequestParam(value = "idDataElezione", required = false) Long idDataElezione) {
        Long userId = Long.valueOf(request.getHeader(headerSessionId));

        if (idDataElezione == null) {
            return restTemplate.postForEntity(urlDownload, new RicevutaVotoRequestDTO(elettoreService.getRicevuteVoto(userId), null), Resource.class);
        } else {
            return restTemplate.postForEntity(urlDownload, new RicevutaVotoRequestDTO(elettoreService.getRicevuteVoto(userId), idDataElezione), Resource.class);
        }

    }

    @GetMapping("/invio-ricevuta-voto")
    @Retry(name = "retry-instance")
    @Bulkhead(name = "bulkhead-instance")
    public ResponseEntity<String> invioRicevutaVoto(HttpServletRequest request, @RequestParam(value = "idDataElezione", required = false) Long idDataElezione) {
        Long userId = Long.valueOf(request.getHeader(headerSessionId));

        if (idDataElezione == null) {
            return restTemplate.postForEntity(urlInvioMail, new RicevutaVotoRequestDTO(elettoreService.getRicevuteVoto(userId), null), String.class);
        } else {
            return restTemplate.postForEntity(urlInvioMail, new RicevutaVotoRequestDTO(elettoreService.getRicevuteVoto(userId), idDataElezione), String.class);
        }
    }

    @PostMapping("/conferma-schede")
    public ResponseEntity<Object> setAccettazioneScheda(@RequestBody @Valid List<ElettoreSchedaDTO> listSchede, HttpServletRequest request) {

        Long userId = Long.valueOf(request.getHeader(headerSessionId));

        if (listSchede.stream().anyMatch(s -> s.getSchedaVoto() == null || userId == null)) {
            throw new IllegalArgumentException("L'id e/o la scheda non possono essere null");
        }

        // Verifico che le schede inviate siano coerenti con l'elezione attiva in corso
        if (!elettoreService.verificaSchede(listSchede)) {
            throw new IllegalArgumentException("Le schede devono essere associate a elezioni in fase di voto");
        }
        ;
        List<ElettoreSchedaDTO> listSchedaDTO = elettoreService.setAccettazioneSchede(listSchede, userId);
        if (!listSchedaDTO.isEmpty()) {
            return ResponseEntity.ok().body(listSchedaDTO);
        } else {
            String errorMessage = "Gli id sono già presenti.";
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(errorMessage);
        }
    }

    @GetMapping("/verifica-voto")
    public ResponseEntity<VerificaVotoResponse> getVerificaVoto(HttpServletRequest request) throws InterruptedException {

        Long userId = Long.valueOf(request.getHeader(headerSessionId));

        VerificaVotoResponse verificaVotoResponse = elettoreService.verificaVoto(userId);

        if (verificaVotoResponse.isVotato()) {
            elettoreService.verificaRicevutaScheda(userId);
        }

        return ResponseEntity.ok().body(verificaVotoResponse);
    }

    @GetMapping("/check-email")
    public ResponseEntity<Object> checkEmail(HttpServletRequest request) {
        Long userId = Long.valueOf(request.getHeader(headerSessionId));
        try {
            ElettoreDTO elettore = elettoreService.getElettoreById(userId);
            return ResponseEntity.ok().body(elettoreService.checkEmail(elettore));
        } catch (Exception e) {
            return new ResponseEntity<>(e.getMessage(), HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

}
