package com.accenture.eVote.elettorims.controller;

import com.accenture.eVote.elettorims.model.request.AutocertificazioneRequest;
import com.accenture.eVote.elettorims.model.response.*;
import com.accenture.eVote.elettorims.service.LoginService;
import com.accenture.eVote.elettorims.service.MetodoAutenticazioneService;
import com.accenture.eVote.elettorims.service.TipoDocumentoService;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.github.resilience4j.bulkhead.annotation.Bulkhead;
import io.github.resilience4j.retry.annotation.Retry;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.servlet.view.RedirectView;

import java.util.Base64;
import java.util.Objects;
import java.util.Random;

@RestController
@RequestMapping("/")
public class LoginController {
    private final WebClient.Builder webClientBuilder;
    @Autowired
    LoginService loginService;
    @Autowired
    TipoDocumentoService tipoDocumentoService;
    @Autowired
    MetodoAutenticazioneService metodoAutenticazioneService;
    @Value("${url.frontend.auth}")
    private String urlAuth;

    @Autowired
    public LoginController(WebClient.Builder webClientBuilder) {
        this.webClientBuilder = webClientBuilder;
    }

    @Value("${url.login.spid.validazione}")
    private String urlValidazione;

    @PostMapping("/login/authsg")
    public RedirectView login1(String token) {

        token = loginService.codificaTokenAuth(token);

        return new RedirectView(urlAuth + "?token=" + token);
    }

    @Autowired
    private RestTemplate restTemplate;

    @GetMapping("/login/authsg/data{token}")
    //@Retry(name = "retry-instance")
    //@Bulkhead(name = "bulkhead-instance")
    public ResponseEntity<Object> login(@RequestParam(name = "token") String token) throws JsonProcessingException {
        ObjectMapper objectMapper = new ObjectMapper();

        LoginResponse loginResponse = new LoginResponse();

        // Decodifica da Base64 e recupero token
        token = loginService.decodificaTokenAuth(new String(Base64.getDecoder().decode(token.getBytes())));

        // Traduzione token da servizio esterno
        LoginSpidResponse loginSpidResponse = webClientBuilder.build()
                .post()
                .uri(urlValidazione + token)
                .retrieve()
                .bodyToMono(LoginSpidResponse.class)
                .block();

        assert Objects.requireNonNull(loginSpidResponse).getFiscalNumber() != null;

        if (loginSpidResponse.getFiscalNumber().contains("TINIT-")) {
            loginSpidResponse.setFiscalNumber(loginSpidResponse.getFiscalNumber().substring(6));
        } else {
            loginSpidResponse.setFiscalNumber(loginSpidResponse.getFiscalNumber());
        }

        if (loginSpidResponse.getDigitalAddress() != null) {
            if (loginSpidResponse.getDigitalAddress().equals("-")) {
                loginSpidResponse.setDigitalAddress(null);
            }
        }

        String listVotiCrittata = objectMapper.writeValueAsString(loginSpidResponse);

        // Verifica diritto al voto
        if (!loginService.verificaDirittoAlVoto(loginSpidResponse)) {
            return new ResponseEntity<>("Utente non ha diritto al voto", HttpStatus.UNAUTHORIZED);
        }

        // verifica maggiore età elettore
        if (loginService.verificaMaggioreEta(loginSpidResponse)) {
            loginResponse = loginService.login(token, loginSpidResponse);
        } else {
            return new ResponseEntity<>("Utente minorenne", HttpStatus.UNAUTHORIZED);
        }

        return ResponseEntity.ok().body(loginResponse);
    }

    @PostMapping("/login-fake")
    public ResponseEntity<Object> loginFake() throws JsonProcessingException, InterruptedException {

        ObjectMapper objectMapper = new ObjectMapper();

        LoginResponse loginResponse = new LoginResponse();

        //Thread.sleep(30000); //30 secondi

        LoginSpidResponse loginSpidResponse = new LoginSpidResponse();

        loginSpidResponse.setIss("ISS");
        loginSpidResponse.setJti("JTI");
        loginSpidResponse.setSub("SUB");
        loginSpidResponse.setExp("EXP");
        loginSpidResponse.setIat("IAT");
        loginSpidResponse.setEnv("TEST");
        loginSpidResponse.setIdp("SIELTEID");
        loginSpidResponse.setService_id("SERVICE_ID");
        loginSpidResponse.setName("PIPPO");
        loginSpidResponse.setFamilyName("PLUTO");
        loginSpidResponse.setPlaceOfBirth("TEST");
        loginSpidResponse.setDateOfBirth("1980-01-01");
        loginSpidResponse.setGender("M");
        loginSpidResponse.setFiscalNumber(generaUtenzaRandomicaUno().concat(generaUtenzaRandomicaDue()));
        loginSpidResponse.setMobilePhone("1234556");
        loginSpidResponse.setDigitalAddress("a@a.it");


        assert Objects.requireNonNull(loginSpidResponse).getFiscalNumber() != null;

        if (loginSpidResponse.getFiscalNumber().contains("TINIT-")) {
            loginSpidResponse.setFiscalNumber(loginSpidResponse.getFiscalNumber().substring(6));
        } else {
            loginSpidResponse.setFiscalNumber(loginSpidResponse.getFiscalNumber());
        }

        if (loginSpidResponse.getDigitalAddress() != null) {
            if (loginSpidResponse.getDigitalAddress().equals("-")) {
                loginSpidResponse.setDigitalAddress(null);
            }
        }

        String listVotiCrittata = objectMapper.writeValueAsString(loginSpidResponse);

        loginResponse = loginService.login(null, loginSpidResponse);

        return ResponseEntity.ok().body(loginResponse);
    }

    @PostMapping("/protected/login/autocertificazione")
    @Retry(name = "retry-instance")
    @Bulkhead(name = "bulkhead-instance")
    public AutocertificazioneResponse autocertificazione(@RequestBody AutocertificazioneRequest autocertificazioneRequest, HttpServletRequest request) {

        return loginService.autocertificazione(autocertificazioneRequest, request.getHeader("Authorization"));
    }

    @GetMapping("/protected/login/tipi/documento")
    public ResponseEntity<TipoDocumentoResponse> getTipiDocumento() {
        return ResponseEntity.ok().body(tipoDocumentoService.getAll());
    }

    @GetMapping("login/authsg/tipi/autenticazione")
    public ResponseEntity<MetodoAutenticazioneResponse> getTipoAutenticazione() {
        return ResponseEntity.ok().body(metodoAutenticazioneService.getAll());
    }

    private static String generaUtenzaRandomicaUno() {

        String caratteriValidi = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
        int lunghezzaUtenza = 10;

        Random random = new Random();
        StringBuilder utenzaBuilder = new StringBuilder(lunghezzaUtenza);

        for (int i = 0; i < lunghezzaUtenza; i++) {
            int index = random.nextInt(caratteriValidi.length());
            char carattereCasuale = caratteriValidi.charAt(index);
            utenzaBuilder.append(carattereCasuale);
        }

        return utenzaBuilder.toString();
    }

    private static String generaUtenzaRandomicaDue() {

        String caratteriValidi = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
        int lunghezzaUtenza = 6;

        Random random = new Random();
        StringBuilder utenzaBuilder = new StringBuilder(lunghezzaUtenza);

        for (int i = 0; i < lunghezzaUtenza; i++) {
            int index = random.nextInt(caratteriValidi.length());
            char carattereCasuale = caratteriValidi.charAt(index);
            utenzaBuilder.append(carattereCasuale);
        }

        return utenzaBuilder.toString();
    }

}
