package com.accenture.eVote.elettorims.controller;

import com.accenture.eVote.elettorims.model.request.OracleRequest;
import com.accenture.eVote.elettorims.model.request.OracleValidateRequest;
import com.accenture.eVote.elettorims.model.response.AttributoQualificatoResponse;
import com.accenture.eVote.elettorims.model.response.OracleGenerateResponse;
import com.accenture.eVote.elettorims.model.response.OracleValidateResponse;
import com.accenture.eVote.elettorims.service.OracleService;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.reactive.function.client.WebClient;

@RestController
@RequestMapping("/protected/oracle")
public class OracleController {
    private final WebClient.Builder webClientBuilder;

    public OracleController(WebClient.Builder webClientBuilder) {
        this.webClientBuilder = webClientBuilder;
    }

    @Autowired
    private OracleService oracleService;
    @Value("${url.oracle.qa.generate}")
    private String urlOracleGenerate;
    @Value("${url.oracle.qa.validate}")
    private String urlOracleValidate;
    @Value("${headers.session.id}")
    private String headerSessionId;

    @PostMapping("/qa")
    public AttributoQualificatoResponse getAttributoQualificato(@RequestBody @Valid OracleRequest oracleRequest, HttpServletRequest request) {

        Long userId = Long.valueOf(request.getHeader(headerSessionId));

        // Setting dello user id nella request per Oracle
        oracleRequest.setUserId(userId);

        // Verifica che le preferenze corrispondono agli id schede accettate
        if (Boolean.FALSE.equals(oracleService.verifySchede(oracleRequest))) {
            return null;
        }
        // cripta la request

        byte[] oracleRequestCod = oracleService.getRequestCodificata(oracleRequest);
        // Genero/recpero l'attributo qualificato
        OracleGenerateResponse oracleGenerateResponse = webClientBuilder.build()
                .post()
                .uri(urlOracleGenerate)
                .bodyValue(oracleRequestCod)
                .retrieve()
                .bodyToMono(OracleGenerateResponse.class)
                .block();
        // Validazione atributo qualificato

        // Genero la richiesta di validazione
        OracleValidateRequest oracleValidateRequest = new OracleValidateRequest(oracleGenerateResponse.getQualifiedAttribute());
        // Genero/recpero l'attributo qualificato

        OracleValidateResponse oracleValidateResponse = webClientBuilder.build()
                .post()
                .uri(urlOracleValidate)
                .bodyValue(oracleValidateRequest)
                .retrieve()
                .bodyToMono(OracleValidateResponse.class)
                .block();

        if (oracleValidateResponse.getVerified()) {

            oracleGenerateResponse.setUserPreferences(oracleRequest.getUserPreferences());
            oracleGenerateResponse.setGeoId(oracleService.getGeoIdByIdElettore(oracleRequest));

            // Genero un token codificato per l'autenticazione di voto
            return new AttributoQualificatoResponse(oracleService.generaTokenVoto(oracleGenerateResponse), oracleGenerateResponse.getQualifiedAttribute());
        }

        // In caso la validazione abbia esito negativo restituiamo null
        return null;
    }
}
