package com.accenture.eVote.elettorims.entity;

import jakarta.persistence.*;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.sql.Date;


@Entity
@Table(name = "CL_ELEVO_ELETTOREEVOTE")
@Data
@NoArgsConstructor
@AllArgsConstructor
public class Elettore {

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "SEQ_ELEVO_USERID")
    @SequenceGenerator(name = "SEQ_ELEVO_USERID", sequenceName = "SEQ_ELEVO_USERID", allocationSize = 1)
    @Column(name = "ELEVO_USER_ID")
    private Long userId;

    @Column(name = "ELEVO_USERNAME")
    private String username;

    @Column(name = "ELEVO_COGNOME")
    private String cognome;

    @Column(name = "ELEVO_NOME")
    private String nome;

    @Column(name = "ELEVO_DATA_NASCITA")
    private Date dataNascita;

    @Column(name = "ELEVO_LUOGO_NASCITA")
    private String luogoNascita;

    @Column(name = "ELEVO_LUOGO_RESIDENZA")
    private String luogoResidenza;

    @Column(name = "ELEVO_INDIRIZZO_RESIDENZA")
    private String indirizzoResidenza;

    @Column(name = "ELEVO_STATO_ESTERO_RESIDENZA")
    private String statoEsteroResidenza;

    @Column(name = "ELEVO_CODICE_FISCALE")
    private String codiceFiscale;

    @Column(name = "ELEVO_EMAIL")
    private String email;

    @Column(name = "ELEVO_TELEFONO")
    private String telefono;

    @Column(name = "ELEVO_FLAG_VALIDO")
    private Long flagValido;

    @Column(name = "ELEVO_TS_CREAZIONE")
    private Date tsCreazione;

    @Column(name = "ELEVO_TS_MODIFICA")
    private Date tsModifica;

    @Column(name = "ELEVO_TIPCO_TIPODOCUMENTO", insertable = false, updatable = false)
    private Long idTipoDocumento;

    @Column(name = "ELEVO_NUM_DOCUMENTO")
    private Long numDocumento;

    @Column(name = "ELEVO_DATA_SCADENZA_DOCUMENTO")
    private Date dataScadenzaDoc;

    //@Column(name = "ELEVO_METAU_METODOAUTENTICAZIONE")
    //private Long idMetodoAutenticazione;

    @Column(name = "ELEVO_EMAIL_PEC")
    private String emailPec;

    @Column(name = "ELEVO_SESSO")
    private String sesso;

    @ManyToOne
    @JoinColumn(name = "ELEVO_TIPCO_TIPODOCUMENTO")
    private TipoDocumento tipoDocumento;

    @ManyToOne
    @JoinColumn(name = "ELEVO_METAU_METODOAUTENTICAZIONE")
    private MetodoAutenticazione metodoAutenticazione;


    public void overrideAttributiNonNulli(Elettore elettore) {
        if (elettore.getUsername() != null && !elettore.getUsername().equals("-")) {
            this.username = elettore.getUsername();
        }
        if (elettore.getCognome() != null && !elettore.getCognome().equals("-")) {
            this.cognome = elettore.getCognome();
        }
        if (elettore.getNome() != null && !elettore.getNome().equals("-")) {
            this.nome = elettore.getNome();
        }
        if (elettore.getDataNascita() != null) {
            this.dataNascita = elettore.getDataNascita();
        }
        if (elettore.getLuogoNascita() != null && !elettore.getLuogoNascita().equals("-")) {
            this.luogoNascita = elettore.getLuogoNascita();
        }
        if (elettore.getLuogoResidenza() != null && !elettore.getLuogoResidenza().equals("-")) {
            this.luogoResidenza = elettore.getLuogoResidenza();
        }
        if (elettore.getIndirizzoResidenza() != null && !elettore.getIndirizzoResidenza().equals("-")) {
            this.indirizzoResidenza = elettore.getIndirizzoResidenza();
        }
        if (elettore.getStatoEsteroResidenza() != null && !elettore.getStatoEsteroResidenza().equals("-")) {
            this.statoEsteroResidenza = elettore.getStatoEsteroResidenza();
        }
        if (elettore.getCodiceFiscale() != null && !elettore.getCodiceFiscale().equals("-")) {
            this.codiceFiscale = elettore.getCodiceFiscale();
        }
        if (elettore.getEmail() != null && !elettore.getEmail().equals("-")) {
            this.email = elettore.getEmail();
        }
        if (elettore.getTelefono() != null && !elettore.getTelefono().equals("-")) {
            this.telefono = elettore.getTelefono();
        }
        if (elettore.getFlagValido() != null) {
            this.flagValido = elettore.getFlagValido();
        }
        if (elettore.getTsCreazione() != null) {
            this.tsCreazione = elettore.getTsCreazione();
        }
        if (elettore.getTsModifica() != null) {
            this.tsModifica = elettore.getTsModifica();
        }
        if (elettore.getIdTipoDocumento() != null) {
            this.idTipoDocumento = elettore.getIdTipoDocumento();
        }
        if (elettore.getNumDocumento() != null) {
            this.numDocumento = elettore.getNumDocumento();
        }
        if (elettore.getDataScadenzaDoc() != null) {
            this.dataScadenzaDoc = elettore.getDataScadenzaDoc();
        }
        if (elettore.getEmailPec() != null && !elettore.getEmailPec().equals("-")) {
            this.emailPec = elettore.getEmailPec();
        }
        if (elettore.getSesso() != null && !elettore.getSesso().equals("-")) {
            this.sesso = elettore.getSesso();
        }
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null || getClass() != obj.getClass()) {
            return false;
        }

        Elettore altroElettore = (Elettore) obj; // Esegue il casting all'oggetto Entity

        // Confronta i campi della classe per determinare l'uguaglianza
        if (!nome.equals(altroElettore.nome)) {
            return false;
        }
        if (!cognome.equals(altroElettore.cognome)) {
            return false;
        }
        if (!dataNascita.equals(altroElettore.dataNascita)) {
            return false;
        }
        if (!luogoNascita.equals(altroElettore.luogoNascita)) {
            return false;
        }
        if (!luogoResidenza.equals(altroElettore.luogoResidenza)) {
            return false;
        }
        if (!indirizzoResidenza.equals(altroElettore.indirizzoResidenza)) {
            return false;
        }
        if (!statoEsteroResidenza.equals(altroElettore.statoEsteroResidenza)) {
            return false;
        }
        if (!codiceFiscale.equals(altroElettore.codiceFiscale)) {
            return false;
        }
        if (!email.equals(altroElettore.email)) {
            return false;
        }
        if (!telefono.equals(altroElettore.telefono)) {
            return false;
        }
        if (!flagValido.equals(altroElettore.flagValido)) {
            return false;
        }
        if (!numDocumento.equals(altroElettore.numDocumento)) {
            return false;
        }
        if (!dataScadenzaDoc.equals(altroElettore.dataScadenzaDoc)) {
            return false;
        }
        //if (!idMetodoAutenticazione.equals(altroElettore.idMetodoAutenticazione)) {return false;}
        if (!emailPec.equals(altroElettore.emailPec)) {
            return false;
        }
        if (!sesso.equals(altroElettore.sesso)) {
            return false;
        }

        return true;
    }
}
