package com.accenture.eVote.elettorims.mapper;

import com.accenture.eVote.elettorims.dto.ElettoreDTO;
import com.accenture.eVote.elettorims.entity.Elettore;
import com.accenture.eVote.elettorims.entity.ElettoreMAECI;
import com.accenture.eVote.elettorims.model.response.LoginSpidResponse;
import com.accenture.eVote.elettorims.util.Crittografia;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.mapstruct.Mapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import java.sql.Date;
import java.text.ParseException;
import java.text.SimpleDateFormat;

@Mapper(componentModel = "Spring")
public class LoginCnsMapper {

    @Autowired
    private Crittografia crittografia;
    @Autowired
    private ElettoreMapper elettoreMapper;
    @Value("${formati.maeci.data.nascita}")
    private String formatoMaeci;
    @Value("${formati.request.data}")
    private String formatoLogin;

    public void integrazioneCns(ElettoreDTO elettoreDTO, String jsonCns) {

        SimpleDateFormat sdf = new SimpleDateFormat(formatoLogin);

        // Decodifica e parsing del jsonCns
        String jsonCnsDec = crittografia.decodificaJsonCns(jsonCns);

        ObjectMapper objectMapper = new ObjectMapper();

        try {
            JsonNode jsonNode = objectMapper.readTree(jsonCnsDec);

            elettoreDTO.setNome(jsonNode.get("SPID_name").toPrettyString().replaceAll("\"", ""));
            elettoreDTO.setCognome(jsonNode.get("SPID_familyName").toPrettyString().replaceAll("\"", ""));
            elettoreDTO.setDataNascita(new Date(sdf.parse(jsonNode.get("SPID_dateOfBirth").toPrettyString().replaceAll("\"", "")).getTime()));
            elettoreDTO.setLuogoNascita(jsonNode.get("SPID_placeOfBirth").toPrettyString().replaceAll("\"", ""));
            elettoreDTO.setSesso(jsonNode.get("SPID_gender").toPrettyString().replaceAll("\"", ""));
            elettoreDTO.setEmailPec(jsonNode.get("SPID_digitalAddress").toPrettyString().replaceAll("\"", ""));

        } catch (JsonMappingException e) {
            e.printStackTrace();
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        } catch (ParseException e) {
            e.printStackTrace();
        }

    }

    public void integrazioneLoginSpidResponse(LoginSpidResponse loginSpidResponse, Elettore elettore){

        SimpleDateFormat dateFormat = new SimpleDateFormat(formatoLogin);

        ElettoreDTO elettoreDTO = crittografia.decodificaDatiSensibili(elettoreMapper.toDto(elettore));

        loginSpidResponse.setName(elettoreDTO.getNome());
        loginSpidResponse.setFamilyName(elettoreDTO.getCognome());
        loginSpidResponse.setGender(elettoreDTO.getSesso());
        loginSpidResponse.setDateOfBirth(dateFormat.format(elettoreDTO.getDataNascita()));
        loginSpidResponse.setPlaceOfBirth(elettoreDTO.getLuogoNascita());
        loginSpidResponse.setDigitalAddress(elettoreDTO.getEmailPec());

    }

    public void integrazioneLoginSpidResponse(LoginSpidResponse loginSpidResponse, ElettoreMAECI elettoreMAECI) {

        SimpleDateFormat sdfMaeci = new SimpleDateFormat(this.formatoMaeci);
        SimpleDateFormat sdfLogin = new SimpleDateFormat(this.formatoLogin);

        loginSpidResponse.setName(elettoreMAECI.getNome());
        loginSpidResponse.setFamilyName(elettoreMAECI.getCognome());
        loginSpidResponse.setGender(elettoreMAECI.getCodiceSesso());
        loginSpidResponse.setPlaceOfBirth(elettoreMAECI.getLuogoNascita());
        loginSpidResponse.setDigitalAddress(null); // mail non presente nella tabella del MAECI

        try {
            loginSpidResponse.setDateOfBirth(sdfLogin.format(sdfMaeci.parse(elettoreMAECI.getDataNascita())));
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }
}
