package com.accenture.eVote.elettorims.mapper;

import com.accenture.eVote.elettorims.dto.ElettoreDTO;
import com.accenture.eVote.elettorims.entity.ElettoreMAECI;
import com.accenture.eVote.elettorims.model.response.LoginSpidResponse;
import org.mapstruct.Mapper;
import org.springframework.beans.factory.annotation.Value;

import java.text.ParseException;
import java.text.SimpleDateFormat;

@Mapper(componentModel = "Spring")
public class LoginResponseMapper {

    @Value("${formati.maeci.data.nascita}")
    private String formatoMaeci;
    @Value("${formati.request.data}")
    private String formatoLogin;

    public ElettoreDTO toElettoreDTO(LoginSpidResponse loginSpidResponse) {

        SimpleDateFormat sdf = new SimpleDateFormat(formatoLogin);
        ElettoreDTO elettoreDTO = new ElettoreDTO();

        try {

            elettoreDTO.setNome(loginSpidResponse.getName());
            elettoreDTO.setCognome(loginSpidResponse.getFamilyName());
            elettoreDTO.setLuogoNascita(loginSpidResponse.getPlaceOfBirth());
            elettoreDTO.setDataNascita(sdf.parse(loginSpidResponse.getDateOfBirth()));
            elettoreDTO.setSesso(loginSpidResponse.getGender());
            elettoreDTO.setCodiceFiscale(loginSpidResponse.getFiscalNumber());
            elettoreDTO.setTelefono(loginSpidResponse.getMobilePhone());
            elettoreDTO.setEmailPec(loginSpidResponse.getDigitalAddress());

            return elettoreDTO;

        } catch (ParseException e) {
            e.printStackTrace();
            return null;
        }

    }

    public ElettoreDTO toElettoreDTO(ElettoreMAECI elettoreMAECI) {

        SimpleDateFormat sdf = new SimpleDateFormat(formatoMaeci);
        ElettoreDTO elettoreDTO = new ElettoreDTO();
        try {
            elettoreDTO.setNome(elettoreMAECI.getNome());
            elettoreDTO.setCognome(elettoreMAECI.getCognome());
            elettoreDTO.setLuogoNascita(elettoreMAECI.getLuogoNascita());
            elettoreDTO.setDataNascita(sdf.parse(elettoreMAECI.getDataNascita()));
            elettoreDTO.setCodiceFiscale(elettoreMAECI.getCodiceFiscale());
            elettoreDTO.setLuogoResidenza(elettoreMAECI.getLocalita());
            elettoreDTO.setIndirizzoResidenza(elettoreMAECI.getIndirizzo());
            elettoreDTO.setStatoEsteroResidenza(elettoreMAECI.getCodiceStatoEsteroResidenza());

            if (elettoreMAECI.getCodiceSesso().equals("1")) {
                elettoreDTO.setSesso("M");
            } else if (elettoreMAECI.getCodiceSesso().equals("2")) {
                elettoreDTO.setSesso("F");
            }

            return elettoreDTO;

        } catch (ParseException e) {
            e.printStackTrace();
            return null;
        }

    }
}
