package com.accenture.eVote.elettorims.service;

import com.accenture.eVote.elettorims.dto.ConfTempoElettoraleDTO;
import com.accenture.eVote.elettorims.dto.ConfTempoVotoDTO;
import com.accenture.eVote.elettorims.dto.ConsensoVotoDTO;
import com.accenture.eVote.elettorims.dto.FinestraTemporaleAttiva;
import com.accenture.eVote.elettorims.entity.ConfTempoElettorale;
import com.accenture.eVote.elettorims.entity.ConsensoVoto;
import com.accenture.eVote.elettorims.entity.Elezioni;
import com.accenture.eVote.elettorims.mapper.ConfTempoElettoraleMapper;
import com.accenture.eVote.elettorims.mapper.ConsensoVotoMapper;
import com.accenture.eVote.elettorims.model.request.ConsensoVotoRequest;
import com.accenture.eVote.elettorims.repository.ConfTempoElettoraleRepository;
import com.accenture.eVote.elettorims.repository.ConsensoVotoRepository;
import com.accenture.eVote.elettorims.repository.DateElezioniRepository;
import com.accenture.eVote.elettorims.repository.ElezioniRepository;
import com.accenture.eVote.elettorims.util.enums.ElettoriEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Date;
import java.sql.Timestamp;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class ConsensoVotoService {
    @Autowired
    private ConsensoVotoRepository consensoVotoRepository;
    @Autowired
    private DateElezioniRepository dateElezioniRepository;

    @Autowired
    private ConfTempoElettoraleRepository confTempoElettoraleRepository;

    @Autowired
    private ElezioniRepository elezioniRepository;

    @Autowired
    public ConsensoVotoMapper consensoVotoMapper;

    @Autowired
    public ConfTempoElettoraleMapper confTempoElettoraleMapper;


    public ConsensoVotoDTO creaModificaConsenso(ConsensoVotoRequest consensoVotoRequest) {
        Optional<ConsensoVoto> consensoVoto = consensoVotoRepository.findById(consensoVotoRequest.getUserId());

        FinestraTemporaleAttiva finestraTemporaleAttiva = finestraTemporaleAttiva(ElettoriEnum.FINESTRA_CONSENSO.getValore());

        ZoneId zonaOrariaItalia = ZoneId.of("Europe/Rome");
        ZonedDateTime orarioCorrenteItalia = ZonedDateTime.now(zonaOrariaItalia).withNano(0);
        Timestamp timestamp = Timestamp.from(orarioCorrenteItalia.toInstant().minusSeconds(60));

        if (finestraTemporaleAttiva.getAttiva()) {
            if (consensoVoto.isPresent()) {
                consensoVoto.get().setConsensoAttivo(consensoVotoRequest.getFlagConsenso());
                consensoVotoRepository.save(consensoVoto.get());
                return consensoVotoMapper.toDto(consensoVoto.get());
            } else {
                ConsensoVoto consensoVoto1 = new ConsensoVoto();
                consensoVoto1.setUserId(consensoVotoRequest.getUserId());
                consensoVoto1.setConsensoAttivo(consensoVotoRequest.getFlagConsenso());
                consensoVoto1.setDataAggiornamento(timestamp);
                consensoVoto1.setIdDataElezione(finestraTemporaleAttiva.getIdDataElezione());
                consensoVotoRepository.save(consensoVoto1);
                return consensoVotoMapper.toDto(consensoVoto1);
            }
        }
        return null;
    }

    public ConsensoVotoDTO getFinestraConsensoVoto(Long userId) {
        Optional<ConsensoVoto> consensoVoto = consensoVotoRepository.findById(userId);
        FinestraTemporaleAttiva finestraTemporaleAttiva = finestraTemporaleAttiva(ElettoriEnum.FINESTRA_CONSENSO.getValore());

        //Settiamo la data inzio e fine elezione anche se non si trova il consenso voto per l'utente
        ConsensoVotoDTO consensoVotoDTO = new ConsensoVotoDTO();
        consensoVotoDTO.setDataInizio(finestraTemporaleAttiva.getDataInizio());
        consensoVotoDTO.setDataFine(finestraTemporaleAttiva.getDataFine());
        consensoVotoDTO.setIdDataElezione(finestraTemporaleAttiva.getIdDataElezione());

        if (consensoVoto.isPresent()) {
            consensoVotoDTO = consensoVotoMapper.toDto(consensoVoto.get());
            consensoVotoDTO.setDataInizio(finestraTemporaleAttiva.getDataInizio());
            consensoVotoDTO.setDataFine(finestraTemporaleAttiva.getDataFine());
            List<ConfTempoElettorale> confTempoElettorale = confTempoElettoraleRepository
                    .findByIdDataElezioneAndTipoFinestraTemporale(consensoVotoDTO.getIdDataElezione(), ElettoriEnum.FINESTRA_CONSENSO.getValore());
            return consensoVotoDTO;
        }
        return consensoVotoDTO;
    }

    private FinestraTemporaleAttiva finestraTemporaleAttiva(Long tipoFinestraTemporale) {
        Boolean attivo = Boolean.FALSE;
        Long idElezione = null;
        Long idDataElezione = null;
        ConfTempoElettorale confTempoElettorale = null;

        List<Elezioni> listElezioni = elezioniRepository.findAll();

        //filtro per codice fase elettorale
        List<Elezioni> listaFiltrata = listElezioni.stream().filter(e -> e.getCodFaseElettorale() != 4).collect(Collectors.toList());

        if (!listaFiltrata.isEmpty()) {
            idElezione = listaFiltrata.get(0).getIdElezione();
            idDataElezione = listaFiltrata.get(0).getIdDataElezione();

            //CONDIZIONE tipo finestra temporale (consenso || voto)
            if (Objects.equals(tipoFinestraTemporale, ElettoriEnum.FINESTRA_CONSENSO.getValore())) {
                confTempoElettorale = confTempoElettoraleRepository.findByElezioneAndTipoFinestraTemporale(idElezione, ElettoriEnum.FINESTRA_CONSENSO.getValore());
            } else {
                confTempoElettorale = confTempoElettoraleRepository.findByElezioneAndTipoFinestraTemporale(idElezione, ElettoriEnum.FINESTRA_VOTO.getValore());
            }

            Calendar calendar = new GregorianCalendar();
            Date dataNow = new Date(calendar.getTime().getTime());

            if (confTempoElettorale != null && dataNow.after(confTempoElettorale.getDataInizio()) && dataNow.before(confTempoElettorale.getDataFine())) {
                attivo = Boolean.TRUE;
            }

            return new FinestraTemporaleAttiva(attivo, idDataElezione, idElezione, (confTempoElettorale == null) ? null : confTempoElettorale.getDataInizio(),
                    (confTempoElettorale == null) ? null : confTempoElettorale.getDataFine());
        }

        return new FinestraTemporaleAttiva(attivo, null, null, null, null);
    }

    public ConfTempoVotoDTO getFinestraVoto(Long userId) {
        FinestraTemporaleAttiva finestraTemporaleAttiva = finestraTemporaleAttiva(ElettoriEnum.FINESTRA_VOTO.getValore());
        ConfTempoElettorale confTempoElettorale = confTempoElettoraleRepository.findByElezioneAndTipoFinestraTemporale(finestraTemporaleAttiva.getIdElezione(), ElettoriEnum.FINESTRA_VOTO.getValore());
        if (confTempoElettorale != null) {
            ConfTempoElettoraleDTO confDTO = confTempoElettoraleMapper.toDto((confTempoElettorale));
            if (finestraTemporaleAttiva.getAttiva()) {
                confDTO.setFinestraAttiva(1L);
            } else {
                confDTO.setFinestraAttiva(0L);
            }
            return new ConfTempoVotoDTO(confDTO, finestraTemporaleAttiva.getIdDataElezione(),
                    dateElezioniRepository.findByDataElezioneId(finestraTemporaleAttiva.getIdDataElezione()).getDataElezione(),
                    elezioniRepository.findById(confTempoElettorale.getElezione()).get().getCodFaseElettorale());
        }
        return new ConfTempoVotoDTO();
    }
}