package com.accenture.eVote.elettorims.service;

import com.accenture.eVote.elettorims.dto.ElettoreDTO;
import com.accenture.eVote.elettorims.entity.*;
import com.accenture.eVote.elettorims.mapper.ElettoreMapper;
import com.accenture.eVote.elettorims.mapper.LoginCnsMapper;
import com.accenture.eVote.elettorims.mapper.LoginResponseMapper;
import com.accenture.eVote.elettorims.mapper.MetodoAutenticazioneMapper;
import com.accenture.eVote.elettorims.model.request.AutocertificazioneRequest;
import com.accenture.eVote.elettorims.model.response.AutocertificazioneResponse;
import com.accenture.eVote.elettorims.model.response.LoginResponse;
import com.accenture.eVote.elettorims.model.response.LoginSpidResponse;
import com.accenture.eVote.elettorims.repository.AutocertificazioneRepository;
import com.accenture.eVote.elettorims.repository.ElettoreEnteRepository;
import com.accenture.eVote.elettorims.repository.ElettoreMaeciRepository;
import com.accenture.eVote.elettorims.repository.ElettoreRepository;
import com.accenture.eVote.elettorims.util.CodiceFiscaleDecoder;
import com.accenture.eVote.elettorims.util.Crittografia;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Service;

import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.sql.Date;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;

@Service
@Configuration
public class LoginService {

    @Autowired
    private ElettoreRepository elettoreRepository;
    @Autowired
    private ElettoreMapper elettoreMapper;
    @Autowired
    private LoginResponseMapper loginResponseMapper;
    @Autowired
    private Crittografia crittografia;
    @Autowired
    private ElettoreMaeciRepository elettoreMaeciRepository;
    @Autowired
    private MetodoAutenticazioneMapper metodoAutenticazioneMapper;
    @Autowired
    private AutocertificazioneRepository autocertificazioneRepository;
    @Autowired
    private CodiceFiscaleDecoder codiceFiscaleDecoder;
    @Autowired
    private LoginCnsMapper loginCnsMapper;
    @Value("${formati.request.data}")
    private String formatoData;
    @Value("${formati.maeci.data.nascita}")
    private String formatoMaeci;

    @Autowired
    private ElettoreEnteRepository elettoreEnteRepository;

    public LoginResponse login(String token, LoginSpidResponse loginSpidResponse) {

        LoginResponse loginResponse = new LoginResponse();
        ElettoreDTO elettoreDTO = new ElettoreDTO();
        Boolean utenteRegistrato = checkElettoreRegistrato(loginSpidResponse);
        Boolean utenteMaeci = (findElettoreMaeci(loginSpidResponse) != null);
        Boolean utenteCNS = loginSpidResponse.getIdp().equals("CNS");

        // verifica presenza MAECI o utente già registrato e idp diverso da CNS
        if ((utenteRegistrato || utenteMaeci) && !utenteCNS) {
            String geoIdConsolato = null;
            // Salvataggio nuovo elettore o aggiornamento esistente
            elettoreDTO = loginResponseMapper.toElettoreDTO(loginSpidResponse);

            // se utente MAECI recupero il geoIdConsolato e altri dati
            if (utenteMaeci) {
                ElettoreMAECI elettoreMAECI = findElettoreMaeci(loginSpidResponse);
                geoIdConsolato = elettoreMAECI.getGeoIdConsolato();
                elettoreDTO.setLuogoNascita(elettoreMAECI.getLuogoNascita());
                elettoreDTO.setLuogoResidenza(elettoreMAECI.getLocalita());
                elettoreDTO.setIndirizzoResidenza(elettoreMAECI.getIndirizzo());
                elettoreDTO.setStatoEsteroResidenza(elettoreMAECI.getCodiceStatoEsteroResidenza());
                if (elettoreMAECI.getCodiceSesso().equals("1")) {
                    elettoreDTO.setSesso("M");
                } else if (elettoreMAECI.getCodiceSesso().equals("2")) {
                    elettoreDTO.setSesso("F");
                }
            }
            // Salvataggio/aggiornamento elettore
            elettoreDTO = salvaAggiornaElettore(elettoreDTO, geoIdConsolato);

            loginResponse.setCns(false);
            loginResponse.setTemporaneo(false);
            // setting userID su risposta
            loginResponse.setUserSessionId(String.valueOf(elettoreDTO.getUserId()));

            // Se utente registrato e CNS non aggiorno i dati in quanto arriva esclusivamente il codice fiscale
        } else if (utenteRegistrato && utenteCNS) {

            // recupero dei dati a db registrati in fase di autocertificazione e mapping su loginSpidResponse
            loginResponse.setUserSessionId(recuperoDatiElettore(loginSpidResponse));
            loginResponse.setCns(false);
            loginResponse.setTemporaneo(false);

            // Se utente CNS non registrato e presente su Maeci, salvo l'elettore con idati del MAECI
        } else if (utenteCNS && !utenteRegistrato && utenteMaeci) {
            // recupero dati Maeci
            ElettoreMAECI elettoreMAECI = findElettoreMaeci(loginSpidResponse);
            elettoreDTO = loginResponseMapper.toElettoreDTO(elettoreMAECI);
            this.loginCnsMapper.integrazioneLoginSpidResponse(loginSpidResponse, elettoreMAECI);

            // Salvataggio/aggiornamento elettore
            elettoreDTO = salvaAggiornaElettore(elettoreDTO, elettoreMAECI.getGeoIdConsolato());

            // setting userID su risposta
            loginResponse.setUserSessionId(String.valueOf(elettoreDTO.getUserId()));
            loginResponse.setCns(false);
            loginResponse.setTemporaneo(false);

            // Se utente CNS non registrato e non presente su Maeci, setting per autocertificazione CNS
        } else if (utenteCNS && !utenteRegistrato && !utenteMaeci) {
            loginResponse.setTemporaneo(true);
            loginResponse.setCns(true);
            // Se utente non CNS non registrato e non presente su Maeci, setting per autocertificazione
        } else {
            loginResponse.setTemporaneo(true);
            loginResponse.setCns(false);
        }

        // caricamento token autotenticazione (codificato) e token dati
        loginResponse.setJwtTokenAuth(crittografia.codificaTokenAuth(token));
        loginResponse.setJwtTokenData(crittografia.generaTokenDati(loginSpidResponse));

        // caricamento user session id criptato
        loginResponse.setUserSessionId(crittografia.codificaUserSessionId(loginResponse.getUserSessionId()));

        return loginResponse;
    }

    public boolean verificaMaggioreEta(LoginSpidResponse loginSpidResponse) {

        DateTimeFormatter formato = DateTimeFormatter.ofPattern(formatoData);
        LocalDate dataNascita = null;

        if (loginSpidResponse.getIdp().equals("CNS")) {
            dataNascita = codiceFiscaleDecoder.getDataNascita(loginSpidResponse.getFiscalNumber());
        } else {
            dataNascita = LocalDate.parse(loginSpidResponse.getDateOfBirth(), formato);
        }

        return Period.between(dataNascita, LocalDate.now()).getYears() >= 18;

    }

    public AutocertificazioneResponse autocertificazione(AutocertificazioneRequest autocertificazioneRequest, String jwtTokenData) {

        AutocertificazioneResponse autocertificazioneResponse = new AutocertificazioneResponse();
        // decofica token dati su elettoreDTO
        ElettoreDTO elettoreDTO = crittografia.tokenDataToElettore(jwtTokenData);
        // Verifica se autenticazione CNS
        if (autocertificazioneRequest.getJsonCNS() != null) {
            // recupero i dati dell'elettore dal dal jsonCNS
            loginCnsMapper.integrazioneCns(elettoreDTO, autocertificazioneRequest.getJsonCNS());
        }

        try {
            elettoreDTO = crittografia.codificaDatiSensibili(elettoreDTO);
            elettoreDTO = aggiornamentoElettore(elettoreDTO, 1L, autocertificazioneRequest.getGeoIdConsolato());

            // persistenza dell'autocertificazione

            Autocertificazione autocertificazione = new Autocertificazione();
            autocertificazione.setIdTipo(autocertificazioneRequest.getCausaleEstero());
            autocertificazione.setDescrizione(autocertificazioneRequest.getDescrizioneCausaleEstero());
            autocertificazione.setId(elettoreDTO.getUserId());

            autocertificazioneRepository.save(autocertificazione);

            autocertificazioneResponse.setUserSessionId(crittografia.codificaUserSessionId(String.valueOf(elettoreDTO.getUserId())));

        } catch (Exception e) {
            e.printStackTrace();
            autocertificazioneResponse.setMessaggio("Errore");
            return autocertificazioneResponse;
        }
        autocertificazioneResponse.setMessaggio("Utente censito");
        return autocertificazioneResponse;
    }

    public String codificaTokenAuth(String token) {
        // Converto in Base64 per evitare i caratteri speciali
        return Base64.getEncoder().encodeToString(crittografia.codificaTokenAuth(token).getBytes());
    }

    public String decodificaTokenAuth(String token) {
        return crittografia.decodificaTokenAuth(token);
    }


    private ElettoreDTO aggiornamentoElettore(ElettoreDTO elettoreDTO, Long metodoAutenticazione, String geoIdConsolato) throws NoSuchPaddingException, IllegalBlockSizeException, NoSuchAlgorithmException, BadPaddingException, InvalidKeyException {

        Elettore elettoreDb = new Elettore();

        ZoneId zonaOrariaItalia = ZoneId.of("Europe/Rome");
        ZonedDateTime orarioCorrenteItalia = ZonedDateTime.now(zonaOrariaItalia).withNano(0);
        java.sql.Timestamp timestamp = Timestamp.from(orarioCorrenteItalia.toInstant().minusSeconds(60));
        ZonedDateTime orarioCorrenteItaliaAddYear = orarioCorrenteItalia.plusYears(1);
        java.sql.Timestamp timestampAddYear = Timestamp.from(orarioCorrenteItaliaAddYear.toInstant().minusSeconds(60));

        // Recupero elettore da codice fiscale se non popolato utilizzo nome, cognome, data nascita e luogo nascita
        if (elettoreDTO.getCodiceFiscale() != null) {
            elettoreDb = elettoreRepository.findByCodiceFiscale(elettoreDTO.getCodiceFiscale());
        } else {
            elettoreDb = elettoreRepository.findByNomeAndCognomeAndDataNascita(
                    elettoreDTO.getNome(),
                    elettoreDTO.getCognome(),
                    elettoreDTO.getDataNascita()).get(0);
        }


        // Verfica e aggioranmento/inserimento
        if (elettoreDb != null && !elettoreDb.equals(elettoreDTO)) {

            elettoreDb.overrideAttributiNonNulli(elettoreMapper.toEntity(elettoreDTO));
            // scrittura data modifica
            elettoreDb.setTsModifica(Date.valueOf(LocalDateTime.now().toLocalDate()));
            elettoreDb.setMetodoAutenticazione(new MetodoAutenticazione(metodoAutenticazione));

            elettoreDTO = elettoreMapper.toDto(elettoreRepository.save(elettoreDb));

        } else if (elettoreDb == null) {
            // primo inserimento
            // generazione username da concatenazione nome e cognome
            elettoreDTO.setMetodoAutenticazione(metodoAutenticazioneMapper.toDto(new MetodoAutenticazione(metodoAutenticazione)));
            setCostantiElettore(elettoreDTO);
            elettoreDTO = elettoreMapper.toDto(elettoreRepository.save(elettoreMapper.toEntity(elettoreDTO)));

            ElettoreEnte elettoreEnte = new ElettoreEnte();
            elettoreEnte.setIdElettore(elettoreDTO.getUserId());
            elettoreEnte.setGeoId(geoIdConsolato);
            elettoreEnte.setDataInizioValidita(Date.valueOf(orarioCorrenteItalia.toLocalDate()));
            elettoreEnte.setDataFineValidita(Date.valueOf(orarioCorrenteItaliaAddYear.toLocalDate()));
            elettoreEnteRepository.save(elettoreEnte);

        }

        return elettoreDTO;
    }


    //Se non ci sono records in EXT_ELEMA_ELETTORIMAECI, non può entrare
    //Se è presente il codice fiscale su spid,cns,ecc ecc verifco che sia presente all'interno
    //del file del MAECI il CF. Se è presente login ok.
    //Qualora sul file non fosse presente facciamo il check sui seguenti campi :
    // nome, cognome, data nascita, luogo di nascita.

    private ElettoreMAECI findElettoreMaeci(LoginSpidResponse loginSpidResponse) {

        // recupero elettore per codice fiscale
        List<ElettoreMAECI> listElettoriMaeci = new ArrayList<>();

        listElettoriMaeci.addAll(elettoreMaeciRepository.findByCodiceFiscale(loginSpidResponse.getFiscalNumber()));

        if (loginSpidResponse.getName() != null &&
                loginSpidResponse.getFamilyName() != null &&
                loginSpidResponse.getDateOfBirth() != null) {

            listElettoriMaeci.addAll(elettoreMaeciRepository.findByNomeAndCognomeAndDataNascita(
                    loginSpidResponse.getName(),
                    loginSpidResponse.getFamilyName(),
                    convertiDataFormatoMaeci(loginSpidResponse.getDateOfBirth())));
        }

        if (listElettoriMaeci.isEmpty()) {
            return null;
        } else {
            if (userExists(loginSpidResponse, listElettoriMaeci)) {
                return listElettoriMaeci.get(0); // recupero il primo della lista
            } else {
                return null;
            }
        }
    }

    private String convertiDataFormatoMaeci(String dataInput) {

        // Parsa la stringa nel formato yyyy-MM-dd
        LocalDate data = LocalDate.parse(dataInput, DateTimeFormatter.ofPattern(formatoData));

        // Formatta la data nel nuovo formato yyyyMMdd
        String dataOutput = data.format(DateTimeFormatter.ofPattern(formatoMaeci));

        return dataOutput;
    }

    private Boolean checkElettoreRegistrato(LoginSpidResponse loginSpidResponse) {

        SimpleDateFormat dateFormat = new SimpleDateFormat(formatoData);

        // verifica presenza elettore da codice fiscale
        try {
            Elettore elettore = new Elettore();
            // verifico che il codice fiscale sia presente
            if (loginSpidResponse.getFiscalNumber() != null) {
                elettore = elettoreRepository.findByCodiceFiscale(crittografia.codificaCf(loginSpidResponse.getFiscalNumber()));
            } else {
                List<Elettore> elettori = elettoreRepository.findByNomeAndCognomeAndDataNascita(
                        crittografia.codificaCf(loginSpidResponse.getName()),
                        crittografia.codificaCf(loginSpidResponse.getFamilyName()),
                        dateFormat.parse(loginSpidResponse.getDateOfBirth()));
                // carico il primo
                elettore = elettori.get(0);

            }

            if (elettore == null) {
                return Boolean.FALSE;
            } else {
                return Boolean.TRUE;
            }
        } catch (Exception e) {
            e.printStackTrace();
            // in caso di errore nel recupero l'elettore risulterà non presente
            return Boolean.FALSE;
        }

    }

    private Boolean userExists(LoginSpidResponse loginSpidResponse, List<ElettoreMAECI> listElettoriMaeci) {

        // Verifico che nella lista elettori MAECI siano sempre presenti i codici fiscali

        boolean verificaCfNulli = listElettoriMaeci.stream()
                .allMatch(elettoreMAECI -> elettoreMAECI.getCodiceFiscale() != null && !elettoreMAECI.getCodiceFiscale().isEmpty());

        if (loginSpidResponse.getFiscalNumber() != null && verificaCfNulli) {

            Boolean esitoCf = listElettoriMaeci.stream()
                    .anyMatch(e -> (e.getCodiceFiscale().equals(loginSpidResponse.getFiscalNumber())));

            return esitoCf;
        } else {

            Boolean esitoMisto = listElettoriMaeci.stream().anyMatch(e ->
                    (e.getNome().equalsIgnoreCase(loginSpidResponse.getName())
                            && e.getCognome().equalsIgnoreCase(loginSpidResponse.getFamilyName())
                            && e.getDataNascita().equals(convertiDataFormatoMaeci(loginSpidResponse.getDateOfBirth()))));

            return esitoMisto;
        }
    }

    private void setCostantiElettore(ElettoreDTO elettoreDTO) {
        elettoreDTO.setFlagValido(1L);
        elettoreDTO.setTsCreazione(Date.valueOf(LocalDateTime.now().toLocalDate()));
    }

    private ElettoreDTO salvaAggiornaElettore(ElettoreDTO elettoreDTO, String geoIdConsolato) {
        try {
            elettoreDTO = crittografia.codificaDatiSensibili(elettoreDTO);
            return aggiornamentoElettore(elettoreDTO, 1L, geoIdConsolato);

        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    private String recuperoDatiElettore(LoginSpidResponse loginSpidResponse) {

        // Recupero dati persistiti in fase di autocertificaizone
        Elettore elettore = elettoreRepository.findByCodiceFiscale(crittografia.codificaCf(loginSpidResponse.getFiscalNumber()));
        // Mapping dei dati in chiaro
        loginCnsMapper.integrazioneLoginSpidResponse(loginSpidResponse, elettore);

        return String.valueOf(elettore.getUserId());
    }

    public boolean verificaDirittoAlVoto(LoginSpidResponse loginSpidResponse) {

        List<ElettoreMAECI> elettoreMAECI = elettoreMaeciRepository.findByCodiceFiscale(loginSpidResponse.getFiscalNumber());

        // verifica del flg dirito al voto se non presente nel MAECI ha diritto al voto
        if (elettoreMAECI.isEmpty() || elettoreMAECI.get(0).getFlagDirittoVoto().equals("S")) {
            return true;
        } else {
            return false;
        }
    }

}
