package com.accenture.eVote.elettorims.service;

import com.accenture.eVote.elettorims.model.request.OracleRequest;
import com.accenture.eVote.elettorims.model.response.OracleGenerateResponse;
import com.accenture.eVote.elettorims.repository.ElettoreEnteRepository;
import com.accenture.eVote.elettorims.repository.ElettoreSchedaRepository;
import com.accenture.eVote.elettorims.repository.ListaCertificatiRepository;
import com.accenture.eVote.elettorims.util.Crittografia;
import com.accenture.eVote.elettorims.util.enums.ElettoriEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class OracleService {

    @Autowired
    private Crittografia crittografia;
    @Autowired
    private ElettoreEnteRepository elettoreEnteRepository;
    @Autowired
    private ListaCertificatiRepository listaCertificatiRepository;
    @Autowired
    private ElettoreSchedaRepository elettoreSchedaRepository;

    public byte[] getRequestCodificata(OracleRequest oracleRequest) {

        return crittografia.codificaOracleRequest(oracleRequest.toString());
    }

    public String getGeoIdByIdElettore(OracleRequest oracleRequest) {
        return elettoreEnteRepository.findByIdElettore(oracleRequest.getUserId()).getGeoId();
    }

    public String generaTokenVoto(OracleGenerateResponse oracleGenerateResponse) {
        // Genero il token di voto con l'attributo qualificato geoId e userPreferences
        return crittografia.generaTokenVoto(oracleGenerateResponse);
    }

    public Boolean verifySchede(OracleRequest oracleRequest) {
        List<Long> listIdSchede = Arrays.stream(oracleRequest.getUserPreferences().split(";"))
                .map(Long::parseLong)
                .collect(Collectors.toList());

        List<Long> listIdAccettate = elettoreSchedaRepository.findIdsSchedeAccettate(oracleRequest.getUserId(), ElettoriEnum.SCHEDA_ACCETTATA.getValore());

        // Ordiniamo le liste prima di fare il paragone
        Collections.sort(listIdSchede);
        Collections.sort(listIdAccettate);

        return listIdSchede.equals(listIdAccettate);
    }
}
