package com.accenture.eVote.elettorims.util;

import com.accenture.eVote.elettorims.util.enums.EnumMesiCodiceFiscale;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;

@Component
public class CodiceFiscaleDecoder {

    @Value("${formati.request.data}")
    private String formatoData;

    public LocalDate getDataNascita(String codiceFiscale) {

        DateTimeFormatter formato = DateTimeFormatter.ofPattern(formatoData);

        // Decodifica mese
        String mese = EnumMesiCodiceFiscale.valueOf(codiceFiscale.substring(8, 9)).getValore();
        // Recupero giorno
        String giorno = codiceFiscale.substring(9, 11);
        // Decodifica giorno
        String giornoDec = (Integer.valueOf(giorno) - 40 < 0) ?
                giorno : // recupero il giorno maschi
                String.valueOf(Integer.valueOf(giorno) - 40); // recupero il giorno sottrato 40 per le femmine

        giornoDec = padLeftZeros(giornoDec, 2);

        // Decodifica dell'anno, non essendo possibile riconoscere il secolo 1900 dal 2000 gli anni verranno tradotti
        // secondo la seguente regola
        // anno corrente = 2023
        // anno nascita = 1923
        // anno codice fiscale = 23
        // se le ultime due cifre di anno nascita sono minori o uguali dell'anno codice fiscale viene considerato come secolo 2000
        // anno utilizzato = 2023

        String annoCf = codiceFiscale.substring(6, 8);
        String annoOggi = String.valueOf(LocalDate.now().getYear() % 100);

        String anno = (Integer.valueOf(annoOggi) >= Integer.valueOf(annoCf)) ?
                "20" + annoCf :
                "19" + annoCf;

        return LocalDate.parse( anno + "-" + mese + "-" + giornoDec, formato);
    }


    public String padLeftZeros(String inputString, int length) {
        if (inputString.length() >= length) {
            return inputString;
        }
        StringBuilder sb = new StringBuilder();
        while (sb.length() < length - inputString.length()) {
            sb.append('0');
        }
        sb.append(inputString);

        return sb.toString();
    }
}
