import { HttpClient, HttpClientModule, HTTP_INTERCEPTORS } from '@angular/common/http';
import { NgModule } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';


import { AppRoutingModule } from './app-routing.module';
import { AppComponent } from './app.component';
import { SecuredComponent } from './modules/secured/secured.component';
import { WelcomePageComponent } from './modules/auth/page/welcome-page/welcome-page.component';
import { SafePipe } from './safe.pipe';
import { IframeTrackerDirective } from './iframe-tracker.directive';
import { LoginFormComponent } from './modules/auth/page/login-form/login-form.component';
import { HttpLoaderFactory } from './modules/shared/modules/translate/translate.module';
import { MessageComponent } from './components/message/message.component';
import { AuthComponent } from './modules/auth/auth.component';
import { ResponseInterceptor } from './interceptors/response.interceptor';
import { TranslateLoader, TranslateModule } from '@ngx-translate/core';
import {AuthInterceptor} from "./interceptors/auth.interceptor";
import { ErrorPageComponent } from './components/error-page/error-page.component';

@NgModule({
  declarations: [
    AppComponent,
    SafePipe,
    IframeTrackerDirective,
    MessageComponent,
    ErrorPageComponent,
  ],
  imports: [
    BrowserModule,
    AppRoutingModule,
    HttpClientModule,
    
    TranslateModule.forRoot({
      loader: {
        provide: TranslateLoader,
        useFactory: HttpLoaderFactory,
        deps: [HttpClient]
      },
    }),
  ],
  providers: [
    {
      provide: HTTP_INTERCEPTORS,
      useClass: ResponseInterceptor,
      multi: true
    },
    {
      provide: HTTP_INTERCEPTORS,
      useClass: AuthInterceptor,
      multi: true
    },
  ],
  bootstrap: [AppComponent]
})
export class AppModule { }
