import { Directive, Input, ElementRef, Renderer2, OnInit } from '@angular/core';

@Directive({
  selector: '[appDeviceVisibility]'
})
export class DeviceVisibilityDirective implements OnInit {
  @Input() showOnMobile: boolean = false;
  @Input() showOnDesktop: boolean = false;

  constructor(private el: ElementRef, private renderer: Renderer2) {}

  ngOnInit() {
    this.checkVisibility();
    window.addEventListener('resize', () => this.checkVisibility());
  }

  private checkVisibility() {
    const isMobile = window.innerWidth < 768;
    
    if ((isMobile && this.showOnMobile) || (!isMobile && this.showOnDesktop)) {
      this.renderer.setStyle(this.el.nativeElement, 'display', 'block');
    } else {
      this.renderer.setStyle(this.el.nativeElement, 'display', 'none');
    }
  }
}
