import {Injectable} from '@angular/core';
import {HttpErrorResponse, HttpEvent, HttpHandler, HttpInterceptor, HttpRequest} from '@angular/common/http';
import {Observable, throwError} from 'rxjs';
import {catchError} from "rxjs/operators";
import {Router} from "@angular/router";
import {MessageService, MessageType} from "../services/message.service";
import {AuthService} from "../services/auth.service";
 
@Injectable()
export class AuthInterceptor implements HttpInterceptor {
 
  constructor(private authService: AuthService, private router: Router, private messageService: MessageService) {
  }
 
  intercept(request: HttpRequest<unknown>, next: HttpHandler): Observable<HttpEvent<unknown>> {
    const token = this.authService.getSessionStorage("jwtTokenData");
    const userSessionId = this.authService.getSessionStorage("userSessionId");

    if (!token) {
      return next.handle(request);
    }

    // Aggiungi i token all'header della richiesta
    let modifiedRequest = request;

    if (token) {
      modifiedRequest = modifiedRequest.clone({
        headers: modifiedRequest.headers.set("Authorization", token)
      });
    }
    if (userSessionId) {
      modifiedRequest = modifiedRequest.clone({
        headers: modifiedRequest.headers.set("User-Session-Id", userSessionId)
      });
    }

    modifiedRequest = modifiedRequest.clone({
      headers: modifiedRequest.headers.set("X-App-Key", "elettori-ms")
    });

    return next.handle(modifiedRequest).pipe(
      catchError(err => {
        if (err instanceof HttpErrorResponse) {
          if (err.status === 401) {
            this.authService.doLogout()
          }
        }
        if (err.error) {
          this.messageService.trigger(`${err.error.code} ${err.error.mess}`, "ERRORE", MessageType.Error)
        }
        return throwError(err)
      })
    )
  }
}