import { Component, HostListener, Inject, LOCALE_ID, OnChanges, OnInit, ɵsetAllowDuplicateNgModuleIdsForTest } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, NavigationExtras, Router } from '@angular/router';
import { AuthService } from 'src/app/services/auth.service';
import { LoginService } from 'src/app/services/login.service';
import { Consolato } from '../../../secured/services/consolato';
import { DatiAutocertificazioneDTO } from '../../../secured/services/dati-autocertificazione-DTO';
import { JwtHelperService } from '@auth0/angular-jwt';
import { MessageService, MessageType } from 'src/app/services/message.service';
import { environment } from 'src/environments/environment';

import CodiceFiscale  from 'codice-fiscale-js';
import { formatDate } from '@angular/common';
import { iif } from 'rxjs';
import * as jwt from 'jsonwebtoken'
import { UserData } from 'src/app/modules/secured/services/userData';
import { BreakpointObserver, BreakpointState } from '@angular/cdk/layout';
import { Title } from '@angular/platform-browser';

const SMALL_WIDTH_BK = 1000;

@Component({
  selector: 'app-login-form',
  templateUrl: './login-form.component.html',
  styleUrls: ['./login-form.component.css']
})
export class LoginFormComponent implements OnInit{
  
  showModal: boolean = false;
  showSecondModal: boolean = false;
  confirm: boolean = false;
  selectOptions: Array<string> = [];
  formData: any = {dropdown1: null};
  listaConsolati: Consolato[]=[];
  invalidForm: boolean = true;
  formAutocertificazione: DatiAutocertificazioneDTO = new DatiAutocertificazioneDTO;
  jwtToken: string = '';
  encryptedText: string = ''; 
  decryptedText: string = '';
  selectedDate: string ='';
  userDataJSON: string ='';
  flagCNS: boolean = true;
  isScreenSmall:boolean=false;

  myForm=new FormGroup({
    lastName: new FormControl(''),
    firstName: new FormControl(''),
    birthDate: new FormControl(''),
    birthPlace: new FormControl(''),
    birthProvince: new FormControl(''),
    birthNation: new FormControl(''),
    gender: new FormControl(''),
    CF: new FormControl({value: '', disabled: true}),
    email: new FormControl(''),

    checkbox3: new FormControl(''),
    checkbox3Input: new FormControl(''),
    checkbox4: new FormControl(''),
    checkbox4Input: new FormControl(''),
    checkbox5: new FormControl(''),
    checkbox5Input: new FormControl(''),
    checkbox6: new FormControl(''),
    checkbox6Input: new FormControl(''),

    selector: new FormControl('', Validators.required),

    checkbox1: new FormControl('', Validators.requiredTrue),
    checkbox2: new FormControl('', Validators.requiredTrue),
  });

  constructor(private loginService: LoginService,
    private breakpointObserver: BreakpointObserver,
    private route: ActivatedRoute,
    private router: Router,
    private authService : AuthService,
    private messageService : MessageService,
    private titleService : Title,
    @Inject(LOCALE_ID) private locale: string
    ) {
      this.breakpointObserver.observe([`(max-width:${SMALL_WIDTH_BK}px)`])
      .subscribe((state: BreakpointState) => {
        this.isScreenSmall = state.matches;
      })
    
  }

  ngOnInit(){
    this.listaConsolati=this.loginService.getConsolati();
    this.jwtToken = this.authService.getSessionStorage('jwtTokenData')!;
    this.flagCNS = this.authService.getSessionStorage('flagCNS') === 'true';
    this.initForm();
  }


  onChange(event:any){
    this.invalidForm = !(this.xorLogic() && (this.myForm.get('selector')!.value !== '') && this.myForm.get('checkbox1')!.valid && this.myForm.get('checkbox2')!.valid && this.checkForm());

    this.checkForm();

    //le 4 checkbox nella seconda parte del form si comportano come RadioButton
    if(event.target.type==='checkbox' && !(event.target.id === 'checkbox1' || event.target.id === 'checkbox2')){
      this.myForm.get('checkbox3')?.setValue('');
      this.myForm.get('checkbox4')?.setValue('');
      this.myForm.get('checkbox5')?.setValue('');
      this.myForm.get('checkbox6')?.setValue('');
      this.myForm.get(event.target.id)?.setValue('true');

      for (let i = 3; i <= 6; i++) {
        const id = 'checkbox'+i;
        this.myForm.get(id+'Input')!.setValue('');
        this.myForm.get(id+'Input')!.disable(); 
      }
      this.myForm.get(event.target.id+'Input')!.enable();
    }
  }

  //Logica XOR per controllare che solo una coppia checkbox-inputText sia valorizzata
  xorLogic(): boolean {
    const couple1Result = this.myForm.get('checkbox3')?.value && (this.myForm.get('checkbox3Input')?.value !== '');
    const couple2Result = this.myForm.get('checkbox4')?.value && (this.myForm.get('checkbox4Input')?.value !== '');
    const couple3Result = this.myForm.get('checkbox5')?.value && (this.myForm.get('checkbox5Input')?.value !== '');
    const couple4Result = this.myForm.get('checkbox6')?.value && (this.myForm.get('checkbox6Input')?.value !== '');

    const couple1MixedValues = this.myForm.get('checkbox3')?.value && !(this.myForm.get('checkbox3Input')?.value !== '') || 
      !(this.myForm.get('checkbox3')?.value) && (this.myForm.get('checkbox3Input')?.value !== '');
    const couple2MixedValues = this.myForm.get('checkbox4')?.value && !(this.myForm.get('checkbox4Input')?.value !== '') || 
      !(this.myForm.get('checkbox4')?.value) && (this.myForm.get('checkbox4Input')?.value !== '');
    const couple3MixedValues = this.myForm.get('checkbox5')?.value && !(this.myForm.get('checkbox5Input')?.value !== '') || 
      !(this.myForm.get('checkbox5')?.value) && (this.myForm.get('checkbox5Input')?.value !== '');
    const couple4MixedValues = this.myForm.get('checkbox6')?.value && !(this.myForm.get('checkbox6Input')?.value !== '') || 
      !(this.myForm.get('checkbox6')?.value) && (this.myForm.get('checkbox6Input')?.value !== '');

    const mixedResults = [couple1MixedValues, couple2MixedValues, couple3MixedValues, couple4MixedValues].filter(result => result);
    const trueResults = [couple1Result, couple2Result, couple3Result, couple4Result].filter(result => result);

    return trueResults.length === 1 && mixedResults.length === 0; // Return true if exactly one couple is true
  }

  onSubmit() {
    this.generateUserData();
    // Qui puoi gestire la logica quando il form viene inviato
    if(!this.myForm.valid){
      this.myForm.markAllAsTouched();
    }
    if (this.myForm.valid) {
      // Form is valid, submit the data
      const userId = 1;
      const navigationExtras: NavigationExtras = {
        state: {
          customData: userId
        }
      };
      sessionStorage.setItem('userId', JSON.stringify(userId));
      this.openModal();  
    }
  }

  isEmailValid(email: string){
    return /^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,4}$/.test(email);
  }

  convertiData(dataString: string): string {
    const parts = dataString.split('/');
    if (parts.length === 3) {
      const day = parts[0];
      const month = parts[1];
      const year = parts[2];
      // Costruisci la data nel formato "yyyy-MM-dd"
      const dataConvertita = `${year}-${month}-${day}`;
      return dataConvertita;
    } else {
      // Gestisci il caso in cui la stringa di input non sia valida
      return 'Data non valida';
    }
  }

  checkForm(){
    if(environment.flagControlloCF === 'n') return true;

    if(environment.flagControlloCF === 'n') return true;

    if(this.myForm.get('firstName')!.value !== '' &&
    this.myForm.get('lastName')!.value !== '' &&
    this.myForm.get('birthDate')!.value !== '' &&
    this.myForm.get('birthPlace')!.value !== '' &&
    this.myForm.get('gender')!.value !== '' &&
    this.myForm.get('CF')!.value !== '' &&
    this.myForm.get('email')!.value !== '' &&
    this.isEmailValid(this.myForm.get('email')!.value+"")
    ){
      const date = this.myForm.get('birthDate')!.value
      const cf = new CodiceFiscale({
        name: this.myForm.get('firstName')!.value+"",
        surname: this.myForm.get('lastName')!.value+"",
        gender: this.myForm.get('gender')!.value === 'M'? 'M':'F',
        day: parseInt(date?.substring(8,10)+""),
        month: parseInt(date?.substring(5,7)+""),
        year: parseInt(date?.substring(0,4)+""),
        birthplace: "B354", 
        birthplaceProvincia: "" // Optional
      });
      return cf.cf.substring(0,11) === this.myForm.get('CF')!.value?.substring(0,11);
    }
    return false;
  }

  initForm(){

    //QUESTA PORZIONE DI CODICE E' STATA RIMOSSA PER MOTIVI DI SICUREZZA

    //Decodifica token
    const helper = new JwtHelperService();
    const decoded = helper.decodeToken(this.decryptedText);

    if(this.flagCNS){
      this.myForm.get('CF')!.setValue(decoded.elettore.SPID_fiscalNumber);
      this.myForm.get('CF')?.disable();
    }
    else{
      this.authService.setSessionStorage('Name', decoded.elettore.SPID_name);
      this.authService.setSessionStorage('FamilyName', decoded.elettore.SPID_familyName);

      this.myForm.get('firstName')!.setValue(decoded.elettore.SPID_name);
      this.myForm.get('firstName')!.disable();
      this.myForm.get('lastName')!.setValue(decoded.elettore.SPID_familyName);
      this.myForm.get('lastName')!.disable();
      this.myForm.get('birthDate')!.setValue(decoded.elettore.SPID_dateOfBirth);
      this.myForm.get('birthDate')!.disable();
      this.myForm.get('birthProvince')!.setValue('');
      this.myForm.get('birthProvince')!.disable();
      this.myForm.get('birthNation')!.setValue('');
      this.myForm.get('birthNation')!.disable();
      this.myForm.get('CF')!.setValue(decoded.elettore.SPID_fiscalNumber);
      this.myForm.get('CF')!.disable();
      if(decoded.elettore.SPID_digitalAddress !== '' && decoded.elettore.SPID_digitalAddress !== null) {
        this.myForm.get('email')!.setValue(decoded.elettore.SPID_digitalAddress);
        this.myForm.get('email')!.disable();
      }
      if(decoded.elettore.idp !== 'CIE') {
        this.myForm.get('gender')!.setValue(decoded.elettore.SPID_gender);
        this.myForm.get('gender')!.disable();
      }
    }
  }

  generateUserData(){
    const userData:UserData={
      SPID_name: this.myForm.get('firstName')!.value+"",
      SPID_familyName: this.myForm.get('lastName')!.value+"",
      SPID_dateOfBirth: formatDate(this.myForm.get('birthDate')!.value+"", 'yyyy-MM-dd', 'en'),
      SPID_placeOfBirth: this.myForm.get('birthPlace')!.value+"",
      SPID_gender: this.myForm.get('gender')!.value+"",
      SPID_digitalAddress: this.myForm.get('email')!.value+"",
    };
    this.authService.setSessionStorage('Name', userData.SPID_name);
    this.authService.setSessionStorage('FamilyName', userData.SPID_familyName);
    this.userDataJSON = JSON.stringify(userData);

    //QUESTA PORZIONE DI CODICE E' STATA RIMOSSA PER MOTIVI DI SICUREZZA
  }

  submitForm(){
    const formDTO: DatiAutocertificazioneDTO={
      jsonCNS: this.generateUserData()!,
      causaleEstero: this.getMotivazione(),
      descrizioneCausaleEstero: this.getDescrizioneMotivazione(),
      geoIdConsolato: this.myForm.get('selector')!.value
    }
    this.loginService.submitAutocertificazione(formDTO).subscribe({
      next: res => {
        const userSessionId: number = res.userSessionId;
        this.authService.setSessionStorage('userSessionId', userSessionId);
      },
      error: err=>{
        this.messageService.trigger('Autocertificazione fallita. ['+err+']','AUTOCERTIFICAZIONE',MessageType.Error);
        this.router.navigateByUrl('/auth/login')
      }
    });
  }

  goBack(){
    this.router.navigateByUrl('/auth/login/tabs');
  }
  //prende il valore dell'unica inputText valorizzata
  getDescrizioneMotivazione(){
    return this.myForm.get('checkbox3')?.value ?
    this.myForm.get('checkbox3Input')?.value :
    this.myForm.get('checkbox4')?.value ?
    this.myForm.get('checkbox4Input')?.value :
    this.myForm.get('checkbox5')?.value ? 
    this.myForm.get('checkbox5Input')?.value :
    this.myForm.get('checkbox6Input')?.value;
  }

  getMotivazione(){
    return this.myForm.get('checkbox3')!.value ? 1 :
    this.myForm.get('checkbox4')!.value ? 2 :
    this.myForm.get('checkbox5')!.value ? 3 : 4;
  }

  openModal() {
    this.showModal = true;
  }

  closeModal() {
    this.showModal = false;
  }

  closeSecondModal(){
    if(this.confirm)this.router.navigateByUrl('/app/home');
    else this.router.navigateByUrl('/auth/login');
    this.showSecondModal = false;
  }

  confirmForm(){
    this.closeModal();
    this.confirm = true;
    this.submitForm();
    this.showSecondModal=true;
  }

  negateForm(){
    this.closeModal();
    this.confirm = false;
    this.showSecondModal=true;
  }

  @HostListener('document:keydown', ['$event'])
  handleKeyboardEvent(event: KeyboardEvent) {
    if (event.key === 'Escape' || event.key === 'Esc' || event.keyCode === 27) {
      this.closeModal();
      this.closeSecondModal();
    }
  }
}
