import { Component, ElementRef, HostListener, Pipe, PipeTransform, Renderer2 } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { DomSanitizer, Title } from '@angular/platform-browser';
import { ActivatedRoute, Router } from '@angular/router';
import { JwtHelperService } from '@auth0/angular-jwt';
import { LoginType } from 'src/app/modules/secured/services/login-type';
import { MetodoAutenticazione } from 'src/app/modules/secured/services/metodo-autenticazione';
import { AuthService } from 'src/app/services/auth.service';
import { LoginService } from 'src/app/services/login.service';

import { environment } from 'src/environments/environment';
import { BreakpointObserver, BreakpointState } from '@angular/cdk/layout';

const SMALL_WIDTH_BK = 1000;

@Component({
  selector: 'app-login-tabs',
  templateUrl: './login-tabs.component.html',
  styleUrls: ['./login-tabs.component.css']
})
export class LoginTabsComponent {

  encryptedText: string = ''; 
  decryptedText: string = '';
 

  flagUtenteMinorenne: boolean = false;

  iframe: any = document.querySelector('iframe');
  tabs: boolean[] = [true, true, false, false];
  listaConsolati: any[] = [];
  listaLogins: LoginType[] = [];
  token?: string;
  temporaneo?: boolean;
  metodoAutenticazioneList: Array<MetodoAutenticazione> = [];
  myForm = new FormGroup({
    selector: new FormControl('', Validators.required),
  });

  isScreenSmall:boolean=false;

  constructor(private loginService: LoginService,
    private sanitizer: DomSanitizer,
    private route: ActivatedRoute,
    private authService: AuthService,
    private router: Router,
    private breakpointObserver: BreakpointObserver,
    private titleService: Title,
  ) {
    this.breakpointObserver.observe([`(max-width:${SMALL_WIDTH_BK}px)`])
    .subscribe((state: BreakpointState) => {
      this.isScreenSmall = state.matches;
    })

  }


  ngOnInit() {

    this.titleService.setTitle('Area personale | E-Vote');
    this.listaConsolati = this.loginService.getConsolati();

    this.token = this.route.snapshot.queryParamMap.get('token') || undefined;
    if (this.token) this.doRedirect()

    this.loginService.getLoginTypeBE().subscribe(data => {
      this.metodoAutenticazioneList = JSON.parse(JSON.stringify(data.metodoAutenticazioneList));
      this.metodoAutenticazioneList.sort((a, b) => a.id - b.id);
      this.metodoAutenticazioneList.forEach((metodo) => {
        metodo.linkAutenticazioneSicuro = this.getSanitized(metodo.linkAutenticazione);
      });
    });
  }

  doRedirect() {
    if (this.token && this.token != 'null') {
      this.authService.getTokensAfterLogin(this.token).subscribe({
        next: res => {
          this.authService.setSessionStorage("jwtTokenData", res.jwtTokenData);
          this.authService.setSessionStorage("jwtTokenAuth", res.jwtTokenAuth);
          this.authService.setSessionStorage("userSessionId", res.userSessionId);
          this.authService.setSessionStorage("flagCNS", res.cns);
          this.temporaneo = res.temporaneo;

          //QUESTA PORZIONE DI CODICE E' STATA RIMOSSA PER MOTIVI DI SICUREZZA
          const helper = new JwtHelperService();
          const decoded = helper.decodeToken(this.decryptedText);
          this.authService.setSessionStorage('Name', decoded.elettore.SPID_name);
          this.authService.setSessionStorage('FamilyName', decoded.elettore.SPID_familyName);
        },
        error: err => {
          this.token = 'null';
          if(err === 'Utente minorenne' || 'Utente non ha diritto al voto'){ this.flagUtenteMinorenne = true;}
        }
      })
    }
  }

  getSanitized(url: string) {
    return this.sanitizer.bypassSecurityTrustResourceUrl(url);
  }

  getSelected(index: number) {
    return this.tabs[index];
  }

  //impostare la tab attiva
  onChange(event: any, id : any) {
    const links = document.querySelectorAll('li a');
    // Add a click event listener to each anchor element
    links.forEach(link => {
      // Remove the "active" class from all anchor elements
      links.forEach(link => {
        link.classList.remove('active');
        if (link.id == id) {
          // Add the "active" class to the clicked anchor element
          link.classList.add('active');
        }
      });
    });
    this.tabs.fill(false);
    this.tabs[id] = true;
  }

  buttonModal() {
    if (this.token && this.token != 'null') {
      if (this.temporaneo) {
        this.router.navigateByUrl('/auth/login/form');
      } else {
        this.router.navigateByUrl('/app/home');
      }
    } else {
      this.router.navigateByUrl('auth/login');
    }
  }

  toWelcome() { this.router.navigateByUrl('auth/login'); }

  toForm() { this.router.navigateByUrl('/auth/login/form'); }

  loginStress() {
    this.loginService.loginStressTest().subscribe({
      next: res => {
        this.authService.setSessionStorage("jwtTokenData", res.jwtTokenData);
        this.authService.setSessionStorage("jwtTokenAuth", res.jwtTokenAuth);
        this.authService.setSessionStorage("userSessionId", res.userSessionId);
        this.authService.setSessionStorage("flagCNS", res.cns);
        this.temporaneo = res.temporaneo;

        //QUESTA PORZIONE DI CODICE E' STATA RIMOSSA PER MOTIVI DI SICUREZZA
        const helper = new JwtHelperService();
        const decoded = helper.decodeToken(this.decryptedText);
        this.authService.setSessionStorage('Name', decoded.elettore.SPID_name);
        this.authService.setSessionStorage('FamilyName', decoded.elettore.SPID_familyName);

        this.toForm();
      },
      error: err => {
        this.token = 'null';
        if(err === 'Utente minorenne'){ this.flagUtenteMinorenne = true;}
      }
    })
  }
}
