import { Component, OnInit } from '@angular/core';
import { DomSanitizer, Title } from '@angular/platform-browser';
import { ActivatedRoute, Router } from '@angular/router';
import { ConsensoVotoService } from 'src/app/services/consenso-voto.service';

interface Scheda {
  idScheda? : any,
  listaSchedaDTOList?: Partito[],
  descTerritorio?:string;
  numPreferenze? : any;
}

interface Partito{
  lista : Lista
}

interface Lista{
  id? : any,
  contrassegno? : Contrassegno,
}

interface Contrassegno{
  id? : any,
  blobImmagine? : string
}

@Component({
  selector: 'app-fac-simile',
  templateUrl: './fac-simile.component.html',
  styleUrls: ['./fac-simile.component.css']
})
export class FacSimileComponent implements OnInit{

  routePath : string = '$[ERRORE]';
  routePathForBack : string = '$[ERRORE]'
  scheda? : Scheda;

  constructor(
    private router: Router,
    private route : ActivatedRoute,
    private consensoVotoService : ConsensoVotoService,
    private sanitizer: DomSanitizer,
    private titleService: Title
  ){
    this.loadSchede();
  }

  ngOnInit(): void {
    this.routePath = this.route.snapshot.url[2].path;
    this.routePathForBack = this.route.snapshot.url[0].path;

    const titlePart1 = this.routePath == 'altro'?'Scheda tipo elezione 3 ':this.routePath == 'camera'? 'Camera Estero ':'Senato Estero ';
    this.titleService.setTitle('Fac-simile '+ titlePart1 + '- ' + this.scheda?.descTerritorio !== undefined ?  this.scheda?.descTerritorio+'' : ' ' + ' | E-Vote');
  }

  loadSchede(){
    this.consensoVotoService.getSchede(this.route.snapshot.url[3].path).subscribe({
      next:  res => {this.scheda = res; }
    })
  }

  goBack(){
    this.router.navigateByUrl('app/'+this.routePathForBack);
  }

  getSrcImage(blobImage : string){
    const objectURL = URL.createObjectURL(new Blob([this.base64ToArrayBuffer(blobImage)]));
    return this.sanitizer.bypassSecurityTrustUrl(objectURL);
  }
  // Funzione per convertire la stringa base64 in un array buffer
  private base64ToArrayBuffer(base64: string): ArrayBuffer {
    const binaryString = window.atob(base64);
    const length = binaryString.length;
    const bytes = new Uint8Array(length);
    for (let i = 0; i < length; i++) {
      bytes[i] = binaryString.charCodeAt(i);
    }
    return bytes.buffer;
  }

}
