import { BreakpointObserver, BreakpointState } from '@angular/cdk/layout';
import { Component, HostListener } from '@angular/core';
import { Title } from '@angular/platform-browser';
import { Router } from '@angular/router';
import { JwtHelperService } from '@auth0/angular-jwt';
import { AuthService } from 'src/app/services/auth.service';
import { ConsensoVotoService } from 'src/app/services/consenso-voto.service';
import { MessageService, MessageType } from 'src/app/services/message.service';
import { VotoService } from 'src/app/services/voto.service';


const SMALL_WIDTH_BK = 1000;
@Component({
  selector: 'app-homepage',
  templateUrl: './homepage.component.html',
  styleUrls: ['./homepage.component.css']
})
export class HomepageComponent {
  showModal: boolean = false;
  flagConsenso: boolean | undefined;
  flagConsensoModale : boolean | undefined;

  flagElezioneAttiva: boolean = true;

  userSessionId: any;

  jwtToken: string = '';

  codFaseElettorale?:any;

  showApertura : boolean = true;  
  idDataElezione?: any;
  dataInizio?: any;
  dataInizioElezione?: any;
  dataFine?: any;
  dataInizioConsenso?: any;
  dataFineConsenso?: any;
  sysDate : any = new Date();
  flagPostVotazione : boolean = false;
  allSchedeRifiutate: boolean = false;
  elezioneChiusa: boolean = false;

  statoFinestraConsenso : 'ChiusaConsenso' | 'ApertaConsenso' | 'NullConsenso' = 'ChiusaConsenso'
  statoFinestraVoto : 'ChiusaVoto' | 'ApertaVoto' = 'ChiusaVoto';

  numberStep: any = 1;
  width = 312;

  isScreenSmall:boolean=false;
  constructor(
    private votoService: VotoService,  
    private router: Router,
    private consensoVotoService: ConsensoVotoService,
    private messageService : MessageService,
    private authService : AuthService,
    private breakpointObserver: BreakpointObserver,
    private titleService : Title
  ) {
    this.titleService.setTitle('Home page area privata | E-Vote');
    this.breakpointObserver.observe([`(max-width:${SMALL_WIDTH_BK}px)`])
    .subscribe((state: BreakpointState) => {
      this.isScreenSmall = state.matches;
    })
    
    if(window.innerWidth<320)this.width = 272;
    this.votoService.getVerificavoto().subscribe(resVoto=>{
        this.consensoVotoService.getFinestraVoto().subscribe({
          next: res => {
            if(res.dataElezione){
              this.authService.setSessionStorage("dataElezione",new Date(res.dataElezione))
              this.votoService.postSchedaSchede(
              this.votoService.getDataFormatForBE(new Date(res.dataElezione))).subscribe({
                  next:resScheda => {
                    if(resScheda.schede && resScheda.schede[0]?.flagAccettazione != undefined){
                      if(resVoto.votato == true){
                        this.flagPostVotazione = true;
                      }
                      this.allSchedeRifiutate = !(resScheda.schede.find((scheda:any) => scheda.flagAccettazione == true))
                      if(!!(resScheda.schede.find((scheda:any) => scheda.flagAccettazione == true) && res.voto == false)){
                        this.router.navigateByUrl('app/schede-elettorali-accettate');
                      }
                     
                    }
                  },
                })
            };
          }
        })
    });
    
  }

  ngOnInit(): void {
    
    this.consensoVotoService.getConsenso().subscribe({
      next: res => {
        if(res.dataInizio)this.dataInizioConsenso = new Date(res.dataInizio);
        this.idDataElezione = res.idDataElezione;
        this.dataFineConsenso = new Date(res.dataFine)
        if(res.dataInizio == null) this.statoFinestraConsenso = 'NullConsenso';
        else if(this.sysDate < this.dataInizioConsenso || this.sysDate >this. dataFineConsenso) {
          this.statoFinestraConsenso = 'ChiusaConsenso';
          if(res.consensoAttivo == undefined)this.statoFinestraConsenso = 'NullConsenso';
        }
        if(this.dataInizioConsenso <= this.sysDate && this.sysDate <= this.dataFineConsenso && res.dataInizio != null){ 
          this.statoFinestraConsenso = 'ApertaConsenso';
        }
        this.flagConsenso = res.consensoAttivo;
        this.getInfoVoto();
      },
      error: err =>{
         this.statoFinestraConsenso = 'NullConsenso';
      }
    })
  }

  getInfoVoto() {
    this.consensoVotoService.getFinestraVoto().subscribe({
      next: res => {
        if(res.dataElezione)this.authService.setSessionStorage("dataElezione",new Date(res.dataElezione));
        this.authService.setSessionStorage("idElezione",res.idDataElezione);
        if(res?.codFaseElettorale=='3'){
          this.statoFinestraConsenso = 'NullConsenso';
          this.codFaseElettorale = res?.codFaseElettorale;
        }
        if(res?.confTempoElettoraleDTO?.dataInizio)this.dataInizio = new Date(res.confTempoElettoraleDTO.dataInizio);
        if(res?.confTempoElettoraleDTO?.dataFine)this.dataFine = new Date(res.confTempoElettoraleDTO.dataFine);
        if(res?.dataElezione)this.dataInizioElezione = new Date(res.dataElezione);
        this.showApertura = this.dataInizio > this.sysDate;
        if(this.sysDate >= this.dataFine && this.flagConsenso == true)this.elezioneChiusa = true;
        //CHECK SU DATA INIZIO MAGGIORNO DI SYSDATE
        if(this.dataFine <= this.sysDate && this.flagConsenso != undefined) this.statoFinestraVoto = 'ChiusaVoto';
        if(this.dataInizio <= this.sysDate && this.sysDate <= this.dataFine && this.flagConsenso != undefined){ 
          this.statoFinestraVoto = 'ApertaVoto';
        }
        this.flagElezioneAttiva = res?.confTempoElettoraleDTO?.finestraAttiva;
      },
      error: err => {
        this.statoFinestraConsenso = 'NullConsenso';
      }
    })
  }

  setConsenso(flagConsenso: boolean) {
    this.consensoVotoService.setConsenso( flagConsenso).subscribe({
      next: res => {
        this.flagConsenso = res.consensoAttivo
        this.flagConsensoModale = res.consensoAttivo
        this.getInfoVoto();
      },
      error: err => {}
    })
  }

  openModal() {
    this.flagConsensoModale = undefined;
    this.showModal = true;
  }

  closeModal() {
    this.showModal = false;
  }

  next() {
    if (this.numberStep != 3) {
      this.numberStep++;
    }
  }

  previous() {
    if (this.numberStep != 1) {
      this.numberStep--;
    }
  }

  // Funzione per aggiungere zero davanti a numeri inferiori a 10 per le date
  padZero(num: number | undefined) {
    if (num != undefined) return num < 10 ? '0' + num : num.toString();
    return undefined;
  }

  goSchede(){
    this.votoService.postSchedaSchede(this.votoService.getDataFormatForBE(this.authService.getSessionStorage('dataElezione'))).subscribe({
        next:res => {
          if(res.schede && res.schede[0].flagAccettazione != undefined){
            this.router.navigateByUrl('app/schede-elettorali-accettate');
          }else{
            this.router.navigateByUrl('app/schede-elettorali');
          }
        },
      })
  }

  @HostListener('document:keydown', ['$event'])
  handleKeyboardEvent(event: KeyboardEvent) {
    if (event.key === 'Escape' || event.key === 'Esc' || event.keyCode === 27) {
      this.closeModal();
    }
  }

  touchStartX?: number;

  onTouchStart(event: TouchEvent): void {
    this.touchStartX = event.touches[0].clientX;
  }

  onTouchMove(event: TouchEvent): void {
    event.preventDefault(); // Evita il comportamento di default dello scrolling durante lo swipe
  }

  onTouchEnd(event: TouchEvent): void {
    const touchEndX = event.changedTouches[0].clientX;
    const deltaX = touchEndX - this.touchStartX!;

    if (deltaX > 0) {
      this.previous()
    } else if (deltaX < 0) {
      this.next()
    }
  }

}
