import { Component, HostListener, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { AuthService } from 'src/app/services/auth.service';
import { environment } from 'src/environments/environment';
import { PostVotazioneService } from 'src/app/services/post-votazione.service';
import { VotoService } from 'src/app/services/voto.service';
import { RequestRicevutaDTO } from '../services/requestRicevutaDTO';
import { BreakpointObserver, BreakpointState } from '@angular/cdk/layout';
import { Title } from '@angular/platform-browser';

const SMALL_WIDTH_BK = 1000;

@Component({
  selector: 'app-recap-post-votazione',
  templateUrl: './recap-post-votazione.component.html',
  styleUrls: ['./recap-post-votazione.component.css']
})
export class RecapPostVotazioneComponent implements OnInit{
  schedeDaMostrare: any[] = []; //?: SchedaDto[];
  schede:any[] = [];
  dataElezione?: any;

  showLoader: boolean = false;

  mailFlag: boolean = false;
  showModal: boolean = false;
  showSecondModal: boolean = false;
  success: boolean = true;
  mail: boolean = false;
  pdf: boolean = false;
  showConfirmModal: boolean = false;
  userMail: string = '';
  userIdMock: number = 181;
  idSchedeVotate: string[] = [];
  idSchedeNonVotate: string[] = [];
  ricevutaDTO: RequestRicevutaDTO = {userId: this.authService.getSessionStorage('userSessionId')? parseInt(this.authService.getSessionStorage('userSessionId')!) : this.userIdMock, 
  idDataElezione: this.authService.getSessionStorage('idDataElezione')? parseInt(this.authService.getSessionStorage('idDataElezione')!) : 0
  };

  isScreenSmall:boolean=false;

  constructor(private router: Router,
    private votoService: VotoService,
    private authService: AuthService,
    private breakpointObserver: BreakpointObserver,
    private titleService: Title,
    private postVotazioneService: PostVotazioneService) {
      this.breakpointObserver.observe([`(max-width:${SMALL_WIDTH_BK}px)`])
    .subscribe((state: BreakpointState) => {
      this.isScreenSmall = state.matches;
    })
     }

  ngOnInit() {
    this.titleService.setTitle('Home page post votazione area privata | E-Vote');
    //QUESTA PORZIONE DI CODICE E' STATA RIMOSSA PER MOTIVI DI SICUREZZA

    this.postVotazioneService.checkUserMail().subscribe({
      next: res => {
        this.mailFlag=res;
      },
      error : ()=>{
        this.router.navigateByUrl('/app/home');
      }
    });

    this.userMail = this.postVotazioneService.getUserMail();

    //schede votate
    for( const id of this.idSchedeVotate){
      if(id !== ''){
        this.votoService.getSchede(id).subscribe({
          next: res => {
            const scheda = {
              descrizioneElezione: res.descrizioneElezione,
              votata: true
            }
            this.schedeDaMostrare.push(scheda);
          },
          error : ()=>{
            this.router.navigateByUrl('/app/home');
          }
        });
      }
    }

    //schede non votate
    for( const id of this.idSchedeNonVotate){
      if(id !== ''){
        this.votoService.getSchede(id).subscribe({
          next: res => {
            const scheda = {
              descrizioneElezione: res.descrizioneElezione,
              votata: false
            }
            this.schedeDaMostrare.push(scheda);
          },
          error : ()=>{
            this.router.navigateByUrl('/app/home');
          }
        });
      }
    }
    this.showLoader = false;
  }

  isSenato(schedaToShow: any): any {
    return schedaToShow.descrizioneElezione === "Senato Estero";
  }

  isCamera(schedaToShow: any): any {
    return schedaToShow.descrizioneElezione === "Camera Estero";
  }

  goToEVote() {
    this.showLoader = true;
  }

  //apre prima modale di scelta
  showReceipt(event:any){
    this.showModal = true;
  }

  closeModal() {
    this.showModal = false;
    this.showConfirmModal=false;
  }

  downloadPdf(){
    this.closeModal();
    this.pdf = true;
    this.mail = false
    
    this.postVotazioneService.downloadPDF().subscribe({
      next: res => {
        this.success = true;
        this.downloadPDF(res);
        this.showSecondModal=true;
      },
      error: err => {
        this.success = false;
        this.showSecondModal=true;}
    });
    
  }

  sendMail(){
    this.closeModal();
    this.pdf = false;
    this.mail = true;
    this.showConfirmModal=true;
  }

  confirmMail(){
    this.postVotazioneService.sendMail().subscribe({
      next: res => {
        this.success = true;
        this.closeModal()
        this.showSecondModal=true;
      },
      error: err => {
        this.success = false;
        this.closeModal()
        this.showSecondModal=true;}
    });
  }

  closeSecondModal(){
    this.showSecondModal = false;
    
  }

  confirmMailMock(){
    this.success = true;
    this.closeModal()
    this.showSecondModal=true;
  }

  retry(){
    if(this.mail) { this.confirmMail() }
    if(this.pdf) { this.downloadPdf() }
  }

  downloadPDF(content: Blob) {

    const blob = new Blob([content], { type: 'application/pdf' });
      
    // Crea un oggetto URL per il blob.
    const url = window.URL.createObjectURL(blob);

    // Crea un elemento HTML 'a' per avviare il download del file.
    const a = document.createElement('a');
    a.href = url;
    a.download = 'ricevuta_di_voto.pdf'; // Imposta il nome del file da scaricare.
    a.click();

    // Pulisce l'URL creato per il blob.
    window.URL.revokeObjectURL(url);
  }

  goToHome(){
    if(this.success) this.router.navigateByUrl('/app/home');
  }

  @HostListener('document:keydown', ['$event'])
  handleKeyboardEvent(event: KeyboardEvent) {
    if (event.key === 'Escape' || event.key === 'Esc' || event.keyCode === 27) {
      this.closeModal();
      this.closeSecondModal();
    }
  }

}
