import { Component, OnInit } from "@angular/core";
import { Title } from "@angular/platform-browser";
import { Router } from "@angular/router";
import { AuthService } from "src/app/services/auth.service";
import { VotoService } from "src/app/services/voto.service";
import { environment } from "src/environments/environment";
import { PostVotazioneService } from "../../../services/post-votazione.service";
import { SchedaDto } from "../services/scheda.dto";


@Component({
    selector: 'app-schede-elettorali-accettate',
    templateUrl: './schede-elettorali-accettate.component.html',
    styleUrls: ['./schede-elettorali-accettate.component.css']
})
export class SchedeElettoraliAccettateComponent implements OnInit {

    schedeDaMostrare: any[] = []; //?: SchedaDto[];
    schede:any[] = [];
    dataElezione?: any;
    private pollingInterval = 1000; // Intervallo in millisecondi (es. ogni secondo)
    private pollingTimer: any;
    votoFlag: boolean = false;
    urlRedirect?:string;

    openTest:boolean =true;

    clickedFlag: boolean = false;

    showLoader: boolean = false;

    constructor(private router: Router,
        private votoService: VotoService,
        private postVotazioneService: PostVotazioneService,
        private authService: AuthService,
        private titleService: Title) {
          this.titleService.setTitle('Schede elettorali accettate - Ripartizione EUROPA area privata | E-Vote');
        if(!this.authService.getSessionStorage('dataElezione')){
            this.router.navigateByUrl('app/home');
          }
         }

    ngOnInit() {
        this.votoService.getSchedeAccettate().subscribe({
            next: res => {
                if(res.schede && res.schede.length > 0){
                    this.schedeDaMostrare = res.schede;
                }
            },
            error : ()=>this.router.navigateByUrl('app/schede-elettorali')
        })
        this.showLoader = false;
        //caricamento dati da page precedenti
        if(this.authService.getSessionStorage('userSessionId'))this.dataElezione = new Date(this.authService.getSessionStorage('dataElezione')!);
        this.votoService.postSchedaSchede(this.votoService.getDataFormatForBE(this.authService.getSessionStorage('dataElezione'))).subscribe(res=>this.schede = res.schede)
      }

    

    isSenato(schedaToShow: any): any {
        return schedaToShow?.descrizioneElezione === "Senato Estero"
    }

    isCamera(schedaToShow: any): any {
        return schedaToShow?.descrizioneElezione === "Camera Estero"
    }

    goToEVote() {

      if(localStorage.getItem('openCounter') !== null) {return}

      this.showLoader = true;
      this.startPolling();
      let userPreferences : string = '';
      this.schedeDaMostrare.forEach(scheda=>{
          if(userPreferences == ''){
              userPreferences = userPreferences + scheda.idScheda
          }else{
              userPreferences = userPreferences + ';' + scheda.idScheda
          }
      })
      this.votoService.getAttributoQualificato(userPreferences).subscribe(res=> {
        this.urlRedirect = environment.urlVoto+btoa(res.jwtTokenVoto);
          window.open(environment.urlVoto+btoa(res.jwtTokenVoto))})
    }

    goFacSimile() {
        this.router.navigateByUrl('app/schede-elettorali-attive/fac-simile/:');
    }

    isAllSchedeRifiutate(){
        let isAllRifiutate = true;
        this.schede.forEach((scheda: any) =>{
          if(scheda.flagAccettazione == true){
            isAllRifiutate = false;
          }
        })
        return isAllRifiutate;
      }

      startPolling(): void {
        this.pollingTimer = setInterval(() => {
          this.checkLocalStorage();
        }, this.pollingInterval);
      }
    
      private checkLocalStorage(): void {
        const qualifiedAttribute = localStorage.getItem('qualifiedAttribute'); 
        const schedeV = localStorage.getItem('idSchedeVotate');
        const schedeNV = localStorage.getItem('idSchedeNonVotate');

        if(localStorage.getItem('openCounter') !== null){
            this.openTest=false;
            this.disableLink();
        } else {
          document.querySelector('.link-to-disable')?.removeAttribute('inert'); 
        }
    
        if (qualifiedAttribute !== null && qualifiedAttribute !== '') {
          // Il valore è stato trovato, fai qualcosa...
          if((schedeV !== null && schedeV !== '') && (schedeNV !== null && schedeNV !== '')) {
            
            this.postVotazioneService.hasUserVoted().subscribe({
                next: res => {
                  this.votoFlag = res.votato;
                  if(this.votoFlag){
                      this.stopPolling();
                  }                  
                    
                    if(this.votoFlag){
                      this.postVotazioneService.setSchedeVotate().subscribe({
                        next: res => {
                        },
                        error : ()=>{}
                      });
                      
                      this.router.navigateByUrl('app/recap')
                    }
                },
                error : ()=>{}
            });
          }
        } else {
          // Il valore non è ancora stato trovato
        }
      }
    
      private stopPolling(): void {
        clearInterval(this.pollingTimer);
      }

      disableLink(){document.querySelector('.link-to-disable')?.setAttribute('inert','');}

}
