import { Component, ElementRef, HostListener, OnInit, ViewChild } from '@angular/core';
import { Router } from '@angular/router';
import { FormBuilder, FormControl, FormGroup, Validators } from '@angular/forms';
import { AuthService } from 'src/app/services/auth.service';
import { VotoService } from 'src/app/services/voto.service';
import { Title } from '@angular/platform-browser';


@Component({
  selector: 'app-schede-elettorali',
  templateUrl: './schede-elettorali.component.html',
  styleUrls: ['./schede-elettorali.component.css']
})
export class SchedeElettoraliComponent implements OnInit {
  schedeDaMostrare: any[] = [];
  dataElezione?: any;

  showModal: boolean = false;
  showError : boolean = false;

  constructor(private router: Router,
    private votoService: VotoService,  
    private authService: AuthService,
    private titleService: Title) {
    this.votoService.postSchedaSchede(
    this.votoService.getDataFormatForBE(this.authService.getSessionStorage('dataElezione'))).subscribe({
        next:res => {
          this.schedeDaMostrare = res.schede
          if(res.schede && res.schede[0].flagAccettazione != undefined){
            this.router.navigateByUrl('app/schede-elettorali-accettate');
          }
        },
        error : ()=>this.router.navigateByUrl('app/home')
      })
    if(!this.authService.getSessionStorage('dataElezione')){
      this.router.navigateByUrl('app/home');
    }
     }

  ngOnInit(): void {
    //caricamento dati da page precedenti
    this.titleService.setTitle('Accettazione o rifiuto delle schede elettorali area privata | E-Vote');
    if(this.authService.getSessionStorage('userSessionId'))this.dataElezione = new Date(this.authService.getSessionStorage('dataElezione')!);
  }

  isSenato(schedaToShow: any): any {
    return schedaToShow?.descrizioneElezione == "Senato Estero"
  }

  isCamera(schedaToShow: any): any {
    return schedaToShow?.descrizioneElezione == "Camera Estero"
  }

  showButton(){
    let showButton : boolean = true;
    this.schedeDaMostrare.forEach((scheda: any) =>{
      if(scheda.flagAccettazione == undefined){
        showButton = false;
      }
    })
    return showButton;
  }

  closeModal() {
    this.showModal = false;
  }

  isAllSchedeRifiutate(){
    let isAllRifiutate = true;
    this.schedeDaMostrare.forEach((scheda: any) =>{
      if(scheda.flagAccettazione == true){
        isAllRifiutate = false;
      }
    })
    return isAllRifiutate;
  }

  haveRifiutata(){
    return !!this.schedeDaMostrare.find((scheda : any) => scheda.flagAccettazione == false);
  }

  conferma(){
    this.showModal = true;
  }

  postSchede(){
    let listPost : any[] = [];
    this.schedeDaMostrare.forEach((scheda:any)=>{
      let schedaPost:any = {
        userId:undefined,
        flagAccettazione : undefined,
        schedaVoto : undefined,
      };
      schedaPost.flagAccettazione = scheda.flagAccettazione?1:0;
      schedaPost.schedaVoto = scheda.idScheda;
      listPost.push(schedaPost)
    })
    this.votoService.postSchedeElezione(listPost).subscribe({
      next: res=>this.router.navigateByUrl('app/schede-elettorali-accettate'),
      error: ()=> {this.showError = true;this.closeModal()}
    })
  }

  @HostListener('document:keydown', ['$event'])
  handleKeyboardEvent(event: KeyboardEvent) {
    if (event.key === 'Escape' || event.key === 'Esc' || event.keyCode === 27) {
      this.closeModal();
    }
  }


}
