import { Component, HostListener } from '@angular/core';
import { AuthService } from 'src/app/services/auth.service';
import { PostVotazioneService } from 'src/app/services/post-votazione.service';
import { ElezioniDTO } from '../services/elezioniDTO';
import { RequestRicevutaDTO } from '../services/requestRicevutaDTO';
import { formatDate } from '@angular/common';
import { Router } from '@angular/router';
import { BreakpointObserver, BreakpointState } from '@angular/cdk/layout';
import { Title } from '@angular/platform-browser';

const SMALL_WIDTH_BK = 1000;

@Component({
  selector: 'app-storico',
  templateUrl: './storico.component.html',
  styleUrls: ['./storico.component.css']
})

export class StoricoComponent {
  
  elezioni: ElezioniDTO[]=[];
  showModal: boolean = false;
  showSecondModal: boolean = false;
  success: boolean = false;
  mail: boolean = false;
  pdf: boolean = false;
  showConfirmModal: boolean = false;
  userMail: string = '';
  //userIdMock: number = 181;
  idDataElezione: number = 0;
  mailFlag: boolean = false;
  isScreenSmall:boolean=false;

  constructor(private authService: AuthService,
    private postVotazioneService: PostVotazioneService,
    private breakpointObserver: BreakpointObserver,
    private router: Router,
    private titleService : Title){
      
    this.breakpointObserver.observe([`(max-width:${SMALL_WIDTH_BK}px)`])
      .subscribe((state: BreakpointState) => {
        this.isScreenSmall = state.matches;
      })
    }

  ngOnInit(){
    this.titleService.setTitle('Storico area privata | E-Vote');
   this.postVotazioneService.getElezioni().subscribe({
      next: res => {
        this.elezioni = res;
      },
      error: err => {
        }
    });

    this.postVotazioneService.checkUserMail().subscribe({
      next: res => {
        this.mailFlag=res;
      },
      error : ()=>{
        this.router.navigateByUrl('/app/home');
      }
    });
    this.userMail = this.postVotazioneService.getUserMail();
  }

  showReceipt(event:any){
    this.idDataElezione = event.target.id
    this.showModal = true;
  }

  closeModal() {
    this.showModal = false;
    this.showConfirmModal=false;
  }

  downloadPdf(){
    this.closeModal();
    this.pdf = true;
    this.mail = false;
    
    this.postVotazioneService.downloadPDF(this.idDataElezione).subscribe({
      next: res => {
        this.success = true;
        this.downloadPDF(res);
        this.showSecondModal=true;
      },
      error: err => {
        this.success = false;
        this.showSecondModal=true;}
    });
    
  }

  sendMail(){
    this.closeModal();
    this.pdf = false;
    this.mail = true;
    this.showConfirmModal=true;
  }

  confirmMail(){
    this.postVotazioneService.sendMail(this.idDataElezione).subscribe({
      next: res => {
        this.success = true;
        this.closeModal()
        this.showSecondModal=true;
      },
      error: err => {
        this.success = false;
        this.closeModal()
        this.showSecondModal=true;}
    });
  }

  closeSecondModal(){
    this.showSecondModal = false;
  }

  retry(){
    if(this.mail) { this.confirmMail() }
    if(this.pdf) { this.downloadPdf() }
  }

  downloadPDF(content: Blob) {
    const blob = new Blob([content], { type: 'application/pdf' });
      
    // Crea un oggetto URL per il blob.
    const url = window.URL.createObjectURL(blob);

    // Crea un elemento HTML 'a' per avviare il download del file.
    const a = document.createElement('a');
    a.href = url;
    a.download = 'ricevuta_di_voto.pdf'; // Imposta il nome del file da scaricare.
    a.click();

    // Pulisce l'URL creato per il blob.
    window.URL.revokeObjectURL(url);
  }

  formatDisplayDate(data?: string){
    return formatDate(data!, 'dd/MM/yyyy', 'en');
  }

  goToHome(){
    if(this.success) this.router.navigate(['/app/home']);
  }

  @HostListener('document:keydown', ['$event'])
  handleKeyboardEvent(event: KeyboardEvent) {
    if (event.key === 'Escape' || event.key === 'Esc' || event.keyCode === 27) {
      this.closeModal();
      this.closeSecondModal();
    }
  }

}


