import { BreakpointObserver, BreakpointState } from '@angular/cdk/layout';
import { Component, ElementRef, HostListener, OnDestroy, OnInit } from '@angular/core';
import { NavigationEnd, Router } from '@angular/router';
import { fromEvent, map, merge, of, Subscription } from 'rxjs';
import { AnyCatcher } from 'rxjs/internal/AnyCatcher';
import { AuthService } from 'src/app/services/auth.service';

const SMALL_WIDTH_BK = 1000;
@Component({
  selector: 'app-header',
  templateUrl: './header.component.html',
  styleUrls: ['./header.component.css']
})
export class HeaderComponent implements OnInit, OnDestroy {
  public logged: boolean = false;
  public login: boolean = false;
  public loggedSmall: boolean = false;
  public storico: boolean = false;
  showModal: boolean = false;
  showSecondModal: boolean = false;
  public pageHeaderShort: boolean = false;
  public showSupport: boolean = false;
  flag1Dec: boolean = false;
  networkStatus: boolean = false;
  networkStatus$: Subscription = Subscription.EMPTY;

  el?: any;
  modalFlag:boolean = false;
  firstFocusable?:any;
  lastFocusable?:any;
  focusableElements:any[]=[];
  id:number=0
  

  name?: string = "";
  surname?: string = "";

  width = 1200;
  isScreenSmall:boolean = false;

  constructor(private router: Router,
    private ref: ElementRef,
    private breakpointObserver: BreakpointObserver,
    private authService: AuthService) {
    this.breakpointObserver.observe([`(max-width:${SMALL_WIDTH_BK}px)`])
    .subscribe((state: BreakpointState) => {
      this.isScreenSmall = state.matches;
    })

    this.width = window.innerWidth;

    /*mostra header completo in base alla pagina*/
    this.router.events.subscribe(event => {
      if (event instanceof NavigationEnd) {
        if (this.router.url === '/auth/login' || this.router.url === '/auth/welcome') {
          this.login = this.logged = false;
          if(this.router.url === '/auth/welcome'){
            this.flag1Dec = true;
          }
        }
        if (this.router.url.includes('/auth/login/tabs') || this.router.url.includes('/auth/login/form')) {
          this.login = true;
        }
        if (!(this.router.url.includes('/auth/login/tabs') || this.router.url.includes('/auth/login/form')) && !(this.router.url === '/auth/login') && !(this.router.url === '/auth/welcome')) {
          this.logged = true;
        }
        if(this.router.url === '/app/storico' || this.router.url === '/app/recap'){this.loggedSmall=true} else {this.loggedSmall=false}
        if(this.router.url === '/app/storico'){this.storico=true} else {this.storico = false}
        if (this.router.url.includes('/schede-elettorali')) {
          this.loggedSmall = true;
        }
      }
    });
  }

  @HostListener('window:offline')
  setNetworkOffline(): void {
    this.authService.doLogout();
  }

  ngOnDestroy(): void {
    this.networkStatus$.unsubscribe();
  }

  checkNetworkStatus() {
    this.networkStatus = navigator.onLine;
    this.networkStatus$ = merge(
      of(null),
      fromEvent(window, 'online'),
      fromEvent(window, 'offline')
    )
      .pipe(map(() => navigator.onLine))
      .subscribe(status => {
        this.networkStatus = status;
      });
  }

  ngOnInit(): void {
    this.checkNetworkStatus();
    setInterval(() => {if(!this.networkStatus){
      localStorage.clear();
      sessionStorage.clear();
      }}), 100;

    this.name = this.authService.getSessionStorage("Name")!;
    this.surname = this.authService.getSessionStorage("FamilyName")!;
  }

  openModal(){
    this.showModal = this.modalFlag = true;
    setTimeout(() => {
      this.findModal('#logoutModal');  // Chiamare il metodo dopo il ritardo
    }, 500);
  }

  closeModal(){
    this.showModal=false;
    this.showSecondModal=false
    this.showSupport = false;
    this.modalFlag = false;
  }

  confirm(){
    this.closeModal();
    setTimeout(() => {
      this.closeModal();
      this.authService.doLogout();
      this.router.navigate(['/auth/login']);
    }, 1500); //delay di 1.5 secondi prima del redirect
    this.showSecondModal=true;
  }
  goHome(){
    this.router.navigate(['auth/login/tabs']);
  }

  openSupport(){
    this.showSupport = this.modalFlag = true;
    setTimeout(() => {
      this.findModal('#supportModal');  // Chiamare il metodo dopo il ritardo
    }, 500);
  }

  @HostListener('document:keydown', ['$event'])
  handleKeyboardEvent(event: KeyboardEvent) {
    if (event.key === 'Escape' || event.key === 'Esc' || event.keyCode === 27) {
      this.closeModal();
    }
  }

  findModal(idMod: string){
    this.el = this.ref.nativeElement.querySelector(idMod)!;
    // Trova tutti gli elementi focalizzabili nella modale
    this.focusableElements = this.el!.querySelectorAll(
      'a[href], button, textarea, input[type="text"], input[type="radio"], input[type="checkbox"], select'
    );

    
    this.firstFocusable = this.focusableElements[0];
    this.lastFocusable = this.focusableElements[this.focusableElements.length - 1];
    this.firstFocusable.focus();
  }

  
}