import {Injectable} from "@angular/core";
import {Subject} from "rxjs";

export enum MessageType {
  Success = 'SUCCESS',
  Warning = 'WARNING',
  Error = 'ERROR'
}

@Injectable({
  providedIn: 'root'
})
export class MessageService {


  active: boolean = false;
  private messageSubject = new Subject<any>();

  public trigger(message: string, title: string, type: MessageType) {
    this.messageSubject.next({message: message, title: title, type: type})
    setTimeout(() => {
      this.stop()
    }, 3000)
    this.active = true;
  }

  public triggerValidationErrorList(errors: Object, title: string) {
    const message = `
    <ul>
    ${(Object.values(errors).map((e: string) => `<li>${e}</li>`)).toString().split(',').join('')}
    </ul>`
    this.trigger(message, title, MessageType.Error);
  }

  public stop() {
    this.active = false;
  }

  getMessage() {
    return this.messageSubject.asObservable();
  }


}
