package com.accenture.eVote.filems.controller;

import com.accenture.eVote.filems.dto.RicevutaVotoRequestDTO;
import com.accenture.eVote.filems.error.EmailException;
import com.accenture.eVote.filems.service.FileService;
import jakarta.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.mail.MailException;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.mail.MessagingException;
import java.io.IOException;

@RestController
@RequestMapping("/protected/file")
public class FileController {


    @Autowired
    private FileService fileService;

    @PostMapping(value= "/download", produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public ResponseEntity<Resource> getFile(@RequestBody @Valid RicevutaVotoRequestDTO ricevutaVotoRequestDTO) throws IOException {
        try {
            ByteArrayResource resource = new ByteArrayResource(fileService.getPDFRicevutaVoto(ricevutaVotoRequestDTO));
            HttpHeaders headers = new HttpHeaders();
            headers.add(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=RicevutaVoto");
            headers.add(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_PDF_VALUE);
            return new ResponseEntity<>(resource, headers, HttpStatus.OK);
        } catch (IOException e) {
            e.printStackTrace();
        }

        return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }

    @PostMapping(value= "/mail-ricevuta-voto")
    public ResponseEntity<String> sendMailWithPDF(@RequestBody @Valid RicevutaVotoRequestDTO ricevutaVotoRequestDTO) {

        try {
            fileService.sendMail(ricevutaVotoRequestDTO);// sendMail(ricevutaVotoRequestDTO);
            HttpHeaders headers = new HttpHeaders();
            headers.add(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=RicevutaVoto");
            headers.add(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_PDF_VALUE);

            return ResponseEntity
                    .status(HttpStatus.OK)
                    .headers(headers)
                    .body("Email inviato correttamente");

        } catch (IOException | MessagingException | EmailException | MailException e) {
            e.printStackTrace();
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(e.getMessage());
        }
    }

}
