package com.accenture.eVote.filems.service;

import com.accenture.eVote.filems.dto.ElettoreDTO;
import com.accenture.eVote.filems.dto.RicevutaVotoRequestDTO;
import com.accenture.eVote.filems.entity.DateElezioni;
import com.accenture.eVote.filems.error.EmailException;
import com.accenture.eVote.filems.repository.DateElezioniRepository;
import com.accenture.eVote.filems.util.enums.FaseElettoraleEnum;
import com.lowagie.text.DocumentException;
import org.apache.pdfbox.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.core.io.Resource;
import org.springframework.core.io.UrlResource;
import org.springframework.mail.MailException;
import org.springframework.stereotype.Service;
import org.xhtmlrenderer.pdf.ITextRenderer;

import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.sql.Date;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.Arrays;
import java.util.Base64;
import java.util.Properties;

import org.springframework.mail.javamail.MimeMessageHelper;
import javax.mail.*;
import javax.mail.internet.MimeMessage;

@Service
public class FileService {

    @Value("${path.template.ricevuta}")
    private String pathTemplateRicevuta;
    @Value("${path.logo.ricevuta}")
    private String pathLogoRicevuta;

    @Value("${mail.username}")
    private String username;

    @Value("${mail.password}")
    private String password;

    @Autowired
    private DateElezioniRepository dateElezioniRepository;

    public byte[] getPDFRicevutaVoto(RicevutaVotoRequestDTO requestDTO) throws IOException {

        DateElezioni dataElezione = new DateElezioni();
        ElettoreDTO elettoreDTO = requestDTO.getElettoreDTO();

        // Se l'idDataElezione non viene passato, andiamo a prendere l'elezione attiva
        if (requestDTO.getIdDataElezione() == null || !dateElezioniRepository.findById(requestDTO.getIdDataElezione()).isPresent()) {
            dataElezione = dateElezioniRepository.getElezioneAttiva(FaseElettoraleEnum.INIZIALIZZAZIONE.getValore(), FaseElettoraleEnum.ELIMINATA.getValore());
        } else {
            dataElezione = dateElezioniRepository.findById(requestDTO.getIdDataElezione()).get();
        }

        // Lettura del contenuto del html e del logo
        String htmlContent = readHtmlFile(pathTemplateRicevuta);
        String image = "data:image/png;base64," + convertToBase64(Paths.get(pathLogoRicevuta));

        // Replace the placeholder in the HTML content with the Java variable
        htmlContent = htmlContent.replace("${nome}", elettoreDTO.getNome());
        htmlContent = htmlContent.replace("${cognome}", elettoreDTO.getCognome());
        htmlContent = htmlContent.replace("${luogoNascita}", elettoreDTO.getLuogoNascita());
        htmlContent = htmlContent.replace("${dataNascita}", formatDate(elettoreDTO.getDataNascita()));
        htmlContent = htmlContent.replace("${dataElezione}", formatDate(dataElezione.getDataElezione()));
        htmlContent = htmlContent.replace("${todayDate}", formatDate(Date.valueOf(LocalDate.now().toString())));
        htmlContent = htmlContent.replace("${image}", image);

        return generatePdf(htmlContent);
    }

    public void sendMail(RicevutaVotoRequestDTO requestDTO) throws MessagingException, IOException, EmailException, MailException {

        String mail = requestDTO.getElettoreDTO().getEmail() != null && !requestDTO.getElettoreDTO().getEmail().isEmpty() ? requestDTO.getElettoreDTO().getEmail() :
                requestDTO.getElettoreDTO().getEmailPec() != null && !requestDTO.getElettoreDTO().getEmailPec().isEmpty() ? requestDTO.getElettoreDTO().getEmailPec() : null;

        if (mail == null) {
            throw new EmailException("Email dell'utente non presente nel database");
        }else {
            try {

                byte[] pdf = getPDFRicevutaVoto(requestDTO);

                Properties props = new Properties();
                props.put("mail.smtp.starttls.enable", "true");
                props.put("mail.smtp.host", "smtpintadv.interno.it");
                props.put("mail.smtp.port", "465");
                props.put("mail.smtp.auth", "true");
                props.put("mail.smtp.starttls.required", "true");
                props.put("mail.smtp.ssl.protocols", "TLSv1.2");
                props.put("mail.smtp.ssl.enable", "true");

                Session session = Session.getInstance(props, new javax.mail.Authenticator() {
                    @Override
                    protected PasswordAuthentication getPasswordAuthentication() {
                        return new PasswordAuthentication(username, password);
                    }
                });

                MimeMessage message = new MimeMessage(session);
                MimeMessageHelper helper = new MimeMessageHelper(message, true);

                helper.setFrom(username);


                helper.setTo(mail);

                helper.setSubject("Ricevuta voto");
                helper.setText("Salve, in allegato la ricevuta di voto.");
                helper.addAttachment("RicevutaVoto.pdf", new ByteArrayResource(pdf));
                Transport.send(message);

            } catch (Exception e) {
                e.printStackTrace();
            }
        }


    }

    private String formatDate(Date data) {
        DateTimeFormatter inputFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        LocalDate date = LocalDate.parse(data.toString(), inputFormatter);
        DateTimeFormatter outputFormatter = DateTimeFormatter.ofPattern("dd/MM/yyyy");
        return date.format(outputFormatter);
    }

    private static byte[] generatePdf(String htmlContent) throws IOException {
        try (ByteArrayOutputStream outputStream = new ByteArrayOutputStream()) {
            ITextRenderer renderer = new ITextRenderer();
            renderer.setDocumentFromString(htmlContent);
            renderer.layout();
            renderer.createPDF(outputStream);
            return outputStream.toByteArray();
        } catch (DocumentException e) {
            throw new RuntimeException(e);
        }
    }

    private static String readHtmlFile(String filePath) {
        try {
            Path path = Paths.get(filePath);
            return new String(Files.readAllBytes(path));
        } catch (IOException e) {
            e.printStackTrace();
            return "";
        }
    }

    private String convertToBase64(Path path) {
        byte[] imageAsBytes = new byte[0];
        try {
            Resource resource = new UrlResource(path.toUri());
            InputStream inputStream = resource.getInputStream();
            imageAsBytes = IOUtils.toByteArray(inputStream);

        } catch (IOException e) {

        }

        return Base64.getEncoder().encodeToString(imageAsBytes);
    }
}
