package com.accenture.eVote.schedems.controller;

import com.accenture.eVote.schedems.dto.ReqRicevutaVotoDTO;
import com.accenture.eVote.schedems.dto.ResponseSchedeUrna;
import com.accenture.eVote.schedems.dto.ResponseUrna;
import com.accenture.eVote.schedems.dto.SchedaDTO;
import com.accenture.eVote.schedems.entity.DateElezioni;
import com.accenture.eVote.schedems.model.request.ElettoreSchedaRequest;
import com.accenture.eVote.schedems.model.request.SchedaElezioneRequest;
import com.accenture.eVote.schedems.model.response.*;
import com.accenture.eVote.schedems.model.response.ElettoreSchedaResponse;
import com.accenture.eVote.schedems.model.response.ElezioneResponse;
import com.accenture.eVote.schedems.model.response.SchedaElezioneResponse;
import com.accenture.eVote.schedems.model.response.SchedeAttiveResponse;
import com.accenture.eVote.schedems.service.SchedaService;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.validation.Valid;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("/protected/scheda")
public class SchedaController {
    @Autowired
    SchedaService schedaService;
    @Value("${headers.session.id}")
    private String headerSessionId;
    @Value("${url.urnams.schede}")
    private String urlUrnaSchede;
    @Value("${url.urnams.votobc}")
    private String urlUrnaVotoBc;
    @Autowired
    private RestTemplate restTemplate;

    @GetMapping("")
    public ResponseEntity<SchedaDTO> getScheda(@RequestParam(name = "id") Long id) {
        return ResponseEntity.ok().body(schedaService.getSchedaById(id));
    }

    @PostMapping("/schede")
    public ResponseEntity<ElettoreSchedaResponse> getElettoreSchede(@RequestBody @Valid ElettoreSchedaRequest elettoreSchedaRequest, HttpServletRequest request) {
        Long userId = Long.valueOf(request.getHeader(headerSessionId));
        return ResponseEntity.ok().body(schedaService.getElettoreSchede(elettoreSchedaRequest, userId));
    }

    @PostMapping("/elezione")
    public ResponseEntity<SchedaElezioneResponse> getSchedaElezione(@RequestBody @Valid SchedaElezioneRequest schedaElezioneRequest) {
        return ResponseEntity.ok().body(schedaService.getSchedaElezioneByIdScheda(schedaElezioneRequest));
    }

    @GetMapping("/schede-accettate")
    public ResponseEntity<ElettoreSchedaResponse> getSchedeAccettate(@RequestParam(value = "idDataElezione", required = false) Long idDataElezione,
                                                                     HttpServletRequest request) {
        Long userId = Long.valueOf(request.getHeader(headerSessionId));

        if (userId == null) {
            throw new IllegalArgumentException("L'utente non può essere null");
        }
        return ResponseEntity.ok().body(schedaService.getSchedeAccettate(userId, idDataElezione));
    }

    @GetMapping("/storico-votazioni")
    public ResponseEntity<List<DateElezioni>> getStoricoVotazioni(HttpServletRequest request) {
        Long userId = Long.valueOf(request.getHeader(headerSessionId));
        return ResponseEntity.ok(schedaService.getStoricoVotazioni(userId));
    }

    @GetMapping("/elezione")
    public ResponseEntity<List<ElezioneResponse>> getElezioneByScheda(@RequestParam @Valid List<Long> idSchede) {
        return ResponseEntity.ok(schedaService.getElezioneByScheda(idSchede));
    }

    @PostMapping("/ricevuta-schede")
    public ResponseEntity<ReqRicevutaVotoDTO> setRicevutaSchede(@RequestBody @Valid ReqRicevutaVotoDTO reqRicevutaVotoDTO, HttpServletRequest request) {
        Long userId = Long.valueOf(request.getHeader(headerSessionId));
        schedaService.setRicevutaSchede(reqRicevutaVotoDTO, userId);
        return ResponseEntity.ok().body(reqRicevutaVotoDTO);
    }

    @GetMapping("/count-votanti")
    public ResponseEntity<CountVotantiResponse> getCountVotanti(){
        ResponseSchedeUrna respUrna = restTemplate.getForEntity(urlUrnaSchede, ResponseSchedeUrna.class).getBody();
        return ResponseEntity.ok().body(schedaService.getCountVotanti(respUrna));
    }

    @GetMapping("/votanti-digitali-cda")
    public ResponseEntity<CountVotantiResponse> getVotantiDigitaliCda(Long idDataElezione, String prefettura) {

        // Recupero la lista di geoId dalla prefettura
        List<String> listGeoId = schedaService.findTerritoriByPrefettura(idDataElezione, prefettura);

        // Recupero i voti per quei geoId
        ResponseUrna respUrna = restTemplate.postForEntity(urlUrnaVotoBc, listGeoId, ResponseUrna.class).getBody();

        if(respUrna.getVotoBcList().isEmpty()) {
            return ResponseEntity.ok().body(new CountVotantiResponse(0, 0));
        } else {
            List<String> listIdSchedeUrna = new ArrayList<>();
            respUrna.getVotoBcList().forEach(voto -> listIdSchedeUrna.add(voto.getIdSchedeVotate()));
            return ResponseEntity.ok().body(schedaService.getCountVotanti(new ResponseSchedeUrna(listIdSchedeUrna)));
        }
    }

    @GetMapping("/verifica/id")
    public ResponseEntity<SchedaVerificaIdResponse> getCheckId(@RequestParam(name = "idScheda")  @Valid Long idScheda) {
        return ResponseEntity.ok(schedaService.getSchedaCheck(idScheda));
    }

    @GetMapping("/schede-attive")
    public ResponseEntity<SchedeAttiveResponse> getSchedeAttive() {
        return ResponseEntity.ok().body(schedaService.getSchedeValide());
    }

}
