package com.accenture.eVote.schedems.mapper;

import com.accenture.eVote.schedems.dto.*;
import com.accenture.eVote.schedems.entity.CoterCoaliTerpa;
import com.accenture.eVote.schedems.entity.ElettoreScheda;
import com.accenture.eVote.schedems.entity.SchedaFacSimile;
import com.accenture.eVote.schedems.repository.ElettoreSchedaRepository;
import org.mapstruct.Mapper;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


@Mapper(componentModel = "spring")
public class SchedaElettoreMapper {

    @Autowired
    private SchedaFacSimileMapper schedaFacSimileMapper;
    @Autowired
    private ElettoreSchedaRepository elettoreSchedaRepository;

    public SchedaDTO toSchedaDTO(SchedaFacSimileDTO schedaFacSimileDTO, List<CoterCoaliTerpa> coterCoaliTerpas) {
        SchedaDTO schedaDTO = new SchedaDTO();

        // Setting dati univoci scheda
        schedaDTO.setIdScheda(schedaFacSimileDTO.getId());
        schedaDTO.setIdElezione(schedaFacSimileDTO.getTerritorio().getIdElezione());
        schedaDTO.setIdTerritorio(schedaFacSimileDTO.getTerritorio().getId());
        schedaDTO.setDescTerritorio(schedaFacSimileDTO.getTerritorio().getDescrizioneEnte());
        schedaDTO.setDataElezione(schedaFacSimileDTO.getTerritorio().getElezione().getDateElezioni().getDataElezione());
        schedaDTO.setDescrizioneElezione(schedaFacSimileDTO.getTerritorio().getElezione().getTipoElezioni().getDescrizione());

        // ordinamento liste
        coterCoaliTerpas.sort(Comparator.comparingLong(CoterCoaliTerpa::getOrdinamentoLista));
        Map<Long, Long> mappaOrdinamentoListe = coterCoaliTerpas.stream()
                .collect(Collectors.toMap(CoterCoaliTerpa::getIdLista, CoterCoaliTerpa::getOrdinamentoLista));

        // recupero delle liste e ordinamento
        List<ListaSchedaDTO> liste = schedaFacSimileDTO.getSchedaFacSimileListas()
                .stream()
                .map(scheda -> new ListaSchedaDTO(scheda.getLista()))
                .sorted(Comparator.comparing(lista -> mappaOrdinamentoListe.getOrDefault(lista.getLista().getId(), Long.MAX_VALUE)))
                .collect(Collectors.toList());

        // recupero dei candidati per ogni lista

        liste.stream()
                .forEach(oggetto -> {
                    List<CandidatoDTO> candidatoDTO = schedaFacSimileDTO.getSchedaFacSimileListas().stream()
                            .filter(lis -> lis.getIdLista().compareTo(oggetto.getLista().getId()) == 0)
                            .flatMap(l -> schedaFacSimileDTO.getSchedaFacSimileCandidatiListas().stream()
                                    .filter(can -> can.getIdLista().compareTo(oggetto.getLista().getId()) == 0)
                                    .map(SchedaFacSimileCandidatiListaDTO::getCandidatoLista)
                                    .sorted(Comparator.comparingLong(candidatoLista -> candidatoLista.getNumCandidatura()))
                                    .map(CandidatoListaDTO::getCandidato))
                            .collect(Collectors.toList());
                    oggetto.setCandidatoLista(candidatoDTO);
                });

        schedaDTO.setListaSchedaDTOList(liste);

        schedaDTO.setCandidatoLeaderListe(schedaFacSimileDTO.getSchedaFacSimileCans().stream()
                .map(SchedaFacSimileCanDTO::getCandidatoLeader)
                .collect(Collectors.toList()).stream()
                .map(CandidatoListaTerritorioDTO::getCandidatoLeader)
                .collect(Collectors.toList()));

        return schedaDTO;
    }

    public SchedeElettoreDTO toSchedeElettoreDTO(SchedaFacSimile schedaFacSimile, Long userId) {
        SchedeElettoreDTO schedeElettoreDTO = new SchedeElettoreDTO();

        //rimozione territorio padre
        schedaFacSimile.getTerritorio().setTerritorioPadre(null);

        schedeElettoreDTO.setIdScheda(schedaFacSimile.getId());
        schedeElettoreDTO.setIdElezione(schedaFacSimile.getTerritorio().getIdElezione());
        schedeElettoreDTO.setDescrizioneElezione(schedaFacSimile.getTerritorio().getElezione().getTipoElezioni().getDescrizione());
        schedeElettoreDTO.setTerritorio(schedaFacSimileMapper.toDto(schedaFacSimile).getTerritorio());
        ElettoreScheda elettoreScheda = elettoreSchedaRepository.findBySchedaVotoAndUserId(schedaFacSimile.getId(), userId);
        if (elettoreScheda != null) {
            schedeElettoreDTO.setFlagAccettazione(elettoreScheda.getFlagAccettazione());
        } else {
            schedeElettoreDTO.setFlagAccettazione(null);
        }

        return schedeElettoreDTO;

    }

}
