package com.accenture.eVote.schedems.service;

import com.accenture.eVote.schedems.dto.ReqRicevutaVotoDTO;
import com.accenture.eVote.schedems.dto.ResponseSchedeUrna;
import com.accenture.eVote.schedems.dto.SchedaDTO;
import com.accenture.eVote.schedems.dto.SchedeElettoreDTO;
import com.accenture.eVote.schedems.entity.*;
import com.accenture.eVote.schedems.entity.id.IdSchedaPreferenze;
import com.accenture.eVote.schedems.mapper.ElettoreSchedaMapper;
import com.accenture.eVote.schedems.mapper.SchedaElettoreMapper;
import com.accenture.eVote.schedems.mapper.SchedaFacSimileMapper;
import com.accenture.eVote.schedems.mapper.SchedaVerificaIdResponseMapper;
import com.accenture.eVote.schedems.model.request.ElettoreSchedaRequest;
import com.accenture.eVote.schedems.model.request.SchedaElezioneRequest;
import com.accenture.eVote.schedems.model.response.*;
import com.accenture.eVote.schedems.model.response.ElettoreSchedaResponse;
import com.accenture.eVote.schedems.model.response.ElezioneResponse;
import com.accenture.eVote.schedems.model.response.SchedaElezioneResponse;
import com.accenture.eVote.schedems.model.response.SchedeAttiveResponse;
import com.accenture.eVote.schedems.repository.*;
import com.accenture.eVote.schedems.util.enums.ElettoriEnum;
import com.accenture.eVote.schedems.util.enums.FaseElettoraleEnum;
import com.accenture.eVote.schedems.util.enums.TipoElezioneEnum;
import jakarta.annotation.Nullable;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class SchedaService {

    @Autowired
    private SchedaFacSimileRepository schedaFacSimileRepository;
    @Autowired
    private ElettoreEnteRepository elettoreEnteRepository;
    @Autowired
    private TerritorioRepository territorioRepository;
    @Autowired
    private SchedaPreferenzeRepository schedaPreferenzeRepository;
    @Autowired
    private CoterCoaliTerpaRepository coterCoaliTerpaRepository;
    @Autowired
    private SchedaFacSimileMapper schedaFacSimileMapper;
    @Autowired
    private SchedaElettoreMapper schedaElettoreMapper;
    @Autowired
    private ElettoreSchedaRepository elettoreSchedaRepository;
    @Autowired
    private ElettoreSchedaMapper elettoreSchedaMapper;
    @Autowired
    private ElezioniRepository elezioniRepository;
    @Autowired
    private RicevutaSchedaRepository ricevutaSchedaRepository;
    @Autowired
    private SchedaVerificaIdResponseMapper schedaVerificaIdResponseMapper;
    @Value("${formati.request.data}")
    private String formatoData;
    @Value("${costanti.flag.yes}")
    private String flagYes;
    @Value("${costanti.flag.no}")
    private String flagNo;

    public SchedaDTO getSchedaById(Long id) {
        // Recupero scheda completa
        SchedaFacSimile schedaFacSimile = schedaFacSimileRepository.findById(id).get();

        // recupero ordinamento liste
        List<CoterCoaliTerpa> coterCoaliTerpas = coterCoaliTerpaRepository.findByIdTerritorio(schedaFacSimile.getIdTerritorio());

        // Mapping dei dati richiesti
        SchedaDTO schedaDTO = schedaElettoreMapper.toSchedaDTO(schedaFacSimileMapper.toDto(schedaFacSimile), coterCoaliTerpas);

        // Recupero preferenze schede
        // Setting id preferenze scheda
        IdSchedaPreferenze idPreferenzeScheda = new IdSchedaPreferenze();
        idPreferenzeScheda.setGeoId(schedaFacSimile.getTerritorio().getGeoId());
        idPreferenzeScheda.setTipoElezione(schedaFacSimile.getTerritorio().getElezione().getTipoElezioni().getId());

        schedaDTO.setNumPreferenze(schedaPreferenzeRepository.findById(idPreferenzeScheda).get().getNumeroPreferenze());

        return schedaDTO;
    }

    public ElettoreSchedaResponse getElettoreSchede(ElettoreSchedaRequest elettoreSchedaRequest, Long userId) {

        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(formatoData);
        ElettoreSchedaResponse elettoreSchedaResponse = new ElettoreSchedaResponse();

        // recupero lista geoId su CL_ELEVE_ELEVOENTE

        List<ElettoreEnte> elettoreEnteList = elettoreEnteRepository.findByIdElettore(userId);
        List<String> geoIdList = elettoreEnteList.stream()
                .map(ElettoreEnte::getGeoId)
                .collect(Collectors.toList());

        // recupero terpa da lista GEOid e data elezione
        // risalgo al primo territorio padre
        List<Territorio> territorioList = territorioRepository.findByGeoIdIn(geoIdList).stream()
                .map(territorio -> {
                    return trovaTerritorioPadrePrimo(territorio);
                })
                .collect(Collectors.toList());

        List<Long> idTerritorioList = territorioList.stream()
                .filter(territorio -> simpleDateFormat.format(territorio
                        .getElezione()
                        .getDateElezioni()
                        .getDataElezione()).compareTo(simpleDateFormat.format(elettoreSchedaRequest.getDataElezione())) == 0)
                .map(Territorio::getId)
                .collect(Collectors.toList());

        // recupero schede da lista terpa

        List<SchedaFacSimile> schedaFacSimileList = schedaFacSimileRepository.findByidTerritorioIn(idTerritorioList);

        List<SchedeElettoreDTO> schedaDTOList = schedaFacSimileList.stream()
                .map(scheda -> schedaElettoreMapper.toSchedeElettoreDTO(scheda, userId))
                .collect(Collectors.toList());

        // mapping su response degli id Scheda
        elettoreSchedaResponse.setSchede(schedaDTOList);

        return elettoreSchedaResponse;
    }


    public SchedaElezioneResponse getSchedaElezioneByIdScheda(SchedaElezioneRequest schedaElezioneRequest) {

        SchedaElezioneResponse schedaElezioneResponse = new SchedaElezioneResponse();
        // recupero la scheda
        List<SchedaFacSimile> schedaFacSimileList = schedaFacSimileRepository.findAllById(schedaElezioneRequest.getIdSchedaLista());
        List<List<SchedaFacSimileCan>> schedaFacSimileCanListDoppia = schedaFacSimileList.stream()
                .map(SchedaFacSimile::getSchedaFacSimileCans)
                .collect(Collectors.toList());

        schedaElezioneResponse.setTipoElezioniList(schedaFacSimileList.stream()
                // Ordinamento in base alla request
                .sorted(Comparator.comparing(obj -> {
                    long id = obj.getId();
                    long index = schedaElezioneRequest.getIdSchedaLista().indexOf(id);
                    return index == -1 ? Long.MAX_VALUE : index;
                }))
                .map(SchedaFacSimile::getTerritorio)
                .map(Territorio::getElezione)
                .map(Elezioni::getTipoElezioni)

                .collect(Collectors.toList()));

        return schedaElezioneResponse;
    }

    public ElettoreSchedaResponse getSchedeAccettate(Long userId, @Nullable Long idDataElezione) {
        List<Long> listSchedeAccettate = elettoreSchedaRepository.findIdsSchedeAccettate(userId, ElettoriEnum.SCHEDA_ACCETTATA.getValore());

        ElettoreSchedaResponse elettoreSchedaResponse = new ElettoreSchedaResponse();

        // recupero lista geoId su CL_ELEVE_ELEVOENTE

        List<ElettoreEnte> elettoreEnteList = elettoreEnteRepository.findByIdElettore(userId);
        List<String> geoIdList = elettoreEnteList.stream()
                .map(ElettoreEnte::getGeoId)
                .collect(Collectors.toList());

        // recupero gli idElezioni da idDataElezione

        if (idDataElezione == null) {
            idDataElezione = elezioniRepository.getElezioneAttiva(FaseElettoraleEnum.ELIMINATA.getValore()).get(0);
        }

        List<Elezioni> listElezioni = elezioniRepository.findByIdDataElezione(idDataElezione);

        List<Long> listIdElezioni = listElezioni.stream()
                .map(Elezioni::getIdElezione)
                .collect(Collectors.toList());

        // recupero terpa da lista GEOid e data elezione
        // risalgo al primo territorio padre
        List<Territorio> territorioList = territorioRepository.findByGeoIdIn(geoIdList).stream()
                .map(territorio -> {
                    return trovaTerritorioPadrePrimo(territorio);
                })
                .collect(Collectors.toList());

        List<Long> idTerritorioList = territorioList.stream()
                .filter(territorio -> listIdElezioni.contains(territorio.getIdElezione()))
                .map(Territorio::getId)
                .collect(Collectors.toList());

        // recupero schede da lista terpa

        List<SchedaFacSimile> schedaFacSimileList = schedaFacSimileRepository.findByidTerritorioIn(idTerritorioList);

        List<SchedeElettoreDTO> schedaDTOList = schedaFacSimileList.stream()
                .filter(scheda -> listSchedeAccettate.contains(scheda.getId()))
                .map(scheda -> schedaElettoreMapper.toSchedeElettoreDTO(scheda, userId))
                .collect(Collectors.toList());

        // mapping su response degli id Scheda

        elettoreSchedaResponse.setSchede(schedaDTOList);

        return elettoreSchedaResponse;

    }


    public List<DateElezioni> getStoricoVotazioni(Long idUser) {
        return ricevutaSchedaRepository.findStoricoDate(idUser);
    }

    private static Territorio trovaTerritorioPadrePrimo(Territorio territorio) {
        while (territorio.getTerritorioPadre() != null) {
            territorio = territorio.getTerritorioPadre();
        }
        return territorio;
    }

    public List<ElezioneResponse> getElezioneByScheda(List<Long> listaIdSchede) {
        List<ElezioneResponse> elezioneResponseList = new ArrayList<>();

        elezioneResponseList = listaIdSchede.stream()
                .map(idScheda -> new ElezioneResponse(schedaFacSimileRepository.findById(idScheda).get().getTerritorio().getIdElezione(), idScheda))
                .collect(Collectors.toList());

        return elezioneResponseList;
    }

    public void setRicevutaSchede(ReqRicevutaVotoDTO reqRicevutaVotoDTO, Long userId) {
        reqRicevutaVotoDTO.getListaIdSchedeVotate().forEach(idScheda -> {
            ricevutaSchedaRepository.save(new RicevutaScheda(userId, idScheda, flagYes));
        });
        reqRicevutaVotoDTO.getListaIdSchedeNonVotate().forEach(idScheda -> {
            ricevutaSchedaRepository.save(new RicevutaScheda(userId, idScheda, flagNo));
        });
    }

    public CountVotantiResponse getCountVotanti(ResponseSchedeUrna respUrna) {

        // Lista con tutti gli id delle schede votate
        List<Long> listIdSchedeVotate = new ArrayList<>();

        respUrna.getIdSchedeVotate().forEach(record -> {

            if (record != null) {
                listIdSchedeVotate.addAll(Arrays.stream(record.split(";"))
                        .map(Long::parseLong)
                        .collect(Collectors.toList()));
            }
        });

        // Mapa che contiene come chiave l'id della scheda e come value la count di voti per quella scheda
        Map<Long, Integer> countMap = new HashMap<>();
        listIdSchedeVotate.forEach(idScheda -> countMap.put(idScheda, countMap.getOrDefault(idScheda, 0) + 1));

        CountVotantiResponse count = new CountVotantiResponse();

        // Corrispondenza della key (idScheda) con il tipo di elezione: camera o senato
        countMap.forEach((key, value) -> {
            Elezioni elezione = elezioniRepository.findElezioneByIdScheda(key);
            if(elezione != null) {
                if(elezione.getTipoElezioni().getId().equals(TipoElezioneEnum.CAMERA_ESTERO.getValore())) {
                    count.setCountVotantiCamera(value);
                } else if(elezione.getTipoElezioni().getId().equals(TipoElezioneEnum.SENATO_ESTERO.getValore())) {
                    count.setCountVotantiSenato(value);
                }
            }
        });

        return count;
    }

    public List<String> findTerritoriByPrefettura(Long idDataElezione, String prefettura) {
        List<Elezioni> listIdElezioni = elezioniRepository.findByIdDataElezione(idDataElezione);
        List<Long> listId = new ArrayList<>();
        listIdElezioni.forEach(elezione -> listId.add(elezione.getIdElezione()));

        List<Territorio> listTerritorio = territorioRepository.findByIdElezioneAndGeoIdPfGuida(listId, prefettura);
        List<Long> listIdTerritorioPadre = new ArrayList<>();
        listTerritorio.forEach(territorio -> {
            listIdTerritorioPadre.add(territorio.getIdTerritorioPadre());
        });
        return territorioRepository.findGeoIdByListTerritoriPadre(listIdTerritorioPadre);
    }

    public SchedaVerificaIdResponse getSchedaCheck(Long id) {
        // Recupero scheda completa
        SchedaFacSimile schedaFacSimile = schedaFacSimileRepository.findById(id).get();

        // recupero ordinamento liste
        List<CoterCoaliTerpa> coterCoaliTerpas = coterCoaliTerpaRepository.findByIdTerritorio(schedaFacSimile.getIdTerritorio());

        // Mapping dei dati richiesti
        SchedaDTO schedaDTO = schedaElettoreMapper.toSchedaDTO(schedaFacSimileMapper.toDto(schedaFacSimile), coterCoaliTerpas);

        // Recupero preferenze schede
        // Setting id preferenze scheda
        IdSchedaPreferenze idPreferenzeScheda = new IdSchedaPreferenze();
        idPreferenzeScheda.setGeoId(schedaFacSimile.getTerritorio().getGeoId());
        idPreferenzeScheda.setTipoElezione(schedaFacSimile.getTerritorio().getElezione().getTipoElezioni().getId());

        schedaDTO.setNumPreferenze(schedaPreferenzeRepository.findById(idPreferenzeScheda).get().getNumeroPreferenze());

        // Mapping del numero preferenze e degli id da verificare della lista
        return schedaVerificaIdResponseMapper.toSchedaVerificaIdResponse(schedaDTO);
    }

    public SchedeAttiveResponse getSchedeValide() {

        // Recupero tutte le schede e filtro quelle con codice fase elettorale a 2
        return new SchedeAttiveResponse(schedaFacSimileRepository.findAll().stream()
                .filter(scheda -> scheda.getTerritorio().getElezione().getCodFaseElettorale().equals(2L))
                .map(SchedaFacSimile::getId)
                .collect(Collectors.toList()));
    }
}
