package it.evota.generate.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import com.fasterxml.jackson.databind.ObjectMapper;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import it.evota.generate.dto.ErrorResponseType;
import it.evota.generate.dto.GenerateAQRequestType;
import it.evota.generate.dto.GenerateAQResponseType;
import it.evota.generate.service.BcGenerateService;

@RestController
@ConditionalOnExpression("${generate.enabled:false}")
public class GenerateQualifyAttributeController {

	@Autowired
	BcGenerateService service;
	
	@Autowired
	private ObjectMapper objectMapper;
	

	


	@Operation(summary = "Generate a new blockchain record")
	@ApiResponses(value = { @ApiResponse(responseCode = "200", description = "Ok", content = {
			@Content(mediaType = "application/json", schema = @Schema(implementation = GenerateAQResponseType.class)) }),
			@ApiResponse(responseCode = "422", description = "Qualified Attribute Used", content = {
					@Content(mediaType = "application/problem+json", schema = @Schema(implementation = ErrorResponseType.class)) }),
			@ApiResponse(responseCode = "500", description = "Internal server error", content = {
					@Content(mediaType = "application/problem+json", schema = @Schema(implementation = ErrorResponseType.class)) }) })
	@PostMapping(value = "/evote/qa/generate", consumes = { MediaType.APPLICATION_OCTET_STREAM_VALUE }, produces = {
			MediaType.APPLICATION_JSON_VALUE, MediaType.APPLICATION_PROBLEM_JSON_VALUE })
	public ResponseEntity<Object> generate(@RequestBody byte[] data) {
		GenerateAQRequestType evoteRequestType=null;
		try {
			return ResponseEntity.ok().contentType(MediaType.APPLICATION_JSON).body(new GenerateAQResponseType(service.createRecord(evoteRequestType.getUserId(),evoteRequestType.getUserPreferences())));
		} catch (Exception e) {
			e.printStackTrace();
			return ResponseEntity.internalServerError().contentType(MediaType.APPLICATION_PROBLEM_JSON)
					.body(new ErrorResponseType(e.getClass().getName(), e.getMessage()));
		}finally {
			if(evoteRequestType!=null) {
			}
		}

	}

}