package it.evota.generate.controller;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import it.evota.generate.dto.ErrorResponseType;
import it.evota.generate.service.BcGenerateService;
import it.evota.generate.util.dto.FindAQRequestType;
import it.evota.generate.util.dto.ListaAttrQualBcDto;
import it.evota.generate.util.exception.EVoteAppException;
import it.evota.generate.util.exception.EVoteAppException.ErrorCode;

@RestController
public class ListaAttrQualBcController {

	@Autowired
	BcGenerateService service;


	@Operation(summary = "Find by Attributo Qualificante", description = "Retrieve ListaAttrQualBc by Attributo Qualificante")
	@ApiResponses(value = {
			@ApiResponse(responseCode = "200", description = "Successful retrieval of ListaAttrQualBc", content = @Content(mediaType = "application/json")),
			@ApiResponse(responseCode = "404", description = "ListaAttrQualBc not found") })
	@PostMapping(value = "/evote/qa/findByAttributoQualificante", consumes = {
			MediaType.APPLICATION_JSON_VALUE }, produces = { MediaType.APPLICATION_JSON_VALUE,
					MediaType.APPLICATION_PROBLEM_JSON_VALUE })
	public ResponseEntity<?> findByAttributoQualificante(@RequestBody FindAQRequestType findAQRequest) {
		try {
			ListaAttrQualBcDto listaAttrQualBcDto = service
					.findByAttributoQualificante(findAQRequest.getQualifiedAttribute());
			if (listaAttrQualBcDto != null) {
				return new ResponseEntity<>(listaAttrQualBcDto, HttpStatus.OK);
			} else {
				return ResponseEntity.status(HttpStatus.NOT_FOUND).contentType(MediaType.APPLICATION_PROBLEM_JSON)
						.body(new ErrorResponseType(ErrorCode.FIND_BY_ATTR_QUAL_ERROR_001.name(),
								ErrorCode.FIND_BY_ATTR_QUAL_ERROR_001.getDescription()));
			}
		} catch (EVoteAppException e) {
			return ResponseEntity.status(422).contentType(MediaType.APPLICATION_PROBLEM_JSON)
					.body(new ErrorResponseType(e.getErrorCode().name(), e.getErrorCode().getDescription()));

		} catch (Exception e) {
			return ResponseEntity.internalServerError().contentType(MediaType.APPLICATION_PROBLEM_JSON)
					.body(new ErrorResponseType(e.getClass().getName(), e.getMessage()));
		}
	}

}