package it.evota.generate.service;

import java.security.PrivateKey;
import java.security.PublicKey;
import java.sql.Clob;
import java.sql.SQLException;
import java.util.Date;
import java.util.Optional;


import org.hibernate.engine.jdbc.ClobProxy;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.DependsOn;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import it.evota.generate.entities.ListaAttrQualBc;
import it.evota.generate.entities.ListaCertificati;
import it.evota.generate.repository.ListaAttrQualBcRepository;
import it.evota.generate.repository.ListaCertificatiRepository;
import it.evota.generate.util.dto.ListaAttrQualBcDto;
import it.evota.generate.util.exception.EVoteAppException;
import it.evota.generate.util.exception.EVoteAppException.ErrorCode;

@Service
@DependsOn("listaAttrQualBcRepository")
public class BcGenerateService {

	@Autowired
	ListaAttrQualBcRepository repo;

	@Autowired
	ListaCertificatiRepository repoCert;
	

	


	private PrivateKey priv = null;
	private long privScadenza = 0;
	private PublicKey pub = null;
	private long pubScadenza = 0;

	public synchronized void readKeys() throws Exception {
		if (priv == null || privScadenza <= System.currentTimeMillis()) {
			ListaCertificati fePriv = repoCert
					.findByStartDateIsNullAndEndDateIsNullOrStartDateBeforeAndEndDateAfterAndCertificateCodeEquals(
							new Date(), new Date(), "fe_priv");
			if (fePriv != null) {
				privScadenza = fePriv.getEndDate().getTime();
			} else {
				throw new EVoteAppException(ErrorCode.EVOTE_ERROR_INIT.name(), ErrorCode.EVOTE_ERROR_INIT);
			}
		}

		if (pub == null || pubScadenza <= System.currentTimeMillis()) {
			ListaCertificati bePub = repoCert
					.findByStartDateIsNullAndEndDateIsNullOrStartDateBeforeAndEndDateAfterAndCertificateCodeEquals(
							new Date(), new Date(), "be_pub");
			if (bePub != null) {
				pubScadenza = bePub.getEndDate().getTime();
			} else {
				throw new EVoteAppException(ErrorCode.EVOTE_ERROR_INIT.name(), ErrorCode.EVOTE_ERROR_INIT);
			}
		}

	}

	public PrivateKey getPrivateKey() throws Exception {
		if (priv == null || privScadenza <= System.currentTimeMillis())
			readKeys();
		return priv;
	}

	public PublicKey getPublicKey() throws Exception {
		if (pub == null || pubScadenza <= System.currentTimeMillis())
			readKeys();
		return pub;
	}

	@Transactional
	public String createRecord(String userId, String userPreferences) throws Exception {
		//Codice eliminato per motivi di sicurezza
		return null;
	}

	@Transactional
	public ListaAttrQualBcDto findByAttributoQualificante(String attributoQualificante) throws EVoteAppException {
		Clob clob = ClobProxy.generateProxy(attributoQualificante);
		Optional<ListaAttrQualBc> result = repo.findByAttributoQualificante(clob);
		if (result.isPresent()) {
			try {
				return convertToDto(result.get());
			} catch (SQLException e) {
				throw new EVoteAppException(ErrorCode.FIND_BY_ATTR_QUAL_ERROR_001.name(),
						ErrorCode.FIND_BY_ATTR_QUAL_ERROR_001);
			}
		} else {
			return null;
		}
	}

	private ListaAttrQualBcDto convertToDto(ListaAttrQualBc entity) throws SQLException {
		ListaAttrQualBcDto dto = new ListaAttrQualBcDto();

		dto.setId(entity.getId());
		dto.setUservoteid(entity.getUservoteid());
		dto.setTokenvoto(entity.getTokenvoto());
		dto.setAccettazione(entity.getAccettazione().getSubString(1, (int) entity.getAccettazione().length()));
		dto.setQualifiedAttribute(
				entity.getAttributoQualificante().getSubString(1, (int) entity.getAttributoQualificante().length()));
		return dto;
	}

}
