package com.accenture.eVote.votoms.controller;

import com.accenture.eVote.votoms.dto.ResponseVotoOracleDTO;
import com.accenture.eVote.votoms.service.VotoService;
import com.accenture.eVote.votoms.util.CustomError;
import com.accenture.eVote.votoms.util.exception.IncongruenzaSchedeCandidatiException;
import jakarta.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.HttpStatusCode;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.server.ResponseStatusException;
import reactor.core.publisher.Mono;

@RestController
@RequestMapping("/protected/voto/votazione")
public class VotoController {

    private final WebClient.Builder webClientBuilder;
    public VotoController(WebClient.Builder webClientBuilder) {
        this.webClientBuilder = webClientBuilder;
    }

    @Value("${url.oracle.evote.submit}")
    private String urlOracleEvoteSubmit;

    @Autowired
    private VotoService votoService;

    @GetMapping("/timer/{id}")
    public ResponseEntity<Object>  getTimer (@PathVariable(value="id", required = true )Long tipologiaElezione){
        if(tipologiaElezione == null) {
            throw new IllegalArgumentException("La tipologia elezione non può essere null");
        }
        Long timer = votoService.getTimer(tipologiaElezione);

        if (timer != null) {
            return ResponseEntity.ok().body(timer);
        } else {
            String errorMessage = "Il timer non 'e configurato.";
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(errorMessage);
        }
    }

    @PostMapping("/conferma-voto")
    public ResponseEntity<?> setVoto(@RequestBody @Valid String requestVotoDTO) throws Exception {
        byte[] requestCrittata = votoService.setVoto(requestVotoDTO);

        try {
            ResponseVotoOracleDTO response = webClientBuilder.build()
                    .post()
                    .uri(urlOracleEvoteSubmit)
                    .bodyValue(requestCrittata)
                    .retrieve()
                    .onStatus(HttpStatusCode::is4xxClientError, clientResponse -> Mono.error(new ResponseStatusException(HttpStatus.UNAUTHORIZED, "Attributo qualificato non valido")))
                    .onStatus(HttpStatusCode::is5xxServerError, clientResponse -> Mono.error(new ResponseStatusException(HttpStatus.INTERNAL_SERVER_ERROR, "Internal server error")))
                    .bodyToMono(ResponseVotoOracleDTO.class)
                    .block();
            return ResponseEntity.ok(response);
        } catch (ResponseStatusException e) {
            return new ResponseEntity<>(new CustomError(e.getReason(), e.getStatusCode()), e.getStatusCode());
        }

    }

    @ExceptionHandler(IncongruenzaSchedeCandidatiException.class)
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    public String handleIncongruenzaSchedeCandidatiException(IncongruenzaSchedeCandidatiException e){
        return e.getMessage();
    }

}
