package com.accenture.eVote.votoms.util;

import com.accenture.eVote.votoms.dto.CandidatoSchedaDTO;
import com.accenture.eVote.votoms.dto.RequestVotoDTO;
import com.accenture.eVote.votoms.dto.SchedeVotoDTO;
import com.accenture.eVote.votoms.model.SchedaVerificaIdResponse;
import com.accenture.eVote.votoms.repository.CandidatoRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Component
public class ValidatoreVoto {

    @Autowired
    private CandidatoRepository candidatoRepository;

    public Boolean validazioneVoto(RequestVotoDTO schedeVoto, List<SchedaVerificaIdResponse> schedeModello) {

        // Verifica numero schede passate
        List<Boolean> checkNumeroSchede = Collections.singletonList(schedeVoto.getListSchedeVoto().size() <= 2);

        // Verifica id schede univoci
        Map<Long, Long> mappaSchede = schedeVoto.getListSchedeVoto().stream()
                .collect(Collectors.groupingBy(SchedeVotoDTO::getChiaveScheda, Collectors.counting()));

        List<Boolean> checkUnivocitaSchede = Collections.singletonList(mappaSchede.values().stream().anyMatch(count -> count <= 1));

        // Recupero booleani per numero preferenze espresse
        List<Boolean> checkNumeroPreferenze = schedeVoto.getListSchedeVoto().stream()
                .map(schedeVotoDTO -> {
                    // Recupero preferenze espresse per la scheda corrente
                    int preferenzeEspresse = schedeVotoDTO.getLista().getListaCandidati().size();
                    // Recupero preferenze consentite per la scheda corrente
                    int preferenzeConsentite = Math.toIntExact(schedeModello.stream()
                            .filter(scheda -> scheda.getIdScheda().equals(schedeVotoDTO.getChiaveScheda()))
                            .map(SchedaVerificaIdResponse::getNumeroPreferenze)
                            .toList()
                            .get(0));
                    return preferenzeEspresse <= preferenzeConsentite;
                }).collect(Collectors.toList());

        // Recupero booleani per id lista e candidati inviati

        List<Boolean> checkCongruenzaIdListe = schedeVoto.getListSchedeVoto().stream()
                .filter(scheda -> scheda.getBianca().equals(Boolean.FALSE) && scheda.getVotata().equals(Boolean.TRUE))
                .map(schedeVotoDTO -> verificaListe(schedeVotoDTO, schedeModello.stream().filter(modello -> modello.getIdScheda().equals(schedeVotoDTO.getChiaveScheda())).findFirst().get()))
                .collect(Collectors.toList());

        List<Boolean> checkCongruenzeCandidati = schedeVoto.getListSchedeVoto().stream()
                .filter(scheda -> scheda.getBianca().equals(Boolean.FALSE) && scheda.getVotata().equals(Boolean.TRUE))
                .map(schedeVotoDTO -> verificaCandidati(schedeVotoDTO, schedeModello.stream().filter(modello -> modello.getIdScheda().equals(schedeVotoDTO.getChiaveScheda())).findFirst().get()))
                .collect(Collectors.toList());

        List<Boolean> verificaComposta = concatenaListe(checkNumeroSchede, checkUnivocitaSchede, checkNumeroPreferenze, checkCongruenzaIdListe, checkCongruenzeCandidati);

        // Metto in and tutti i check
        return verificaComposta.stream()
                .reduce(true, (corrente, successivo) -> corrente && successivo);
    }

    private Boolean verificaListe(SchedeVotoDTO schedaVoto, SchedaVerificaIdResponse schedaModello) {

        // Verifico che l'id lista sia presente nel modello
        List<Boolean> checkListe = new ArrayList<>();
        checkListe.add(Optional.ofNullable(schedaModello.getMappaListeCandidati().get(schedaVoto.getLista().getIdlista())).isPresent());

        // Metto in and tutti i check
        return checkListe.stream()
                .reduce(true, (corrente, successivo) -> corrente && successivo);
    }

    private Boolean verificaCandidati(SchedeVotoDTO schedaVoto, SchedaVerificaIdResponse schedaModello) {

        // Verifico che l'id candidato sia coerente con l'id della lista
        List<Boolean> checkCandidati = new ArrayList<>(schedaVoto.getLista().getListaCandidati().stream()
                // Recupero prima la lista e sccessivamente provo a recuperare il candidato
                .map(listaCandidati -> Optional.ofNullable(schedaModello.getMappaListeCandidati().get(schedaVoto.getLista().getIdlista()))
                        .orElse(new ArrayList<>()).stream()
                        .anyMatch(candidato -> candidato.equals(listaCandidati.getId())))
                .toList());

        // Verifico che l'id dei candidati siano univoci e che il numero di preferenze sia minore uguale a 2
        Long candidatiVotatiDistinti = schedaVoto.getLista().getListaCandidati().stream()
                .map(CandidatoSchedaDTO::getId)
                .distinct()
                .count();

        Long candidatiVotatiTotali  = schedaVoto.getLista().getListaCandidati().stream()
                .map(CandidatoSchedaDTO::getId)
                .count();

        // Aggiungo le verifiche alla lista
        checkCandidati.add(candidatiVotatiDistinti.equals(candidatiVotatiTotali) && candidatiVotatiTotali <= 2);

        // Verifica della alternanza di genere
        Boolean votoAlternanzaGenere = Boolean.TRUE;

        // Metto in and tutti i check
        return checkCandidati.stream()
                .reduce(true, (corrente, successivo) -> corrente && successivo);
    }

    private static List<Boolean> concatenaListe(List<Boolean>... liste) {
        return Stream.of(liste)
                .flatMap(List::stream)
                .collect(Collectors.toList());
    }

}
