package it.evota.validate.service;

import java.util.Date;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import it.evota.validate.dto.CountAQResultType;
import it.evota.validate.dto.VerifyAQRequestType;
import it.evota.validate.entities.ListaAttrQualSpesiBc;
import it.evota.validate.entities.ListaAttrQualSpesiBc.StatusEnum;
import it.evota.validate.repository.ListaAttrQualSpesiBcRepository;
import it.evota.validate.service.rest.ListaAttrQualBcRest;
import it.evota.validate.util.dto.FindAQRequestType;
import it.evota.validate.util.dto.ListaAttrQualBcDto;
import it.evota.validate.util.exception.EVoteAppException;
import it.evota.validate.util.exception.EVoteAppException.ErrorCode;

@Service
public class BcValidateService {

	@Autowired
	ListaAttrQualSpesiBcRepository attrQualSpesiBcRepo;

	@Autowired
	ListaAttrQualBcRest listaAttrQualBcRest;

	Logger logger = LogManager.getLogger(getClass());

	public synchronized void readKeys() throws Exception {
		logger.info("Keys initialized");
	}

	@Transactional(propagation = Propagation.REQUIRES_NEW)
	public void verify(VerifyAQRequestType req) throws EVoteAppException {
		FindAQRequestType findAQRequestType = new FindAQRequestType(req.getQualifiedAttribute());
		ListaAttrQualBcDto listaAttrQualBcDtoRE = listaAttrQualBcRest.findByAttributoQualificante(findAQRequestType);

		if (listaAttrQualBcDtoRE != null) {
			ListaAttrQualBcDto bc = listaAttrQualBcDtoRE;

			if (!attrQualSpesiBcRepo.existsByTokenvotoAndStatus(bc.getTokenvoto(), StatusEnum.V)) {
				ListaAttrQualSpesiBc record = new ListaAttrQualSpesiBc();
				record.setTokenvoto(bc.getTokenvoto());
				record.setStatus(StatusEnum.V);
				record.setCreatedDate(new Date());
				attrQualSpesiBcRepo.save(record);
			} else if (attrQualSpesiBcRepo.existsByTokenvotoAndStatus(bc.getTokenvoto(), StatusEnum.U)) {
				throw new EVoteAppException("Error token already cosumed", ErrorCode.VERIFY_ERROR_002);
			}
			return;
		} else {
			throw new EVoteAppException("Error on validate token: qualified attribute not found",
					ErrorCode.VERIFY_ERROR_001);
		}
	}

	public CountAQResultType getCounts(Date startDate) throws EVoteAppException {
		return attrQualSpesiBcRepo.countAttributes(startDate);
	}
}
