package it.evota.validate.service.rest;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import it.evota.validate.config.AppConfig;
import it.evota.validate.util.dto.FindAQRequestType;
import it.evota.validate.util.dto.ListaAttrQualBcDto;
import it.evota.validate.util.exception.EVoteAppException;
import it.evota.validate.util.exception.EVoteAppException.ErrorCode;
import jakarta.servlet.http.HttpServletRequest;

@Service
public class ListaAttrQualBcRest {
	@Autowired
	private final AppConfig appConfig;

	Logger logger = LogManager.getLogger(getClass());

	@Autowired
	public ListaAttrQualBcRest(AppConfig appConfig) {
		this.appConfig = appConfig;
	}

	@Value("${server.servlet.context-path}")
	private String contextPath;

	@SuppressWarnings("null")
	public ListaAttrQualBcDto findByAttributoQualificante(FindAQRequestType findAQRequest) throws EVoteAppException {
		String serviceUrl = getBaseUrl() + appConfig.generateRestApiServicePath()
				+ "/evote/qa/findByAttributoQualificante";

		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(MediaType.APPLICATION_JSON);

		HttpEntity<FindAQRequestType> requestEntity = new HttpEntity<>(findAQRequest, headers);
		RestTemplate restTemplate = new RestTemplate();
		ResponseEntity<?> re = restTemplate.exchange(serviceUrl, HttpMethod.POST, requestEntity,
				ListaAttrQualBcDto.class);
		if (re.getStatusCode().equals(HttpStatus.OK)) {
			return (ListaAttrQualBcDto) re.getBody();
		} else if (re.getStatusCode().equals(HttpStatus.NOT_FOUND)) {
			return null;
		} else {
			logger.error("Error calling service: " + re.getStatusCode().value() + " " + re.getBody()==null?"":re.getBody().toString());
			throw new EVoteAppException("Error calling service", ErrorCode.EVOTE_ERROR_REST);
		}
	}

	public String getBaseUrl() {
		if (appConfig.getGenerateRestApiBasePath() != null && !appConfig.getGenerateRestApiBasePath().isBlank()) {
			return appConfig.getGenerateRestApiBasePath();
		} else {
			@SuppressWarnings("null")
			HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes())
					.getRequest();
			String ret = request.getRequestURL().toString().replace(request.getRequestURI(), request.getContextPath());
			ret = ret.replace(contextPath, "");
			ret = ret.replace(appConfig.serverPort(), appConfig.generateRestApiPort());
			return ret;
		}
	}
}
